/***************************************************************************
* Generate an overlay file for TOP50 topographic maps from SatCal generated lists                *
*                                                                          *
*                                                                          *
* Author: Gerhard HOLTKAMP                      28-FEB-2006
*  Open Source software. 
***************************************************************************/
/*------------ include files and definitions -----------------------------*/
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <iostream>
#include <cmath>

using namespace std;

void getnm (char* lin, int m, double& f, bool& succ)
 {
  // convert coordinate text into floating point number 
  int j, n, k;
  char nm[20];
 
  n = m;
  succ = false;
  for (j=m; j>0; j--)
   {
    if (lin[j] == ' ')
     {
      n = j;
      break;
     };
   };
   
   if (((n < (m-1)) && ((m-n) < 15)))
    {
     for (j=0; j<20; j++) nm[j] = '\0';
     k = 0;
     for (j=m; j<80; j++)
      {
       if (lin[j] == ' ')
        {
         k = j;
         break;
        };
      };
     if ((k != 0) && ((k-n) < 18))
      {
       for (j=n; j<k; j++) nm[j-n] = lin[j];
       f = atof(nm);
       succ = true;
      };
    };

  return;
 } 

int main(int argc, char* argv[]) {

  cout << "Generation of Map Overlays for TOP50" << endl << endl;

 // cin >> day >> month >> year >> hour;

  int const mxpt = 7000;
  int k, j, m, n, leof;
  double xpt[mxpt], ypt[mxpt];    

  bool succ;
  char lin1[81];
  FILE* ovlfile;
  FILE* satfile;

  if (argc > 1) satfile = fopen (argv[1], "r");
  else satfile = fopen ("satsave.txt","r");
  if (satfile == NULL)
   {
    cout << "Unable to open input file " << endl << endl;
    return 0;
   };
  if (argc > 2) ovlfile = fopen (argv[2], "w+");
  else ovlfile = fopen ("satovl.ovl","w+");
  if (ovlfile == NULL) 
   {
    cout << "Unable to open output file " << endl << endl;
    return 0;
   };

  if (feof(satfile) == 0) leof = 0;
  else leof = 1;

  k = 0;
  while (!leof)
   {
    if (fgets(lin1, 80, satfile) == NULL) leof = 1;  // get one line
    if (!leof && strlen(lin1) > 30)
     {
      succ = false; 
      if ((lin1[10] == ':') && (lin1[13] == ':')) // line contains coordinates
       {
        for (j=0; j<80; j++)
         {
          m = j;
          if (lin1[j] == '.') break;
          if (lin1[j] == '\0') break;
         };
        if (lin1[m] == '.') getnm(lin1, m, xpt[k], succ);
        if (succ)
         {
          m++;
          for (j=m; j<80; j++)
           {
            succ = false;
            if (lin1[j] == '.')
             {
              getnm(lin1, j, ypt[k], succ);  
              break; 
             };
            if (lin1[j] == '\0') break;
           };
         };
       }; // if lin1[10]
      if (succ) k++;
      if (k > (mxpt-2)) break;
     };  // if !leof
   };  // leof

  if (k <= 0) return 0;
  
  // limit coordinates to the maximum and minimum ones for Germany
  m = 0;
  for (j=0; j<k; j++)
   {
     if ((xpt[j] > 5.8472) && (xpt[j] < 15.05)  && (ypt[j] > 47.2639) && (ypt[j] < 55.0583)) m++;
     else xpt[j] = 0.0;
    };  
  if (m <= 0) return 0;

  // print header
  fprintf(ovlfile,"[Symbol 1]\n");
  fprintf(ovlfile,"Typ=3\n");
  fprintf(ovlfile,"Group=1\n");
  fprintf(ovlfile,"ObjTyp=16\n");
  fprintf(ovlfile,"Col=1\n");
  fprintf(ovlfile,"Zoom=1\n");
  fprintf(ovlfile,"Size=104\n");
  fprintf(ovlfile,"Art=1\n");
  sprintf(lin1, "Punkte=%i\n", m);
  fprintf(ovlfile, lin1);

  // print coordinates
  m = 0;
  for (j=0; j<k; j++)
   {
     if (xpt[j] > 0.0)
      {	 
       fprintf(ovlfile,"XKoord%i=%10.5lf\n", m, xpt[j]);
       fprintf(ovlfile,"YKoord%i=%10.5lf\n", m, ypt[j]);
       m++;
     };  
   };	

  // print footer
  fprintf(ovlfile,"[Overlay]\n");
  fprintf(ovlfile,"Symbols=1\n");

  fclose (satfile);
  fclose (ovlfile);

  cout << "Map generated with " << m << " points. "<< endl << endl;

 }

