/***************************************************************************
* Generate a file for XEphem of sky positions from SatCal generated lists  *
* Special version for minutes time step                                    *
*                                                                          *
* Author: Gerhard HOLTKAMP                      1-NOV-2005
*  Open Source software. 
***************************************************************************/
/*------------ include files and definitions -----------------------------*/
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <iostream>
#include <cmath>

using namespace std;

void getnm (char* lin, int m, double& f, bool& succ)
 {
  int j, n, k;
  char nm[20];
 
  n = m;
  succ = false;
  for (j=m; j>0; j--)
   {
    if (lin[j] == ' ')
     {
      n = j;
      break;
     };
   };
   
   if (((n < (m-1)) && ((m-n) < 15)))
    {
     for (j=0; j<20; j++) nm[j] = '\0';
     k = 0;
     for (j=m; j<80; j++)
      {
       if (lin[j] == ' ')
        {
         k = j;
         break;
        };
      };
     if ((k != 0) && ((k-n) < 18))
      {
       for (j=n; j<k; j++) nm[j-n] = lin[j];
       f = atof(nm);
       succ = true;
      };
    };

  return;
 } 

int main(int argc, char* argv[]) {

  cout << "Generation of sky data for XEphem " << endl << endl;

  int const mxpt = 7000;
  int k, j, m, n, leof, b;
  double xpt[mxpt], ypt[mxpt];    
  int secpt[mxpt];

  bool succ;
  char lin1[81];
  char outlin[50];
  FILE* ovlfile;
  FILE* satfile;

  if (argc > 1) satfile = fopen (argv[1], "r");
  else satfile = fopen ("satsave.txt","r");
  if (satfile == NULL)
   {
    cout << "Unable to open input file " << endl << endl;
    return 0;
   };
  if (argc > 2) ovlfile = fopen (argv[2], "w+");
  else ovlfile = fopen ("satovl.edb","w+");
  if (ovlfile == NULL) 
   {
    cout << "Unable to open output file " << endl << endl;
    return 0;
   };

  if (feof(satfile) == 0) leof = 0;
  else leof = 1;

  k = 0;
  while (!leof)
   {
    if (fgets(lin1, 80, satfile) == NULL) leof = 1;  // get one line
    if (!leof && strlen(lin1) > 50)
     {
      succ = false; 
      if (lin1[10] == ':') // line contains coordinates
       {
        if (lin1[13] == ':') b = 40; 
        else b = 36;  // time is given in minutes
        for (j=b; j<80; j++)
         {
          m = j;
          if (lin1[j] == '.') break;
          if (lin1[j] == '\0') break;
         };
        if (lin1[m] == '.') getnm(lin1, m, xpt[k], succ);
        if (succ)
         {
          m++;
          for (j=m; j<80; j++)
           {
            succ = false;
            if (lin1[j] == '.')
             {
              getnm(lin1, j, ypt[k], succ);  
              break; 
             };
            if (lin1[j] == '\0') break;
           };
         };
       }; // if lin1[10]
      if (succ)
       {
        if (b == 40)
         {
          lin1[0] = lin1[14];
          lin1[1] = lin1[15];
         }
        else
         {
          lin1[0] = lin1[11];
          lin1[1] = lin1[12];
         };
        lin1[2] = '\0';
        secpt[k] = atoi(lin1);
        k++;
       }; 
      if (k > (mxpt-2)) break;
     };  // if !leof
   };  // leof

  if (k <= 0) return 0;

  // print coordinates
  for (j=0; j<k; j++)
   {
    sprintf(lin1, "%8.4lf", xpt[j]); 
    sprintf(outlin, "%2i", secpt[j]);
    if (outlin[0] == ' ') outlin[0] = '0';
    outlin[2] = ',';
    outlin[3] = 'f';
    outlin[4] = '|';
    outlin[5] = 'S';
    outlin[6] = '|';
    outlin[7] = 'K';
    outlin[8] = '9';
    outlin[9] = ',';
    outlin[10] = lin1[1];
    outlin[11] = lin1[2];
    outlin[12] = ':';
    outlin[13] = lin1[4];
    outlin[14] = lin1[5];
    outlin[15] = ':';
    outlin[16] = lin1[6];
    outlin[17] = lin1[7];
    outlin[18] = '.';
    outlin[19] = '0';
    outlin[20] = ',';
    sprintf(lin1, "%8.4lf", ypt[j]); 
    outlin[21] = lin1[0];
    outlin[22] = lin1[1];
    outlin[23] = lin1[2];
    outlin[24] = ':';
    outlin[25] = lin1[4];
    outlin[26] = lin1[5];
    outlin[27] = ':';
    outlin[28] = lin1[6];
    outlin[29] = lin1[7];
    outlin[30] = '.';
    outlin[31] = '0';
    outlin[32] = ',';
    outlin[33] = '6';
    outlin[34] = '.';
    outlin[35] = '0';
    outlin[36] = '\r';
    outlin[37] = '\n';
    outlin[38] = '\0';
    fprintf(ovlfile,outlin);
   };	

  fclose (satfile);
  fclose (ovlfile);

  cout << "Map generated with " << k << " points. "<< endl << endl;

 }

