// Marble Widget for displaying SatCal tracks
// Gerhard HOLTKAMP  JUN 2012
// License: GNU Lesser General Public License Version 2
// Marble Author: Torsten Rahn and others (see Marble Desktop Globe for full credits)

#include <QtGui/QApplication>
 
#include <QtGui/QLayout>
#include <QtGui/QSlider>
#include <QtGui/QLabel>
#include <QtGui/QInputDialog>
#include <QLocale>
#include <QString>
 
#include <marble/MarbleWidget.h>
#include <marble/GeoPainter.h>

#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <iostream>
#include <cmath>

using namespace std;
using namespace Marble;

int const mxpt = 3000;

bool first=true;
bool hflag = false;  // true if heights are to be plotted
double xpt[mxpt], ypt[mxpt];  // array with longitude and latitude
int numcoords;
char sfilename[120];

double Atof (char* buff)
{
 // similar to atof but uses only decimal dots independent of the locale setting
 double res;
 QString s;

 s = buff;

 res = s.toDouble();

 return res;
}

void getnm (char* lin, int m, double& f, bool& succ)
 {
  // convert coordinate text into floating point number 
  int j, n, k;
  char nm[20];
 
  n = m;
  succ = false;
  for (j=m; j>0; j--)
   {
    if (lin[j] == ' ')
     {
      n = j;
      break;
     };
   };
   
   if (((n < (m-1)) && ((m-n) < 15)))
    {
     for (j=0; j<20; j++) nm[j] = '\0';
     k = 0;
     for (j=m; j<80; j++)
      {
       if ((lin[j] == ' ') || (lin[j] == '\n') || (lin[j] == '\r') || (lin[j] == '\0'))
        {
         k = j;
         break;
        };
      };
     if ((k != 0) && ((k-n) < 18))
      {
       for (j=n; j<k; j++) nm[j-n] = lin[j];
       f = Atof(nm);
       succ = true;
      };
    };

  return;
 } 

int readCoordinateLine(FILE * satfle, double & lat, double & lng, double & hgt)
{
  // return 1 if eof, 0 otherwise

  bool succ;
  int leof, j, m, k;
  char lin1[81];
  
  leof = 0;
  lat = 100.0;
  lng = 0;
  hgt = 0;
  succ = true;

  k = 0;
  if (fgets(lin1, 80, satfle) == NULL) leof = 1;  // get one line
  if (!leof && strlen(lin1) > 30)
   {
    succ = false; 
    if ((lin1[10] == ':') && (lin1[13] == ':')) // line contains coordinates
     {
      for (j=0; j<80; j++)
       {
        m = j;
        if (lin1[j] == '.') break;
        if (lin1[j] == '\0') break;
       };
      if (lin1[m] == '.') getnm(lin1, m, lng, succ);  // get longitude
      if (succ)  // get latitude
       {
        m++;
        succ = false;
        for (j=m; j<80; j++)
         {
	  m = j;
          if (lin1[j] == '.')
           {
            getnm(lin1, j, lat, succ);
            break; 
           };
          if (lin1[j] == '\0') break;
         };
       };
      if (hflag && succ)  // height
       {
        m++;
        succ = false;
        for (j=m; j<80; j++)
         {
          if (lin1[j] == '.')
           {
            getnm(lin1, j, hgt, succ);
	    if (succ) hgt *= 1000.0;
	    else hgt = 0.0;
            break; 
           };
          if (lin1[j] == '\0') break;
         };
       };
     } // if lin1[10]
    else
     {
      if ((lin1[0] == '_') && (k>0))  // a new track begins
       {
        lng = 0.0;
        lat = 100.0;
       };	   
     }; 	 
   };
   
   return leof;  
 }

void getSatCoords(int argc, char** argv)
{
  int k, j, m, leof;

  bool succ;
  char lin1[81];
  FILE* satfile;

  if (argc > 1) satfile = fopen (argv[1], "r");
  else satfile = fopen ("satsave.txt","r");
  if (satfile == NULL)
   {
    cout << "Unable to open input file " << endl << endl;
    return;
   };

  if (feof(satfile) == 0) leof = 0;
  else leof = 1;

  k = 0;
  while (!leof)
   {
    if (fgets(lin1, 80, satfile) == NULL) leof = 1;  // get one line
    if (!leof && strlen(lin1) > 30)
     {
      succ = false; 
      if ((lin1[10] == ':') && (lin1[13] == ':')) // line contains coordinates
       {
        for (j=0; j<80; j++)
         {
          m = j;
          if (lin1[j] == '.') break;
          if (lin1[j] == '\0') break;
         };
        if (lin1[m] == '.') getnm(lin1, m, xpt[k], succ);
        if (succ)
         {
          m++;
          for (j=m; j<80; j++)
           {
            succ = false;
            if (lin1[j] == '.')
             {
              getnm(lin1, j, ypt[k], succ);
              numcoords = k;                             	      
              break; 
             };
            if (lin1[j] == '\0') break;
           };
         };
       } // if lin1[10]
      else
       {
	if ((lin1[0] == '_') && (k>0))  // a new track begins
	 {
	  xpt[k] = 0.0;
	  ypt[k] = 100.0;
	  succ = true;
	 };	   
       }; 	 
      if (succ) k++;
      if (k > (mxpt-2)) break;
     };  // if !leof
   };  // leof

  fclose(satfile);
  
  return;
}  
  
class MyMarbleWidget : public MarbleWidget
{
public:
    virtual void customPaint(GeoPainter* painter);
};
 
void MyMarbleWidget::customPaint(GeoPainter* painter)
{
  int j;
  double hgt0, hgt1;
  FILE* satflx;

  satflx = fopen (sfilename,"r");
  if (satflx == NULL) return;
  
  painter->setPen(Qt::red);

  if (numcoords < 2) return;

  j = readCoordinateLine(satflx, ypt[0], xpt[0], hgt0);
  j = readCoordinateLine(satflx, ypt[1], xpt[1], hgt1);
  
  while (j == 0)
   {
    if ((ypt[0] < 100.0) && (ypt[1] < 100.0))
    {
      painter->drawLine( 
       GeoDataCoordinates(xpt[0], ypt[0], hgt0, GeoDataCoordinates::Degree),
       GeoDataCoordinates(xpt[1], ypt[1], hgt1, GeoDataCoordinates::Degree));
      painter->drawPoint( 
       GeoDataCoordinates(xpt[0], ypt[0], hgt0, GeoDataCoordinates::Degree));
    }
    xpt[0] = xpt[1];
    ypt[0] = ypt[1];
    hgt0 = hgt1;
    
    j = readCoordinateLine(satflx, ypt[1], xpt[1], hgt1);
   }
   
  fclose(satflx); 

}   

int main(int argc, char** argv)
{
    int hc;
    bool ok;
    double xhc, yhc;
    
    QApplication app(argc,argv);
    QWidget *window = new QWidget;
  
    QLocale::setDefault(QLocale::English);
 
    // Create a Marble QWidget without a parent
    MarbleWidget *mapWidget = new MyMarbleWidget();

    if (argc > 1) strcpy(sfilename,argv[1]);
    else strcpy (sfilename,"satsave.txt");
    
    if (argc > 2) hflag = true;
    
    // Load the Plain map
    hc = 0;
    hc = QInputDialog::getInt(0, "MapTheme",
         "Set Map Theme\n 0: bluemarble \n 1: openstreetmap \n 2: clouds", 
               0, 0, 2, 1,&ok); 
			      
    if (hc == 1) mapWidget->setMapThemeId("earth/openstreetmap/openstreetmap.dgml");
    else mapWidget->setMapThemeId("earth/bluemarble/bluemarble.dgml");
  
    // Hide the FloatItems: OverviewMap, ScaleBar and Compass
    mapWidget->setShowOverviewMap(false);
    mapWidget->setShowScaleBar(false);
    mapWidget->setShowCompass(false);
    mapWidget->setShowCrosshairs(false);
  
    if (hc == 2) mapWidget->setShowClouds( true );

    // Create a horizontal zoom slider and set the default zoom
    QSlider * zoomSlider = new QSlider(Qt::Horizontal);
    zoomSlider->setMinimum( 1000 );
    zoomSlider->setMaximum( 3100 );
 
    mapWidget->zoomView( zoomSlider->value() );
 
    // Create a label to show the geodetic position
    QLabel * positionLabel = new QLabel();
    positionLabel->setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Fixed );
 
    // Add all widgets to the vertical layout.
    QVBoxLayout *layout = new QVBoxLayout;
    layout->addWidget(mapWidget);
    layout->addWidget(zoomSlider);
    layout->addWidget(positionLabel);
 
    // Center the map onto a given position
    
    getSatCoords(argc, argv);
    if (numcoords > 2)
     {
      hc = numcoords / 2;
      if (ypt[hc] > 99.0) hc = hc + 1;
      xhc = xpt[hc];
      yhc = ypt[hc];
     }
    else  
     {
      xhc = 0.0;
      yhc = 0.0;
     };       
      
    GeoDataCoordinates home(xhc, yhc, 0.0, GeoDataCoordinates::Degree);
    mapWidget->centerOn(home);
 
    // Connect the map widget to the position label.
    QObject::connect( mapWidget, SIGNAL( mouseMoveGeoPosition( QString ) ),
                      positionLabel, SLOT( setText( QString ) ) );
 
    // Connect the zoom slider to the map widget and vice versa.
    QObject::connect( zoomSlider, SIGNAL( valueChanged(int) ),
                      mapWidget, SLOT( zoomView(int) ) );
    QObject::connect( mapWidget, SIGNAL( zoomChanged(int) ),
                      zoomSlider, SLOT( setValue(int) ) );

    window->setLayout(layout);
    window->resize( 700, 500 );
 
    window->show();

    zoomSlider->setValue(1200);
    
    return app.exec();
}

