/***************************************************************************
                          satcal.h  -  description

 Open Source software. See header of main.cpp for details.                                                  
 ***************************************************************************/

#ifndef SATCAL_H
#define SATCAL_H

// include files for QT
#include <QMainWindow>

class QAction;
class QMenu;
class QHttp;

#include <QApplication>
#include <QAction>
#include <QMenuBar>
#include <QToolButton>
#include <QStatusBar>
#include <QString>
#include <QImage>
#include <QPixmap>
#include <QMessageBox>
#include <QFileDialog>
#include <QPainter>
#include <QTimer>
#include <QKeyEvent>
#include <QLabel>
#include <QScrollArea>

// application specific includes
#include "satcalview.h"

class SatcalApp : public QMainWindow
{
  Q_OBJECT
  
  public:
    SatcalApp();
    ~SatcalApp();
    void initActions();
    void initMenuBar();
    void initToolBar();
    void initStatusBar();
    void initView();

    virtual void keyPressEvent(QKeyEvent *e);

  public slots:

    void slotFileOpen();
    void slotFileUpdate();
    void slotFileSave();
    void slotFileSaveAs();
    void slotFileTLESave();
    void slotFileTLESaveAs();
    void slotFilePicture();
    void slotFileQuit();

    void slotInputSelectSat();
    void slotInputRemoveSat();
    void slotInputNextSat();
    void slotInputDate();
    void slotInputLocation();
    void slotInputIers();

    void slotCalcAOS();
    void slotCalcAllSats();
    void slotCalcOrbitTrack();
    void slotCalcIridium();
    void slotCalcMatch();

    void slotManvEdit();
    void slotManvDeltaV();
    void slotManvLaunch();
    void slotManvRendz();
    void slotManvOrbDet();
    
    void slotPlotRT();
    void slotPlotDelta();
    void slotPlotCurrent();
    void slotPlotNext();
    void slotPlotLast();
    void slotPlotForward();
    void slotPlotBackward();

    void slotViewRefresh();
    void slotViewSwitch();
    void slotViewMap();
    void slotViewText();
    void slotViewNames(bool on);
    void slotViewPicture(bool on);
    void slotViewVisibility();
    void slotViewStep();
    void slotViewMagnitude();
    void slotViewFontSize();
    void slotViewLeft(bool on);
    void slotViewToolBar(bool toggle);
    void slotViewStatusBar(bool toggle);
    void size25();
    void size50();
    void normalSize();

    void slotSettSysConf();

    void slotHelpAbout();
    void slotQuickHelp();

    void slotTmCheck();
    void slotStopTime();
    void httpDone(bool error);

  private:

    SatcalView *view;
    QLabel *imageLabel;
    QScrollArea *sv;
 
    QMenu *fileMenu;
    QMenu *inputMenu;
    QMenu *calcMenu;
    QMenu *manvMenu;
    QMenu *plotMenu;
    QMenu *viewMenu;
    QMenu *settingsMenu;
    QMenu *helpMenu;
    QToolBar *fileToolbar;

    QAction *fileOpen;
    QAction *fileUpdate;
    QAction *fileSave;
    QAction *fileSaveAs;
    QAction *fileTLESave;
    QAction *fileTLESaveAs;
    QAction *filePicture;
    QAction *fileQuit;

    QAction *inputSelectSat;
    QAction *inputRemoveSat;
    QAction *inputNextSat;
    QAction *inputDate;
    QAction *inputLocation;
    QAction *inputIers;

    QAction *calcAOS;
    QAction *calcAllSats;
    QAction *calcOrbitTrack;
    QAction *calcIridium;
    QAction *calcMatch;

    QAction *manvEdit;
    QAction *manvDeltaV;
    QAction *manvLaunch;
    QAction *manvRendz;
    QAction *manvOrbDet;

    QAction *plotRT;
    QAction *plotDelta;
    QAction *plotCurrent;
    QAction *plotNext;
    QAction *plotLast;
    QAction *plotForward;
    QAction *plotBackward;

    QAction *viewRefresh;
    QAction *viewSwitch;
    QAction *viewMap;
    QAction *viewText;
    QAction *viewNames;
    QAction *viewPicture;
    QAction *viewVisibility;
    QAction *viewStep;
    QAction *viewMagnitude;
    QAction *viewFontSize;
    QAction *viewLeft;
    QAction *viewToolBar;
    QAction *viewStatusBar;
    QAction *size25Act;
    QAction *size50Act;
    QAction *normalSizeAct;

    QAction *settSysConf;
 
    QAction *helpAboutApp;
    QAction *aboutQtAct;
    QAction *quickHelp;
    QAction *whts;

    QTimer *timer;
    QFile wwwfile;
    QFile wwwsvf;
    QHttp *http;

    int timon;  // 0: no timer; 1: RT; 2: Delta-Plot
    int wwwon; // 1: download action pending; 0 otherwise
    bool wwwsave; // true if wwwsave file written

};
#endif 


