// Open Source software. See header of main.cpp for copyright details.
#if !defined(__satccom_h)
#define __satccom_h

//using namespace std;

#include "attlib.h"

static int const GBL_MAXLIN = 160;  // maximum number of lines in Window Buffer
static int const GBL_MAXCPL = 77;  // maximum number of characters per line in buffer
static int const GBL_MXSAT = 10;    // maximum number of selected satellites
static int const GBL_MAXPASS = 120;// maximum number of passes calculated

struct MapBuf                 // Buffer for World Map parameters
 {
  int emagrid;  // 0 if no grid, 1 if 30 deg grid, 2 if 10 deg grid
  int emagrey;  // 0 if map background white, 1 if grey, 2 if black
  };

struct OrbitEl                // Buffer for orbit elements
 {
  int norad[GBL_MXSAT+2];        // NORAD #
  char ldsgn[GBL_MXSAT+2][12];   // International Launch Designator
  char sname[GBL_MXSAT+2][21];   // Satellite name to search for
  char fsname[GBL_MXSAT+2][21];   // matching satellite name found
  int aflg[GBL_MXSAT+2]; // 1 if SPG4 model, 0 for simple model
  int numflg[GBL_MXSAT+2]; // 1 if numeric calc, 0 for analytic model
  int lflg;  // 1 if only orbits below 250 km are considered
  int nflg;  // 1 if only S/C with < 100 revolutions
  int nrflg; // 1 search by NORAD number; 2 search by Int.Launch#
  int paflg; // 1 if input/edit via perigee and apogee height
  double mjdep[GBL_MXSAT+2]; // MJD of epoch of elements
  double a[GBL_MXSAT+2];   // semi-major axis
  double ra[GBL_MXSAT+2];  // Right Ascension of Ascending Node in degrees
  double aper[GBL_MXSAT+2]; // Argument of Perigee in degrees
  double inc[GBL_MXSAT+2];  // inclination in degrees
  double ecc[GBL_MXSAT+2];  // eccentricity
  double m0[GBL_MXSAT+2];  // Mean Anomaly at Epoch in degrees
  double n0[GBL_MXSAT+2];  // Mean Motion in rev./day
  double adot[GBL_MXSAT+2]; // change in a
  double ndot[GBL_MXSAT+2]; // change in mean motion (in rev/day/day)
  double n6dot[GBL_MXSAT+2]; // second time derivative of Mean Motion
  double bstar[GBL_MXSAT+2]; // BSTAR drag term for GP4
  double x0[GBL_MXSAT+2];  // state vector at epoch in m and m/sec
  double y0[GBL_MXSAT+2];
  double z0[GBL_MXSAT+2];
  double vx0[GBL_MXSAT+2];
  double vy0[GBL_MXSAT+2];
  double vz0[GBL_MXSAT+2];
  double mjdcur[GBL_MXSAT+2];  // MJD of current state vector
  double x[GBL_MXSAT+2];  // current state vector in m and m/sec
  double y[GBL_MXSAT+2];
  double z[GBL_MXSAT+2];
  double vx[GBL_MXSAT+2];
  double vy[GBL_MXSAT+2];
  double vz[GBL_MXSAT+2];
  double F107[GBL_MXSAT+2];  // Solar Flux F10.7 index
  double FB107[GBL_MXSAT+2]; // 90 day Average F10.7 index
  double Kp[GBL_MXSAT+2];    // geomagnetic index
  double Cd[GBL_MXSAT+2];    // Drag Coefficient
  double Cr[GBL_MXSAT+2];    // Radiation Pressure Coefficient
  double Am[GBL_MXSAT+2];    // Area to mass ration (m^2/kg)
  double deltv[GBL_MXSAT+2]; // delta-v in m/s or continuous thrust in m/s^2.
  double dyaw[GBL_MXSAT+2];  // yaw angle of delta-v (in deg)
  double dpitch[GBL_MXSAT+2];// pitch angle of delat-v (in deg)
  bool ionflg[GBL_MXSAT+2];  // true if continuous thrust
  bool ionsun[GBL_MXSAT+2];  // true if thrust only during sunshine
  bool updepoch[GBL_MXSAT+2];// true if epoch is updated after each step
 };

struct WinpData   // Input from and data exchange with Main Window
 {
  int day;           // date
  int month;
  int year;
  int hour;          // time
  int minute;
  int second;
  int dspyear;       // year currently displayed
  int hdyear;        // year currently in header
  int tzauto;			// 1 if timezone automatic by system, 0 if manual
  double lastmjd;    // last MJD time (UT)
  double mrefmjd;		// MJD time (UT) of last Sun/Moon map refresh
  double tzone;      // timezone in hours
  double del_tdut;   // TT - UTC in sec
  double duration;   // duration of simulation in days
  double rt_offset;  // offset in hours of the R/T clock
  double tstep;      // time step in days
  double tstpsec;    // second part of time step
  int tstpmin;	     // minute part of time step
  int dfsz;          // font size for display
  int mfsz;          // font size for map
  bool fbold;        // true if bold font for map
  bool leftm;        // true if left mouse button enabled for details
  int svfok;         // 1 = file can be written to, 0 = do not write on file
  int svfirst;       // 1 = first call to save file, 0 otherwise
  int svtlefst;		// 1 = first call to save tle's, 0 otherwise
  int invalflg;      // 1 = if special invalidation necessary
  int graphflg;      // 1 = graphic mode active, 0 = text mode, -1 sat pass list
  int lstgraph;		// 0 = last graph output was normal map; 1 = sun/moon map
  int mpnight;			// 1 if day/night map, 0 normal map
  int skymap;			// 0 = no sky, 1 = sky map
  int skyprj;			// 0 = zenith, 1, 2, 3, 4, 5 = N, E, S, W, Azimuth
  int mapprj;     // 0 = world map, 1 = North stereogr., 2 = South st., 3 = physical
  int mapwidth;		// overall width of the screen
  int mapheight;		// overall height of the screen
  bool physmap;   // true if physical map exists
  bool satsky;    // true if sky map from position of prime satellite
  double skazimuth;	// central azimuth (deg) for sky map
  double centmer;   // central meridian of the standard world map in deg.
  double geolat;     // geographic coordinates of observer (angles in degrees)
  double geolong;
  double geoheight;  // in meters
  double lnchlat;		// geographic coordinates of launch site (in degrees)
  double lnchlng;
  double tmeco;      // time of Main Engine Cutoff in minutes
  double lnchazm;    // launch azimuth in deg
  double rendtim;    // time until rendezvous (days)
  int lnchflg;       // 1 if launch option active, 0 otherwise
  int lsouthflg;     // 1 if launch is to the south
  int lsitex;        // index of the launch site
  int rendzs;        // -1 if no rendezvous launch, otherwise sat ix
  int rendzt;        // sat ix of target for rendezvous launch
  int occflg;        // 1: Sun, 2: Moon transit, 3: RA/Dec Occultation, 0: no
  bool cylflg;       // true if check for cylidrical flares
  double yrepoch;    // year of epoch of RA/Dec. 0 if True of Date
  double occra;      // Right Ascension (in HH.MMSS) of occultation target
  double occdec;     // Declination (in DDD.MMSS) of occultation target
  double cylra;      // R.A or Yaw direction of cylinder axis for cylindrical flare
  double cyldec;     // Declination or Pitch of cylinder axis for cylindrical flare
  double az1;        // azimuth 1 for orbit determination
  double el1;        // elevation 1 for orbit determination
  double az2;        // azimuth 2 for orbit determination
  double el2;        // elevation 2 for orbit determination
  double deltm;      // delta time for orbit determination in seconds
  int azra1flg;      // = 1 if obs 1 in RA/DEC (orbit determination)
  int azra2flg;      // = 1 if obs 2 in RA/DEC (orbit determination)
  bool odtmatch;     // true if satellite is to be matched
  double sdaz1;      // azimuth for sat match
  double sdel1;      // elevation for sat match
  double sdmxang;    // max angular distance for sat match
  double sddirt;     // direction angle for sat match
  double sddeltm;    // delta time for sat match in minutes
  int sdra1flg;      // = 1 if obs in RA/DEC (sat match)
  int sddrctflg;     // = 1 if direction considered (sat match)
  int sdfinish;      // = 1 if normalfinish of the sat match dialog
  bool sdmvis;       // true if only large visible object to be considered
  bool pastro;       // true if astro picture active
  int pnstars;       // number of reference stars - 1. 0 if no astro picture
  int picsize;       // 0 if normal, 2 if 50%, 3 if 25%
  double rstarra[5];  // R.A. of reference stars
  double rstardec[5]; // the last two entries are marked postions
  int pstarx[3];  // plate position of reference stars
  int pstary[3];
  double pltcx[3]; // plate constants for X and Y
  double pltcy[3];
  char idsgn[9];     // International Launch Designator in tle format
  char ldsgn[12];    // Int. Launch Designator in standard format
  char geoname[31];
  char svfname[105]; 	// name of save file
  char tlename[105]; 	// name of tle file
  char tlesave[105]; 	// name of file to save tle's
  char wwwname[105];  // URL of online download file
  int dmsflg;     // 0 = decimal degrees, 1 = degrees, minutes, seconds
  int filsflg;  //  1, 2 = if file save active  ?
  int timer_on;  // 1 if R/T event timer on, 0 otherwise
  int timer_del; // 1 if timer used for delta-plot
  int relsat;   // 1 if relative satellite positions displayed, 0 normal
  int satidx;   // satellite index
  int sidxcn;   // satellite index count (number of satellites selected)
  int lastsx;   // last satellite index used by SGP4 routine
  int visflg;   // = 1 if satellite illuminated to be listed as AOS
  int delflg;   // = 1 if satellite was deleted by Delete Satellite
  int namflg;   // = 1 if satellite name to be displayed on map
  int moonflg;  // = 1 if moon visibility to be calculated
  int flarflg;  // = 1 : flare calcs; 0 otherwise
  int flarmod;  // Flare Mode. +1 : prediction, known local attitude;
					 //   +2 : prediction known inertial attitude;
					 //   -1 : determine surface, local att; -2 : det.surf. inert.att
  int flaryst;  // 1 if Local Yaw Steering Mode; 0 otherwise
  int sunmplt;  // 1 if Sun or Moon transit plot; 0 normally
  int ra4dflg;  // = 1 if R.A./Dec in display with 4 digits, 0 if 2 digits
  double minsatel;  // minimum satellite elevation (deg) for AOS
  double minsunel;  // maximum sun elevation (deg) for AOS
  double maglng; // characteristic length for mag calculation. 0 = no mag calc.
  double frequ;  // 0 if Rate to be displayed. Frequency in MHz for Doppler
  double tpass[GBL_MAXPASS]; // (MJD) times of beginning of passes
  double tpole;  // (MJD) time for which sat-track pole to be calculated
  double iridmag; // minimum mag for Iridium Flares
  double iridkm; // maximum distance in km for transit or max flare site
  double flrang; // flare angle in deg.
  double flrmagx; // maximum flare magnitude
  bool flrmagf; // true if flare magnitude enabled
  int iridflg;  // Iridium Flag 0: normal; 1: Transit; 2: Flare; 3: Iridium Flr.
  int iridops;  // 1 if only operational Iridium Satellites; 0 otherwise
  bool iridsol;  // true if Iridium solar panel flares to be considered
  bool iridsock; // true if Iridium solar panel flare to be checked
  bool iridmess; // true if message about Iridium file save is included.
  int wgs84;     // 1 if wgs84 constants, 0 if wgs72 constants for SGP4
  bool usesdp4;  // true if SDP4 model is to be used for long period orbits
  bool adjflg;  // true if orbit element adjustment
  double adjtim;  // time (in seconds) for epoch adjustment
  double adjn0;  // adjusted value for the mean motion
  double adjra;  // adjustment in Right Ascension (degrees)
  bool solflar;  // true if solar panel flares to be calculated
  bool eclflg;   // true if Solar Eclipse Check
  int ecltyp;    // 0 no eclipse, 1 partial, 2 total, 3 annular
  double eclang; // apparent angle between Sun and Moon
  double eclmag; // magnitude of eclipse
  double eclpow; // fraction of power (sunlight) during eclipse
  int npass;     // number of passes calculated
  int npsel;     // number of the pass selected for detailed display
  int npoffs;    // offset for the pass selection
  int passact;   // 1 if pass currently active, 0 otherwise
  int lorbrelsat; // value of relsat of last call to AOS
  int lorbnorad; // NORAD# of satellite of last call to AOS
  int RKFmxitr;  // maximum number of iterations for integration
  double lorbmjdep; // epoch of satellite of last call to AOS
  double RKFtol;  // relative tolerance for RKF45 integration
  double dF107;  // default Solar Flux F10.7 index
  double dFB107; // default 90 day Average F10.7 index
  double dKp;    // default geomagnetic index
  double dCd;    // default Drag Coefficient
  double dCr;    // default Radiation Pressure Coefficient
  double dAm;    // default Area to mass ratio (m^2/kg)
  double IERSxp; // xp Polar Motion
  double IERSyp;  // yp Polar Motion
  double Ut1Utc; // difference UT1 - UTC in sec
  int UtcTai;    // difference UTC - TAI in sec
  bool IERSflg;  // true if IERS parameters to be used, false if automatic
  int tmje; // Epoch. 0: TOD, 1: Mean of Date, 2: J2000, 3: Earth fixed
  bool moongrav; // true if Moon's gravity included in numeric integration
  bool sungrav; // true if Sun's gravity included in numeric integration
  bool refract; // true if elevation to be corrected for refraction
  bool atmxtin; // true if atmospheric extinction considered in Iridium mag
  double lndist; // distance to Moon in km if numeric
  double liftdrag; // ratio lift to drag for reentry orbit

  MapBuf mapbuf;
  OrbitEl orbelm;
 };

struct WoutData                // Main Window output text buffer
 {
  int curlines;  // currently active lines
  char wbuf[GBL_MAXLIN][GBL_MAXCPL];  // output buffer
  };

struct SatPosData             // output of satellite positional data
 {
  double lat[GBL_MXSAT];        // sub-satellite latiude in degrees
  double lng[GBL_MXSAT];        // sub-satellite longitude in degrees
  double height[GBL_MXSAT];     // height of satellite in km
  double RA[GBL_MXSAT];         // Right Ascension (in hours.min)
  double Dec[GBL_MXSAT];        // Declination  (in degress.min)
  double azim[GBL_MXSAT];       // Azimuth in degrees
  double elev[GBL_MXSAT];       // Elevation in degrees
  double range[GBL_MXSAT];      // Range in km
  double sunang[GBL_MXSAT];     // sun elevation as viewed from satellite
  double moonang[GBL_MXSAT];    // moon elevation as viewed from satellite
  double rrate[GBL_MXSAT];      // range rate in km/s;
  double occlat[GBL_MXSAT];     // occultation latiude in degrees
  double occlng[GBL_MXSAT];     // occultation longitude in degrees
  double beta[GBL_MXSAT];       // Beta angle of orbit plane
  double sunazm[GBL_MXSAT];     // sun azimuth as viewed from satellite
  double obssun;     // sun elevation at observer's site in degrees
  double osunaz;     // sun azimuth at observer's site in radians
  double omoonel;    // moon elevation at observer's site in radians
  double omoonaz;    // moon azimuth at observer's site in radians
  double omoonrd;	   // app. radius of moon at observer in degrees
  double moonphs;    // cos of phase angle of moon
  double sunlat;     // latitude of sub-solar point
  double sunlng;     // longitude of sub-solar point
  double ssun1;   // sun angle of first satellite (in relative satellite mode)
  double ssun2;   // sun angle of second satellite (in relative satellite mode)
  double mag;        // magnitude of satellite
  double satvlat[GBL_MXSAT][64]; // latitude of satellite visibility circle (in radians)
  double satvlng[GBL_MXSAT][64]; // longitude of satellite visibility circle
  int nsatvp;        // number of points in satellite visibility circle
  Vec3 r2;         // position of second satellite in relative mode
  Vec3 v2;         // velocity of second satellite in relative mode
 };

void aospass(WinpData* indata, WoutData* outdata);  // calculate passes
double DegFDms (double h);  // convert from Deg.fff into Deg.MMSSS
double DmsDegF (double h);  // convert from Deg.MMSSS into Deg.fff
void swelcm(WinpData* indata, WoutData* outdata);  // welcoming message
int getcfg (WinpData* indata);  // get cfg from file
void putcfg (WinpData* indata); // write configuration file
double ElvRefr(WinpData* indata, double el);  // correct for refraction
double SetTtUt (WinpData* indata, int yy);  // set difference TT - UTC
void DefDate (int &dd, int &mm, int &yr);  // Get Default Date
void DefTime (int &hh, int &mm, int &ss);  // Get Default Time

void setNewMjd(WinpData* indata);  // reset MJD to new time input
double RT_MJD (WinpData* indata);  // Get Real Time MJD
void GetMonth (int mm, char* mchr);
void convldsn (WinpData* indata, bool tlsc);  // convert Int.Launch#
int tleread(WinpData* indata, FILE* satfle, char* otxt, int morflg);
Mat3 GetRVMat (Vec3& r1, Vec3& v1, double yaw, double pitch, int inpl);
void GetSatPos (double t, WinpData* indata, SatPosData* satpos,
    								int sx, int rngflg);
void EpochState (WinpData* indata, int sx, bool nw); // get state vector at epoch            
void Sgp4 (WinpData* indata, double dt, int sx, Vec3& r1, Vec3& v1);
int cksatok(WinpData* indata, SatPosData* satpos, int sx);
int nextpass(WinpData* indata, int sx, double t, double ts, double *ti, double *el, double *az);
int lastpass(WinpData* indata, int sx, double t, double ts, double *ti, double *el, double *az);
void SkyAzim (WinpData* indata, double t); // get azimuth for sky map
int GetPassDTxt(WinpData* indata, double& t, char* jtxt, int flsav); // pass dt.
void GetPoleTxt(WinpData* indata, char* jtxt);  // get text for pole of pass
int saveOrbitTrack(WinpData* indata);  // save the orbit track
void LocalInf (WinpData* indata, char* otxt);  // get text for local info
int AllSatAos(WinpData* indata);  // get passes from all satellites
void ShiftElm(WinpData* indata, int sx1, int sx2);  // copy sat from sx2 to sx1;
void Iriddspy (WinpData* indata, WoutData* outdata); // display Iridium Flares
int Nxt4Pass(WinpData* indata);   // get next (4) satellites to pass
double getn0 (double a, double ecc, double inc);
double getsax (int aflg, double n0, double ecc, double inc);
int SaveTle (WinpData* indata);  // save TLE data on file
int ExDeltaV (WinpData* indata, double delv, double yaw, double pitch);
void LaunchOrb (WinpData* indata);  // calculate launch orbit
void OrbitDet (WinpData* indata);   // orbit determination
int SatMatch(WinpData* indata, WoutData* outdata);  // match satellites
void prpang (char* ntx, int ra4flg, int dmsflg, double lat);
void rv2el(Vec3& r1, Vec3& v1, double& xincl, double& xnodeo,
double& eo, double& omegao, double& xmo, double& xno );  // get SGP4 elements

#endif         // __satccom_h sentry.


