/********************************************************************************
********************************************************************************/

#include <QtGui>

#include "stdeltadialog.h"

StDeltaDialog::StDeltaDialog()
{
        setObjectName(QString::fromUtf8("StDeltaDialog"));
        resize(340, 210);
        deltavEdit = new QLineEdit(this);
        deltavEdit->setObjectName(QString::fromUtf8("deltavEdit"));
        deltavEdit->setGeometry(QRect(20, 30, 121, 24));
        yawEdit = new QLineEdit(this);
        yawEdit->setObjectName(QString::fromUtf8("yawEdit"));
        yawEdit->setGeometry(QRect(20, 90, 111, 24));
        pitchEdit = new QLineEdit(this);
        pitchEdit->setObjectName(QString::fromUtf8("pitchEdit"));
        pitchEdit->setGeometry(QRect(20, 150, 111, 24));
        textLabel1 = new QLabel(this);
        textLabel1->setObjectName(QString::fromUtf8("textLabel1"));
        textLabel1->setGeometry(QRect(21, 11, 141, 21));
        textLabel1->setWordWrap(false);
        textLabel2 = new QLabel(this);
        textLabel2->setObjectName(QString::fromUtf8("textLabel2"));
        textLabel2->setGeometry(QRect(20, 70, 111, 21));
        textLabel2->setFrameShape(QFrame::NoFrame);
        textLabel2->setFrameShadow(QFrame::Plain);
        textLabel2->setWordWrap(false);
        ionBox = new QCheckBox(this);
        ionBox->setObjectName(QString::fromUtf8("ionBox"));
        ionBox->setGeometry(QRect(160, 30, 161, 21));
        sunBox = new QCheckBox(this);
        sunBox->setObjectName(QString::fromUtf8("sunBox"));
        sunBox->setGeometry(QRect(160, 60, 161, 21));
        textLabel3 = new QLabel(this);
        textLabel3->setObjectName(QString::fromUtf8("textLabel3"));
        textLabel3->setGeometry(QRect(20, 130, 111, 21));
        textLabel3->setWordWrap(false);
        epuBox = new QCheckBox(this);
        epuBox->setObjectName(QString::fromUtf8("epuBox"));
        epuBox->setGeometry(QRect(160, 90, 161, 21));
        executeButton = new QPushButton(this);
        executeButton->setObjectName(QString::fromUtf8("executeButton"));
        executeButton->setGeometry(QRect(240, 130, 81, 29));
        executeButton->setDefault(true);
        cancelButton = new QPushButton(this);
        cancelButton->setObjectName(QString::fromUtf8("cancelButton"));
        cancelButton->setGeometry(QRect(240, 170, 81, 29));
#ifndef QT_NO_SHORTCUT
        textLabel1->setBuddy(deltavEdit);
        textLabel2->setBuddy(yawEdit);
        textLabel3->setBuddy(pitchEdit);
#endif // QT_NO_SHORTCUT
        QWidget::setTabOrder(deltavEdit, yawEdit);
        QWidget::setTabOrder(yawEdit, pitchEdit);
        QWidget::setTabOrder(pitchEdit, ionBox);
        QWidget::setTabOrder(ionBox, sunBox);
        QWidget::setTabOrder(sunBox, epuBox);
        QWidget::setTabOrder(epuBox, executeButton);
        QWidget::setTabOrder(executeButton, cancelButton);

        connect(executeButton, SIGNAL(clicked()), this, SLOT(accept()));
        connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));
        connect(ionBox, SIGNAL(toggled(bool)), this, SLOT(ionSlot()));

        setWindowTitle(QApplication::translate("StDeltaDialog", "Delta-V Maneuver", 0, QApplication::UnicodeUTF8));
#ifndef QT_NO_TOOLTIP
        deltavEdit->setToolTip(QApplication::translate("StDeltaDialog", "Velocity change", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
#ifndef QT_NO_WHATSTHIS
        deltavEdit->setWhatsThis(QApplication::translate("StDeltaDialog", "Note that the velocity change is in m/sec.", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_WHATSTHIS
#ifndef QT_NO_TOOLTIP
        yawEdit->setToolTip(QApplication::translate("StDeltaDialog", "Yaw direction of the velocity change.", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
#ifndef QT_NO_WHATSTHIS
        yawEdit->setWhatsThis(QApplication::translate("StDeltaDialog", "To point to the delta-v direction first yaw (positive if right) then pitch (positive if up).", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_WHATSTHIS
#ifndef QT_NO_TOOLTIP
        pitchEdit->setToolTip(QApplication::translate("StDeltaDialog", "Pitch direction of the velocity change.", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
#ifndef QT_NO_WHATSTHIS
        pitchEdit->setWhatsThis(QApplication::translate("StDeltaDialog", "To point to the delta-v direction first yaw (positive if right) then pitch (positive if up).", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_WHATSTHIS
#ifndef QT_NO_TOOLTIP
        textLabel1->setToolTip(QApplication::translate("StDeltaDialog", "Velocity change", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
#ifndef QT_NO_WHATSTHIS
        textLabel1->setWhatsThis(QApplication::translate("StDeltaDialog", "Delta-V (in m/s) for impulsive maneuver or acceleration (in mm/s^2) for continuous thrust.", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_WHATSTHIS
        textLabel1->setText(QApplication::translate("StDeltaDialog", "&Delta-V (m/sec):", 0, QApplication::UnicodeUTF8));
#ifndef QT_NO_TOOLTIP
        textLabel2->setToolTip(QApplication::translate("StDeltaDialog", "Yaw direction for velocity change", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
#ifndef QT_NO_WHATSTHIS
        textLabel2->setWhatsThis(QApplication::translate("StDeltaDialog", "To point to the delta-v direction first yaw (positive if right) then pitch (positive if up).", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_WHATSTHIS
        textLabel2->setText(QApplication::translate("StDeltaDialog", "&Yaw (deg.):", 0, QApplication::UnicodeUTF8));
#ifndef QT_NO_TOOLTIP
        ionBox->setToolTip(QApplication::translate("StDeltaDialog", "Set continuous thrust.", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
#ifndef QT_NO_WHATSTHIS
        ionBox->setWhatsThis(QApplication::translate("StDeltaDialog", "Continuous acceleration (e.g. solar electric propulsion).", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_WHATSTHIS
        ionBox->setText(QApplication::translate("StDeltaDialog", "Continuous &Thrust ", 0, QApplication::UnicodeUTF8));
        ionBox->setShortcut(QApplication::translate("StDeltaDialog", "Alt+T", 0, QApplication::UnicodeUTF8));
#ifndef QT_NO_TOOLTIP
        sunBox->setToolTip(QApplication::translate("StDeltaDialog", "Continuous thrust only when S/C in sunshine.", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
#ifndef QT_NO_WHATSTHIS
        sunBox->setWhatsThis(QApplication::translate("StDeltaDialog", "This will typically be marked for solar electric propulsion.", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_WHATSTHIS
        sunBox->setText(QApplication::translate("StDeltaDialog", "&Sunshine required", 0, QApplication::UnicodeUTF8));
        sunBox->setShortcut(QApplication::translate("StDeltaDialog", "Alt+S", 0, QApplication::UnicodeUTF8));
#ifndef QT_NO_TOOLTIP
        textLabel3->setToolTip(QApplication::translate("StDeltaDialog", "Pitch direction of the velocity change.", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
#ifndef QT_NO_WHATSTHIS
        textLabel3->setWhatsThis(QApplication::translate("StDeltaDialog", "To point to the delta-v direction first yaw (positive if right) then pitch (positive if up).", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_WHATSTHIS
        textLabel3->setText(QApplication::translate("StDeltaDialog", "&Pitch (deg.):", 0, QApplication::UnicodeUTF8));
#ifndef QT_NO_TOOLTIP
        epuBox->setToolTip(QApplication::translate("StDeltaDialog", "Update Epoch State Vector after each step.", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_TOOLTIP
#ifndef QT_NO_WHATSTHIS
        epuBox->setWhatsThis(QApplication::translate("StDeltaDialog", "If you update the state vector at epoch after each step the changes due to the continuous thrust will be kept. If this box is not marked you can go back to the original state vector before the continuous thrust was performed. ", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_WHATSTHIS
        epuBox->setText(QApplication::translate("StDeltaDialog", "&Update Epoch State", 0, QApplication::UnicodeUTF8));
        epuBox->setShortcut(QApplication::translate("StDeltaDialog", "Alt+U", 0, QApplication::UnicodeUTF8));
        executeButton->setText(QApplication::translate("StDeltaDialog", "&Execute", 0, QApplication::UnicodeUTF8));
        executeButton->setShortcut(QApplication::translate("StDeltaDialog", "Alt+E", 0, QApplication::UnicodeUTF8));
        cancelButton->setText(QApplication::translate("StDeltaDialog", "&Cancel", 0, QApplication::UnicodeUTF8));
        cancelButton->setShortcut(QApplication::translate("StDeltaDialog", "Alt+C", 0, QApplication::UnicodeUTF8));

};

void StDeltaDialog::setParams( double deltv, double yaw, double pitch, bool ionf, bool suns, int nmflg, bool epu )
{
 QString s;
 
 s.setNum(deltv);
 deltavEdit->setText(s);
 s.setNum(yaw);
 yawEdit->setText(s);
 s.setNum(pitch);
 pitchEdit->setText(s);
 ionBox->setChecked(ionf);
 sunBox->setChecked(suns);
 epuBox->setChecked(epu);
 if (nmflg == 0)
  {
   textLabel1->setText("&Delta-V (m/sec):");
   ionBox->setEnabled(false);
   ionBox->hide();
   sunBox->setEnabled(false);
   sunBox->hide();
   epuBox->setEnabled(false);
   epuBox->hide();
  }
 else ionSlot();
}

void StDeltaDialog::getParams( double &deltv, double &yaw, double &pitch, bool &ionf, bool &suns, bool &epu )
{
 QString s;

  s=deltavEdit->text();
  deltv = s.toDouble();
  s=yawEdit->text();
  yaw = s.toDouble();
  s=pitchEdit->text();
  pitch = s.toDouble();
  ionf = ionBox->isChecked();
  suns = sunBox->isChecked();
  epu = epuBox->isChecked();
  if (!ionf) epu = false;
}

void StDeltaDialog::ionSlot()
{
 if (ionBox->isChecked())
  {
   textLabel1->setText("&Acceleration (mm/s^2):");
   sunBox->setEnabled(true);
   sunBox->show();
   epuBox->setEnabled(true);
   epuBox->show();
  }
 else
  {
   textLabel1->setText("&Delta-V (m/sec):");
   sunBox->setEnabled(false);
   sunBox->hide();
   epuBox->setEnabled(false);
   epuBox->hide();
  };    
}
