/***************************************************************************
                          satcalview.cpp  -  description
                             -------------------
    Open Source Code. See main.cpp for details.
    copyright            : (C) 2012 by Holtkamp
 ***************************************************************************/

#include <QtGui>

#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <cmath>

using namespace std;
//using namespace Qt;

#include "stselsatdialog.h"
#include "stobsdialog.h"
#include "stdatedialog.h"
#include "stpassdialog.h"
#include "stlocaldialog.h"
#include "stremsatdialog.h"
#include "stselsatdialog.h"
#include "stswitchdialog.h"
#include "steditorbdialog.h"
#include "ststepdialog.h"
#include "stmapdialog.h"
#include "stvisdialog.h"
#include "stiridiumdialog.h"
#include "stmatchdialog.h"
#include "stdeltadialog.h"
#include "stlaunchdialog.h"
#include "storbdetdialog.h"
#include "stiersdialog.h"
#include "stsysconfdialog.h"
#include "stastdialog.h"

#include "satcalview.h"
#include "satcal.h"
#include "satccom.h"
#include "astrolib.h"
#include "astr2lib.h"
#include "satcnum.h"
#include "nolocal.h"

int epos[GBL_MAXPASS];  // y position of satellite pass j

WoutData *outdata = new WoutData;
WinpData *indata = new WinpData;
SatPosData *satps = new SatPosData;

int fsz = 10;  // display font size
int dsplmd = 0; // display mode. 0: no display; 1: tables; 2: reference text; 3: map
bool irdsf = false;  // true if iridum flare save file active

FILE* satfle;  // needed for the select satellite dialog

SatcalView::SatcalView(QWidget *parent) 
: QScrollArea(parent) 
{
  int wdt, hgt;

  QSize size;
  size.setWidth(720);
  size.setHeight(600);
  resize(size);

  outdata->curlines = 0;
  swelcm(indata, outdata);  // initialize data and display welcoming message
  fsz = indata->dfsz;
  timsuspd = false;

  wdt = GBL_TEXTWIDTH;
  hgt = GBL_TEXTLENGTH;
  buffer = new QPixmap(wdt, hgt);
  buffer->fill();

  wdt = GBL_MAPWIDTH;
  hgt = GBL_MAPHEIGHT + GBL_MAPTEXTHT;
  resize(wdt, hgt);
  showtxt(false);

  buffM = new QPixmap(wdt, hgt);
  buffPWM = new QPixmap(wdt, hgt);

  wdt = GBL_TEXTWIDTH;
  indata->mapwidth = wdt;
  indata->mapheight = hgt;

  buffP = new QPixmap(wdt,hgt);
  hgt = GBL_MAPTEXTHT;
  buffoot = new QPixmap(wdt,hgt);
  buffoot->fill(QColor(0,50,200));

  // load physical world map
  if (buffPWM->load(":/satpearth.png")) indata->physmap = true;

  vRefresh();
  vText();
}


SatcalView::~SatcalView()
{
}

void SatcalView::update()
{
 viewport()->repaint();
}

void SatcalView::keyPressEvent(QKeyEvent *)
{
 viewport()->clearFocus();
}

void SatcalView::mousePressEvent( QMouseEvent *e )
{
 int x, y;
 bool lbutton;

 if((indata->pastro) && (indata->graphflg > 0))
  {
   if (e->button() != Qt::LeftButton) lbutton = false;
   else lbutton = true;
   
   x = e->x();
   y = e->y();
   astroEvent(x, y, lbutton);
   return;
  };

if (!indata->leftm)
 {
  if (e->button() == Qt::LeftButton) return;
 }; 

 x = e->x();
 y = e->y();

 if (indata->graphflg == -1)
  {
   showPassDetails(y);
   this->clearFocus();
   return;
  };
 if ((indata->graphflg == 1) && (indata->lstgraph == 0))
  {
   getMapPos(x, y);
  };

 this->setFocus();

}

void SatcalView::paintEvent( QPaintEvent *  )
{
 QPainter painter(viewport());

 // first get the buffer size right
 if (indata->graphflg > 0)
  {
   if(indata->pastro)
    {
      if (viewport()->size() != buffPPic->size()) 
              resize(buffPPic->width(), buffPPic->height()); 
    }
   else
    {
     if (viewport()->height() == buffer->height()) 
              resize(buffP->width(), buffP->height()); 
    };
  }
 else resize(buffer->width(),buffer->height());

 // now paint
 if (indata->pastro)
   {
    if (indata->graphflg > 0) painter.drawPixmap(0,0, *buffPPic);
    else painter.drawPixmap(0,0, *buffer);
   }
  else
   {
    if (indata->graphflg > 0) painter.drawPixmap(0,0, *buffP);
    else painter.drawPixmap(0,0, *buffer);
   };
 painter.end();
}

void SatcalView::vPicture( const QImage &image )
{
  int picsize;

  if (indata->pnstars == 0)  // initialize the reference star coordinates
   {
    for (picsize=0; picsize<5; picsize++)
     {
      indata->rstarra[picsize] = 0;
      indata->rstardec[picsize] = 0;
     };
   };

  picsize = indata->picsize;
 
  buffPic = new QPixmap(QPixmap::fromImage(image));

  vImage(picsize);
 
}

void SatcalView::vImage(int picsize)
{
  int wdt, hgt;

  indata->pastro = true;
  indata->pnstars = 1;
  indata->picsize = picsize;
  indata->graphflg = 1;
  this->setCursor( Qt::CrossCursor );

  switch (picsize) {
  case 2 :
          wdt = buffPic->width() / 2;
          hgt = buffPic->height() / 2;
          break;

  case 3 :
          wdt = buffPic->width() / 4;
          hgt = buffPic->height() / 4;
          break;

  default:
          wdt = buffPic->width();
          hgt = buffPic->height();
  };

  buffTPic = new QPixmap(wdt, hgt);
  buffPPic = new QPixmap(wdt, hgt);

  QPainter painter;
  painter.begin(buffTPic);
  if (picsize == 0) painter.drawPixmap(0,0,*buffPic);
  else painter.drawPixmap(0,0,buffPic->scaled(wdt,hgt,Qt::KeepAspectRatio,
                                       Qt::SmoothTransformation));
  painter.end();

  painter.begin(buffPPic);
  painter.drawPixmap(0,0,*buffTPic);
  painter.end();

  resize(wdt, hgt);
  update();
}

void SatcalView::astroEvent(int px, int py, bool lbutton)
{
  int nstars;
  int tht, twt, j;
  double ra, dec;
  bool abort;
  char dts[31];
  char ntx[31];
  QString qs;
  QFont f;
  QPoint p0(0,0);
  QPoint poffset;

  indata->relsat = 0;
  poffset = this->mapFromParent(p0);
  twt = poffset.x();

  if (lbutton) ra = 0;  // just a dummy to avoid a warning

  nstars = indata->pnstars - 1;

  if (nstars < 3)  // get reference stars
   {
    StAstDialog *dlg = new StAstDialog();

    ra = DegFDms(indata->rstarra[nstars]*12.0/M_PI);
    dec = DegFDms(indata->rstardec[nstars]*180.0/M_PI);
    dlg->setParams(ra, dec);

    if (dlg->exec())
     {
      dlg->getParams(ra,dec,abort);
     }
    else return;

    if(abort)
     {
      QMessageBox::information(this, "PicAst: ","Astrometry aborted.");
      indata->pnstars = 1;
      for (j=0; j<3; j++)
       {
        indata->rstarra[j] = 0;
        indata->rstardec[j] = 0;
       };
      return;
     };

    indata->rstarra[nstars] = DmsDegF(ra)*M_PI/12.0;
    indata->rstardec[nstars] = DmsDegF(dec)*M_PI/180.0;
    indata->pstarx[nstars] = px;
    indata->pstary[nstars] = py;
    nstars++;

    if (nstars > 2) getPlateConst ();
   }

  else  // get star coordinates
   {
    getRaDec (px, py, ra, dec);
    if (nstars < 5)
     {
      indata->rstarra[nstars] = ra;
      indata->rstardec[nstars] = dec;
      nstars++;
     }
    else
     {
      indata->rstarra[3] = indata->rstarra[4];
      indata->rstardec[3] = indata->rstardec[4];
      indata->rstarra[4] = ra;
      indata->rstardec[4] = dec;
     };

    f.setFamily("Helvetica");
    f.setPointSize(indata->mfsz);
    f.setBold(indata->fbold);

    // display marked position
    QPainter WorldMap;
    WorldMap.begin(buffPPic);
    WorldMap.drawPixmap(0,0,*buffTPic);
    WorldMap.setFont(f);
    QFontMetrics fm = WorldMap.fontMetrics();
    tht = fm.ascent() + 10;
    tht = tht + poffset.y();

    WorldMap.setPen(QColor(255,255,0));

    strcpy (dts, "R.A.:");
    prpang (ntx, 1, -2, ra);
    strcat (dts, ntx);
    qs = dts;
    WorldMap.drawText(twt+180,tht,qs);
    strcpy (dts, "Dec.:");
    prpang (ntx, 1, -1, dec);
    strcat (dts, ntx);
    qs = dts;
    WorldMap.drawText(twt+320,tht,qs);

    WorldMap.end();
   };

  indata->pnstars = nstars + 1; 
  this->setCursor( Qt::CrossCursor );
  update();
}

void SatcalView::lsqfit (double* a, double* r)
{
 // Solve system of 3 linear equations

 const double eps = 1.0e-20;  // to avoid problems close to zero

 int dim2;
 double p;

 dim2 = 4;

 r[2] = a[3];
 p = a[2*dim2]*a[dim2+1] - a[dim2]*a[2*dim2+1];
 if (fabs(p) < eps) p = eps;  // just in case
 r[1] = (a[dim2]*a[2*dim2+3]-a[2*dim2]*a[dim2+3]+(a[dim2]-a[2*dim2])*r[2])/p;
 p = a[dim2];
 if (fabs(p) < eps) p = eps;  // just in case
 r[0] = -(a[dim2+1]*r[1] + r[2] + a[dim2+3]) / p; 

 return;
}
 
void SatcalView::getPlateConst()
{
 int j, dim2;
 double xx, yy;
 double ra0, dec0, ra1, dec1;
 double a[3*4];

 dim2 = 4;

 for (j=0; j<3; j++)
  {
   ra0 = indata->rstarra[0];
   dec0 = indata->rstardec[0];
   ra1 = indata->rstarra[j];
   dec1 = indata->rstardec[j];
   equplt(ra0, dec0, ra1, dec1, xx, yy);
   a[j*dim2] = double(indata->pstarx[j]-indata->pstarx[0]);
   a[j*dim2+1] = double(indata->pstary[j]-indata->pstary[0]);
   a[j*dim2+2] = 1.0;
   a[j*dim2+3] = -xx;
  };
 lsqfit (a, indata->pltcx);

 for (j=0; j<3; j++)
  {
   ra0 = indata->rstarra[0];
   dec0 = indata->rstardec[0];
   ra1 = indata->rstarra[j];
   dec1 = indata->rstardec[j];
   equplt(ra0, dec0, ra1, dec1, xx, yy);
   a[j*dim2] = double(indata->pstarx[j]-indata->pstarx[0]);
   a[j*dim2+1] = double(indata->pstary[j]-indata->pstary[0]);
   a[j*dim2+2] = 1.0;
   a[j*dim2+3] = -yy;
  };
  lsqfit (a, indata->pltcy);

}

void SatcalView::equplt(double ra0, double dec0, double ra, double dec, 
                       double &xx, double &yy)
{
 double d;

 d = cos(dec0)*cos(dec)*cos(ra-ra0) + sin(dec0)*sin(dec);
 if (fabs(d) < 1.0e-60) d = 1.0;  // just in case

 xx = -cos(dec)*sin(ra-ra0) / d;
 yy = -(sin(dec0)*cos(dec)*cos(ra-ra0) - cos(dec0)*sin(dec)) / d;
}

void SatcalView::getRaDec(int px, int py, double &ra, double &dec)
{
 double xx, yy, d;

 d = double(px-indata->pstarx[0]);
 yy = double(py-indata->pstary[0]);
 xx = indata->pltcx[0]*d + indata->pltcx[1]*yy + indata->pltcx[2];
 yy = indata->pltcy[0]*d + indata->pltcy[1]*yy + indata->pltcy[2];

 d = indata->rstardec[0];
 d = cos(d) - yy*sin(d);
 if (fabs(d) < 1e-60) d = 1.0;  // just in case
 ra = indata->rstarra[0] + atan2(-xx, d);
 if(ra < 0) ra = ra + 2.0 * M_PI;
 dec = asin((sin(indata->rstardec[0]) +
             yy*cos(indata->rstardec[0]))/sqrt(1.0+xx*xx+yy*yy));

 ra = DegFDms(ra*12.0/M_PI);
 dec = DegFDms(dec*180.0/M_PI);

}

void SatcalView::setFont(int fnt)
{
 fsz = fnt;  // set display font size
 indata->dfsz = fnt;
}

int SatcalView::getFsz()
{
 // return font

 return fsz;
} 

bool SatcalView::getLeft()
{
 // return left mouse button state

 return indata->leftm;
}

void SatcalView::keyPress(int n)
{
  if (n <= indata->sidxcn) indata->satidx = n - 1;
  if (indata->satidx < 0) indata->satidx = 0; 
}

void SatcalView::scrollDsp(int dx, int dy)
{
 QPoint p0(0,0);
 QPoint poffset;
 int k;
 
 if (dx == 0)
  {
   poffset = this->mapFromParent(p0);
   k = dy - poffset.y();
   if (k > 0) k = 0;
   this->move(0,k);
  }
 else 
  {
   if (dx > 0) vBackward();
   else vForward();
  };  
}

void SatcalView::goHome()
{
  this->move(0,0);
  update();
}

void SatcalView::vQuit()  // save configuration
{
  putcfg(indata);
}

QString SatcalView::getTLEname()  // get the name of the current TLE file
{
 QString s;

 s = QString::null;
 if (strlen(indata->tlename) > 0)
  {
    if (indata->tlename[0] != '*') s = indata->tlename;
  };
    
 return s;
}

void SatcalView::setTLEname(const QString &tlename)  // set the name of the current TLE file
{
  QsToChar(tlename, indata->tlename, 104);
}

QString SatcalView::getSaveName()  // get the name of the Save file
{
 QString s;

 setFileName (indata->svfname);
 s = QString::null;
 if (strlen(indata->svfname) > 0) s = indata->svfname;

 return s;
}

void SatcalView::setSaveName(const QString &tlename)  // set the name of the Save file
{
  QsToChar(tlename, indata->svfname, 104);
  indata->svfirst = 1;
}

QString SatcalView::getTSaveName()  // get the name of the TLE Save file
{
 QString s;

 setFileName (indata->tlesave);
 s = QString::null;
 if (strlen(indata->tlesave) > 0) s = indata->tlesave;

 return s;
}

void SatcalView::setTSaveName(const QString &tlename)  // set the name of TLE Save file
{
  QsToChar(tlename, indata->tlesave, 104);
  indata->svtlefst = 1;
}

void SatcalView::vSelectSat()  // prepare Select Satellite Dialog
{
  QString s1, s2;
  
  satfle = fopen (indata->tlename,"r");
  if (satfle != NULL)
   {
    StSelSatDialog *dlg = new StSelSatDialog();
    dlg->setParams();
    dlg->exec();
    fclose(satfle);
   }
  else
   {
    s1 = "Unable to open TLE file ";
    s2 = indata->tlename;
    s1 += s2; 
    QMessageBox::warning(this, "SatCal", s1, 0,  "Quit", 0, 0, 1 );
   }
}

void SatcalView::setFileName(char *fname)  // complete file name
{
 vsFileName(fname);
}

bool SatcalView::vUpdate(QString &shost, QString &spath, QString &sfnm)
{
 bool ok;
 int j, len, ff;
 QString s;
 char sm[105];

 // append current TLE file name to URL
 s = indata->wwwname;
 QsToChar(s, sm, 104);
 len = strlen(sm);
 if (len > 5)
  {
   for (j=(len-1); j>0; j--)
    {
     if (sm[j] == '/') break;
     else sm[j] = '\0';
    };
   s = sm;
   strcpy(sm, indata->tlename);
   len = strlen(sm);
   sfnm = "";
   if (len > 0)
    {
     for (j=0; j<len; j++)
      {
       if(sm[j] != '/') sfnm += sm[j];
       else sfnm = "";
      };  
    };
   s += sfnm;  
  };  
 
 s = QInputDialog::getText(this, "Online Update", "URL for TLE file to be downloaded:", QLineEdit::Normal, s, &ok);

 if (!ok) return true;               

 QsToChar(s, sm, 104);
 len = strlen(sm);
 if (len < 2) return true;

 strcpy(indata->wwwname, sm);
 shost = "";
 spath = "";
 sfnm = "";
 ff = 0;

 for (j=0; j<len; j++)
  {
   if (sm[j] == '/')
    {
     ff = 1;
     sfnm = ""; 
    };
   if (ff)
    {
     spath += sm[j];
     if (sm[j] != '/') sfnm += sm[j]; 
    }
   else shost += sm[j];
  };

 QsToChar(sfnm, sm, 104);
 for (j=103;j>0;j--) sm[j] = sm[j-1];
 sm[104] = '\0';  // just in case
 sm[0] = '*';
 setFileName(sm);
 sfnm = sm;
 
 return false;
}
                                                                     
void SatcalView::vFileSave()  // save the AOS info on file
{
  QString s1, s2;
  
  if ((indata->sidxcn < 1) && (indata->iridflg != 3))
   {
     QMessageBox::information(this, "SatCal",
          "*** No Satellite Selected ***");
     return;
   };       

  setFileName (indata->svfname);

  indata->filsflg = 1;
  if (indata->iridflg != 3)
   {
    vCalcAOS();
    s1 = "Saved on File ";
    s2 = indata->svfname;
    s1 += s2;
    QMessageBox::information(this, "SatCal", s1);
   }
  else CmIridium();

  indata->filsflg = 0;
}

void SatcalView::vTLESave()  // save the TLE's on file
{
  QString s1, s2;
  int res;

  if (indata->sidxcn < 1)
   {
     QMessageBox::information(this, "SatCal",
          "*** No Satellite Selected ***");
     return;
   };

  setFileName (indata->tlesave);

  res = SaveTle(indata);
  if (res == 0)
   {
    s1 = "TLE's saved on file ";
    s2 = indata->tlesave;
    s1 += s2;
    QMessageBox::information(this, "SatCal", s1);
   };
  if (res == 2) QMessageBox::information(this, "TLE File Save",
        "*** ERROR *** while opening or writing TLE Save File");
}

void SatcalView::vRemoveSat()  // handle Remove Satellite Dialog
{
  int satnr, j, k;
  bool sats[GBL_MXSAT];

  if (indata->sidxcn == 0) return;

  satnr = indata->sidxcn;
  
  StRemSatDialog *dlg = new StRemSatDialog();
  dlg->setParams(satnr);
  if(dlg->exec())
   {
    dlg->getParams(sats);

    for (j=satnr-1; j>=0; j--)
     {
      if (sats[j])  // delete j-th satellite from list
       {
        // adjust prime satellite
        if (indata->satidx >= j) indata->satidx--;
        if (indata->satidx < 0) indata->satidx = 0;

        // decrease satellite counter
        indata->sidxcn--;

        // close the gaps in the list if applicable
        if (j < indata->sidxcn)
         {
	  for (k=j; k<indata->sidxcn; k++) ShiftElm (indata, k, k+1);
         };  // if j < sidxcn

        indata->delflg = 1;
       };  // if sats[j]
     }; // for j
    // adjust prime satellite again  just in case
    if (indata->satidx >= indata->sidxcn) indata->satidx = indata->sidxcn - 1;;
    if (indata->satidx < 0) indata->satidx = 0;
   };  
}

int SatcalView::vNextSat()
{
 // find next 10 satellites to pass
 int res, irdsv;

 if (indata->iridflg == 3)
  {
   irdsv = 1;
   indata->iridflg = 0;
  }
 else irdsv = 0;
 QApplication::setOverrideCursor(Qt::WaitCursor);
 res =	Nxt4Pass(indata);
 QApplication::restoreOverrideCursor();
 if (irdsv) indata->iridflg = 3;

 return res;
}

bool SatcalView::vInputDate()
{
  int year, month, day, hour, minute, second, tzauto;
  double tzone, duration, rtoff;
  
  StDateDialog *dlg = new StDateDialog();
  duration = indata->duration;
  rtoff = indata->rt_offset;
  if (indata->lnchflg == 1) duration = -duration;
  dlg->setParams(indata->year, indata->month, indata->day, indata->hour, indata->minute, indata->second,
          indata->tzone, indata->tzauto, duration, rtoff);
  if (dlg->exec())
   {
    dlg->getParams(year, month, day, hour, minute, second, tzone, tzauto, duration, rtoff);
    indata->year = year;
    indata->month = month;
    indata->day = day;
    indata->hour = hour;
    indata->minute = minute;
    indata->second = second;
    if (indata->lnchflg == 0) indata->duration = duration;
    if (indata->duration < 0) indata->duration = -indata->duration;
    indata->tzauto = tzauto;
    indata->tzone = tzone;
    indata->rt_offset = rtoff;
    setNewMjd(indata);
   }
  else return true;
 
  return false;
}

bool SatcalView::vInputLocation()
{
  double lat, lng, height;
  QString gn;
  bool ddmm;

  StObsDialog *dlg = new StObsDialog();
  dlg->setParams(indata->geolat, indata->geolong, indata->geoheight, indata->geoname, indata->dmsflg );
  if (dlg->exec())
   {
    dlg->getParams(lat, lng, height, ddmm );
    indata->geolat = lat;
    indata->geolong = lng;
    indata->geoheight = height;
    indata->dmsflg = ddmm;

    gn = dlg->getGeoname();
    QsToChar(gn, indata->geoname, 30);
   }
  else return true;

  return false;
}

void SatcalView::vSetIERS()
{
 double IERSxp, IERSyp, Ut1Utc;
 int UtcTai, yy;
 bool IERSflg;

 IERSxp = indata->IERSxp;
 IERSyp = indata->IERSyp;
 Ut1Utc = indata->Ut1Utc;
 IERSflg = indata->IERSflg;
 if (IERSflg) UtcTai = indata->UtcTai;
 else UtcTai = -int(indata->del_tdut - 32.184 + 0.0001);

 StIersDialog *dlg = new StIersDialog();
 dlg->setParams(IERSflg, IERSxp, IERSyp, UtcTai, Ut1Utc);
 if (dlg->exec())
  {
   dlg->getParams(IERSflg, IERSxp, IERSyp, UtcTai, Ut1Utc);
   indata->IERSxp = IERSxp;
   indata->IERSyp = IERSyp;
   indata->Ut1Utc = Ut1Utc;
   indata->UtcTai = UtcTai;
   indata->IERSflg = IERSflg;
   yy = indata->year;
   indata->del_tdut = SetTtUt(indata, yy);
  };
}

void SatcalView::vSysConf()
{
 double RKFtol, dF107, dFB107, dKp, dCd, dCr, dAm;
 double lift;
 bool sunf, moonf, refrc, atmxt;
 int mxit;

 RKFtol = indata->RKFtol;
 dF107 = indata->dF107;
 dFB107 = indata->dFB107;
 dKp = indata->dKp;                                    
 dCd = indata->dCd;
 dCr = indata->dCr;
 dAm = indata->dAm;
 lift = indata->liftdrag;
 sunf = indata->sungrav;
 moonf = indata->moongrav;
 refrc = indata->refract;
 atmxt = indata->atmxtin;
 mxit = indata->RKFmxitr;

 StSysConfDialog *dlg = new StSysConfDialog();
 dlg->setParams(RKFtol, dF107, dFB107, dKp, dCd, dCr, dAm, lift,
                  sunf, moonf, refrc, mxit, atmxt);
 if (dlg->exec())
  {
   dlg->getParams(RKFtol, dF107, dFB107, dKp, dCd, dCr, dAm, lift,
                  sunf, moonf, refrc, mxit, atmxt);
   indata->RKFtol = RKFtol;
   indata->dF107 = dF107;
   indata->dFB107 = dFB107;
   indata->dKp = dKp;
   indata->dCd = dCd;
   indata->dCr = dCr;
   indata->dAm = dAm;
   indata->liftdrag = lift;
   indata->sungrav = sunf;
   indata->moongrav = moonf;
   indata->refract = refrc;
   indata->atmxtin = atmxt;
   indata->RKFmxitr = mxit;
  };
}

void SatcalView::vCalcAOS()
{
  int irdsv;
  double res;
  bool ok, flrmgsav;
  char svfsave[110];

  flrmgsav = indata->flrmagf;
  indata->flrmagf = false;
  
  if (indata->iridflg == 3)  // iridium flare without file save
   {
    irdsv = indata->svfirst;
    strcpy (svfsave, indata->svfname);
    strcpy (indata->svfname, "./satird.tmp");
    indata->svfirst = 1;
    CmIridium();
    strcpy (indata->svfname, svfsave);
    indata->svfirst = irdsv;
    remove ("./satird.tmp");
    indata->flrmagf = flrmgsav;
    return;
   };

  irdsv = indata->iridflg;
  // check transit or flare case
  if (indata->occflg) indata->iridflg = 1;
  if (indata->flarflg && (indata->flarmod > 0)) indata->iridflg = 2;

  ok = true;
  if (indata->iridflg > 0)
   {
    res = QInputDialog::getDouble(this, "Transit/Flares", "Maximum Distance (km):", indata->iridkm, 0, 50000.0, 1, &ok);
    if (ok) indata->iridkm = res;
   }; 

  if (ok)
   {
    QApplication::setOverrideCursor(Qt::WaitCursor);
    aospass(indata, outdata);
    QApplication::restoreOverrideCursor();

    if (indata->filsflg == 0)  // display on screen
     {
      indata->graphflg = -1;
      showSattxt();
     };
   };

  indata->iridflg = irdsv;
  indata->flrmagf = flrmgsav;
}

void SatcalView::vAllSat()  // list of all satellite passes
{
  int succ, irdsv;
  double res;
  bool ok, flrmgsav;
  QString s1, s2;

  setFileName (indata->svfname);

  irdsv = indata->iridflg;
  flrmgsav = indata->flrmagf;
  indata->flrmagf = false;

  // check transit or flare case
  if (indata->iridflg < 3)
   {
    if (indata->occflg) indata->iridflg = 1;
    if (indata->flarflg && (indata->flarmod > 0)) indata->iridflg = 2;
   }; 

  ok = true;
  if ((indata->iridflg > 0) && (indata->iridflg < 3))
   {
    res = QInputDialog::getDouble(this, "Transit/Flares", "Maximum Distance (km):", indata->iridkm, 0, 50000.0, 1, &ok);
    if (ok) indata->iridkm = res;
   };

  if (ok)
   {
    QApplication::setOverrideCursor(Qt::WaitCursor);
    succ = AllSatAos(indata);
    QApplication::restoreOverrideCursor();

    if (succ == 0)
     {
      s1 = "List of passes saved on file ";
      s2 = indata->svfname;
      s1 += s2;
      if (indata->iridflg != 3)
             QMessageBox::information(this, "SatCal", s1);
     }
    else
     {
      if (succ == 1) QMessageBox::information(this, "All Passes",
                      "*** Problems with the output file");
      else QMessageBox::information(this, "All Passes",
                      "*** Problems with the input file");
     };   
   };  // if ok

  indata->iridflg = irdsv;
  indata->flrmagf = flrmgsav;
}

void SatcalView::vOrbitTrack()  // save track coordinates on file
{
  int succ;
  QString s1, s2;

  setFileName (indata->svfname);

    QApplication::setOverrideCursor(Qt::WaitCursor);
    succ = saveOrbitTrack(indata);
    QApplication::restoreOverrideCursor();

    if (succ == 0)
     {
      s1 = "Track saved on file ";
      s2 = indata->svfname;
      s1 += s2;
      if (indata->iridflg != 3)
             QMessageBox::information(this, "SatCal", s1);
     }
    else
     {
        if (succ == 1) QMessageBox::information(this, "Save Orbit Track",
                      "*** Problems with the output file");
      else QMessageBox::information(this, "All Passes",
                      "*** Problems with the input file");
     };
}

void SatcalView::CmIridium()  // calculate Iridium Flares
{
  int flflgsv, flmodsv, occflsv, flystsv, irdsv;
  double occrasv, occdecsv, maglsv;
  double mag;
  bool ops, irsol;

  irdsv = indata->iridflg;
  indata->iridflg = 3;
  flflgsv = indata->flarflg;
  flmodsv = indata->flarmod;
  flystsv = indata->flaryst;
  occflsv = indata->occflg;
  occrasv = indata->occra;
  occdecsv = indata->occdec;
  maglsv = indata->maglng;

  indata->flarflg = 1;
  indata->flarmod =  1;
  indata->flaryst = 0;
  indata->occflg = 0;
  indata->occra = 0;
  indata->occdec = -40.0;

  StIridiumDialog *dlp = new StIridiumDialog();
  ops = (indata->iridops == 1);
  mag = indata->iridmag;
  irsol = indata->iridsol;
  dlp->setParams(mag, ops, irsol);
  if(dlp->exec())
   {
    dlp->getParams(mag, ops, irsol);
    indata->iridmag = mag;
    if (ops) indata->iridops = 1;
    else indata->iridops = 0;
    indata->iridsol = irsol;
    if (indata->moonflg) indata->maglng = 1.0;
    else indata->maglng = 0;
    vAllSat ();
    Iriddspy(indata, outdata);
    showtxt(true);
   };

  indata->flarflg = flflgsv;
  indata->flarmod = flmodsv;
  indata->flaryst = flystsv;
  indata->occflg = occflsv;
  indata->occra = occrasv;
  indata->occdec = occdecsv;
  indata->maglng = maglsv;
  indata->iridflg = irdsv;
}

void SatcalView::vIridium()  // prepare Iridium Flares
{
  int svfsf;
  char svfsave[105];
  char tlesvn[105];
  char tleird[105];
  QString s1, s2;

  svfsf = indata->svfirst;
  strcpy (svfsave, indata->svfname);
  strcpy (indata->svfname, "*iridiumf.doc");
  setFileName (indata->tlesave);
  strcpy (tleird, "*iridium.txt");
  setFileName (tleird);
  strcpy (tlesvn, indata->tlename);
  strcpy (indata->tlename, tleird);
  indata->svfirst = 1;
  indata->iridmess = true;
  CmIridium();
  indata->iridmess = false;
  strcpy (indata->svfname, svfsave);
  strcpy (indata->tlename, tlesvn);
  indata->svfirst = svfsf;
}  

void SatcalView::vMatch()  // find matching sats
{
  int irdsv;
  double az, el, mxa, dra, tm;
  bool rdf, drf, vis;

  // get the time of the observation
  indata->lnchflg = 1;
  rdf = vInputDate();
  indata->lnchflg = 0;
  if (rdf) return;
  
  // get the observation data and search for matching sats
  if (indata->iridflg == 3)
   {
    irdsv = 1;
    indata->iridflg = 0;
   }
  else irdsv = 0;
  indata->sdfinish = 0;
  indata->odtmatch = false;  // reset possible flag from Orbit Det.

  StMatchDialog *dlg = new StMatchDialog();
  az = indata->sdaz1;
  el = indata->sdel1;
  mxa = indata->sdmxang;
  dra = indata->sddirt;
  tm = indata->sddeltm;
  rdf = (indata->sdra1flg==1);
  drf = (indata->sddrctflg==1);
  vis = indata->sdmvis;
  dlg->setParams(az, el, mxa, dra, tm, rdf, drf, vis);
  if(dlg->exec())
   {
    dlg->getParams(az, el, mxa, dra, tm, rdf, drf, vis);
    indata->sdaz1 = az;
    indata->sdel1 = el;
    indata->sdmxang = mxa;
    indata->sddirt = dra;
    indata->sddeltm = tm;
    if (indata->sddeltm < 0) indata->sddeltm = -indata->sddeltm;
    if (rdf) indata->sdra1flg = 1;
    else indata->sdra1flg = 0;
    if (drf) indata->sddrctflg = 1;
    else indata->sddrctflg = 0;
    indata->sdmvis = vis;

    QApplication::setOverrideCursor(Qt::WaitCursor);
    if (SatMatch(indata, outdata)) 
             QMessageBox::information(this, "Find Matching Sats",
                 "*** Unable to open TLE file ***");
    indata->sdfinish = 1;
    QApplication::restoreOverrideCursor();
   }; 

  if (irdsv) indata->iridflg = 3;
  if (indata->sdfinish) showtxt(true);
}

void SatcalView::vEditOrb()
{
  int sx, norad, aflg, paflg;
  QString fsname;
  QString ldsn;
  double mjdep, day, a, ra, aper, inc, ecc, m0, n0, adot, ndot, n6dot, bstar;
  bool numflg, newflg;

  StEditOrbDialog *dlg = new StEditOrbDialog();

  newflg = false;
  indata->lastsx = 9999;  // reset sx marker for SGP4 initialization

  do {

   if (newflg)
    {
     if (indata->sidxcn < GBL_MXSAT)
      {
       indata->satidx = indata->sidxcn;
       indata->sidxcn++;
      }
     else indata->satidx = GBL_MXSAT - 1;  
    };    
   sx = indata->satidx;
  
   if ((indata->sidxcn < 1) || newflg)  // preset the elements
    {
     indata->orbelm.norad[sx] = 0;
     strcpy(indata->orbelm.fsname[sx], "          ");
     strcpy(indata->orbelm.ldsgn[sx], "          ");
     indata->orbelm.aflg[sx] = 0;
     indata->orbelm.numflg[sx] = 0;
     indata->orbelm.mjdep[sx] = RT_MJD(indata);
     indata->orbelm.a[sx] = 0;
     indata->orbelm.ra[sx] = 0;
     indata->orbelm.aper[sx] = 0;
     indata->orbelm.inc[sx] = 0;
     indata->orbelm.ecc[sx] = 0;
     indata->orbelm.m0[sx] = 0;
     indata->orbelm.n0[sx] = 0;
     indata->orbelm.adot[sx] = 0;
     indata->orbelm.ndot[sx] = 0;
     indata->orbelm.n6dot[sx] = 0;
     indata->orbelm.bstar[sx] = 0;
     indata->orbelm.F107[sx] =  indata->dF107;
     indata->orbelm.FB107[sx] = indata->dFB107;
     indata->orbelm.Kp[sx] = indata->dKp;
     indata->orbelm.Cd[sx] = indata->dCd;
     indata->orbelm.Cr[sx] = indata->dCr;
     indata->orbelm.Am[sx] = indata->dAm;
     indata->orbelm.deltv[sx] = 0.0;
     indata->orbelm.dyaw[sx] = 0.0;
     indata->orbelm.dpitch[sx] = 0.0;
     indata->orbelm.ionflg[sx] = false;
     indata->orbelm.ionsun[sx] = true;
     indata->orbelm.updepoch[sx] = false;
    };
   fsname = indata->orbelm.fsname[sx];
   ldsn = indata->orbelm.ldsgn[sx];
   norad =  indata->orbelm.norad[sx];
   numflg =  indata->orbelm.numflg[sx];
   aflg =  indata->orbelm.aflg[sx];
   paflg =  indata->orbelm.paflg;
   mjdep = indata->orbelm.mjdep[sx];
   a = indata->orbelm.a[sx];
   ra = indata->orbelm.ra[sx];
   aper = indata->orbelm.aper[sx];
   inc = indata->orbelm.inc[sx];
   ecc = indata->orbelm.ecc[sx];
   m0 = indata->orbelm.m0[sx];
   n0 = indata->orbelm.n0[sx];
   adot = indata->orbelm.adot[sx];
   ndot = indata->orbelm.ndot[sx];
   n6dot = indata->orbelm.n6dot[sx];
   bstar = indata->orbelm.bstar[sx];

   dlg->setParam1(fsname, norad, aflg, paflg, mjdep, indata->lnchflg);
   dlg->setParam2(a, ra, aper, inc, ecc, m0, n0, adot, ndot, n6dot, bstar);
   dlg->setParam3(ldsn, numflg);
   if(dlg->exec())
    {
     dlg->getParam1(fsname, norad, aflg, paflg, mjdep, day);
     dlg->getParam2(a, ra, aper, inc, ecc, m0, n0, adot, ndot, n6dot, bstar);
     dlg->getParam3(ldsn, numflg, newflg);
  
     QsToChar(fsname, indata->orbelm.fsname[sx], 20);
     QsToChar(ldsn, indata->orbelm.ldsgn[sx], 10);
     indata->orbelm.norad[sx] = norad;
     indata->orbelm.numflg[sx] = numflg;
     indata->orbelm.aflg[sx] = aflg;
     indata->orbelm.paflg = paflg;
     if (indata->lnchflg != 1)
      {
       if (!numflg || (ecc < 1.0))
	       indata->orbelm.mjdep[sx] = mjd(0,1,int(mjdep),0) + day;
       indata->orbelm.ra[sx] = ra;
       indata->orbelm.aper[sx] = aper;
       indata->orbelm.m0[sx] = m0;
      };
     indata->orbelm.a[sx] = a;
     indata->orbelm.inc[sx] = inc;
     indata->orbelm.ecc[sx] = ecc;
     indata->orbelm.n0[sx] = n0;
     indata->orbelm.adot[sx] = adot;
     indata->orbelm.ndot[sx] = ndot;
     indata->orbelm.n6dot[sx] = n6dot;
     indata->orbelm.bstar[sx] = bstar;
     if (!numflg) EpochState (indata, sx, false);
     if (indata->sidxcn == 0) indata->sidxcn = 1;
    }
   else
    {
     if (newflg)
      {
       newflg = false;  
       indata->sidxcn = indata->satidx;
       indata->satidx--;
       if (indata->satidx < 0) indata->satidx = 0;
      };
    };
  } while (newflg);  
}

void SatcalView::vDeltaV()  // delta-v maneuver
{
  bool dum, ionsun, ionold, epu;
  double deltv, dyaw, dpitch;
  int rlsat, sx, nmflg;
  
  if (indata->sidxcn > 0)
   {
    sx = indata->satidx;

    // get the time of maneuver first
    if (!indata->orbelm.ionflg[sx])
     {
      indata->lnchflg = 1;
      dum = vInputDate();
      indata->lnchflg = 0;
      if (dum) return;
     }; 

    // now the delta-v maneuver
    StDeltaDialog *dlg = new StDeltaDialog();
    deltv = indata->orbelm.deltv[sx];
    dyaw = indata->orbelm.dyaw[sx];
    dpitch = indata->orbelm.dpitch[sx];
    dum = indata->orbelm.ionflg[sx];
    nmflg = indata->orbelm.numflg[sx];
    epu = indata->orbelm.updepoch[sx];
    if (dum && nmflg)
     {
      deltv *= 1000.0;  // convert into mm/s^2
      ionold = true;
     }
    else ionold = false;  
    ionsun = indata->orbelm.ionsun[sx];

    dlg->setParams(deltv, dyaw, dpitch, dum, ionsun, nmflg, epu);
    if(dlg->exec())
     {
      dlg->getParams(deltv, dyaw, dpitch, dum, ionsun, epu);
      indata->orbelm.deltv[sx] = deltv;
      indata->orbelm.dyaw[sx] = dyaw;
      indata->orbelm.dpitch[sx] = dpitch;
      indata->orbelm.ionflg[sx] = dum;
      indata->orbelm.ionsun[sx] = ionsun;
      indata->orbelm.updepoch[sx] = epu;

      if (!(dum && nmflg))  // perform delta-v maneuver
       {
        if (!ionold)
         {       
          rlsat = indata->relsat;
          indata->relsat = 0;   // better to avoid confusion.
          if (ExDeltaV(indata, deltv, dyaw, dpitch))
           QMessageBox::information(this, "SatCal",
	               "Delta-V resulted in a hyperbolic orbit");
          indata->relsat = rlsat;
         }; 
       }
      else indata->orbelm.deltv[sx] /= 1000.0;  // mm/s^2 -> m/s^2
     }; 
   }
  else QMessageBox::information(this, "SatCal",
          "*** No Satellite Selected ***");
}

void SatcalView::vLaunch()  // launch orbit
{
  bool dum, lsouth;
  int lsx, svaflg;
  double lat, lng, tmeco;
	char otxt[105];
  QString s;
  
  // get the time of launch first
  indata->lnchflg = 1;
  dum = vInputDate();

  // get preliminary orbit elements
  svaflg = indata->orbelm.aflg[indata->satidx];
//  indata->orbelm.aflg[indata->satidx] = 0;
  if (!dum) vEditOrb();
  indata->lnchflg = 0;
  if (dum)
   {
    indata->orbelm.aflg[indata->satidx] = svaflg;
    return;
   }; 

  // get launch site info and perform launch calculations
  StLaunchDialog *dlg = new StLaunchDialog();
  lsouth = (indata->lsouthflg == 1);
  tmeco = indata->tmeco;
  lsx = indata->lsitex;
  lat = indata->lnchlat;
  lng = indata->lnchlng;
  dlg->setParams(lsx, lat, lng, lsouth, tmeco);
  if(dlg->exec())
   {
    dlg->getParams(lsx, lat, lng, lsouth, tmeco);
    if(lsouth) indata->lsouthflg = 1;
    else indata->lsouthflg = 0;
    indata->tmeco = tmeco;
    indata->lsitex = lsx;
    indata->lnchlat = lat;
    indata->lnchlng = lng;

    LaunchOrb(indata);   // now calculate the launch orbit
    if (indata->rendzs < 0)
     {
      sprintf(otxt, "Launch Azimuth (deg):\n\t%8.2lf", indata->lnchazm);
      decdot0(otxt);
      s = otxt;
      QMessageBox::information(this, "Launch Orbit", s);
     }; 
   }; 
}

void SatcalView::vRendz()  // rendezvous launch orbit
{
  QString s;
  char tmsg[40];
  int sx1, sx2;
  int day, month, year, hour, minute, second;
  double mjd, tzn;
  bool ok;

  if (indata->sidxcn <= 0)
   {
    QMessageBox::information(this, "Rendezvous Launch",
        "*** No target satellite selected ***");
    return;    
   };

  tzn = QInputDialog::getDouble(this, "Rendezvous Launch", "Time until close approach (days):", indata->rendtim, 0, 1000.0, 2, &ok);
  if (ok) indata->rendtim = tzn;
  indata->rendzt = indata->satidx; 
  indata->rendzs = indata->satidx + 1;
  if (indata->rendzs >= GBL_MXSAT) indata->rendzs = indata->satidx - 1;
  else
   {
    if (indata->rendzs >= indata->sidxcn) indata->sidxcn++;
   };  
  indata->satidx = indata->rendzs;

  sx1 = indata->rendzs;
  sx2 = indata->rendzt;
  ShiftElm(indata, sx1, sx2);

  vLaunch();

  // time of launch (preliminary)
  mjd = indata->orbelm.mjdep[sx1] + indata->tzone/24.0;;
  caldat (mjd, day, month, year, tzn);
  dms (tzn, hour, minute, mjd);
  second = int(mjd);
  indata->year = year;
  indata->month = month;
  indata->day = day;
  indata->hour = hour;
  indata->minute = minute;
  indata->second = second;

  // one iteration to get a more accurate time
  LaunchOrb(indata);   // now calculate the launch orbit

  // time of launch
  mjd = indata->orbelm.mjdep[sx1] + indata->tzone/24.0;;
  caldat (mjd, day, month, year, tzn);
  dms (tzn, hour, minute, mjd);
  second = int(mjd);
  indata->year = year;
  indata->month = month;
  indata->day = day;
  indata->hour = hour;
  indata->minute = minute;
  indata->second = second;
  sprintf(tmsg, "Launch time: %4i-%2i-%2i  %2i:%2i:%2i",
                  year, month, day, hour, minute, second);
  s = tmsg;
  QMessageBox::information(this, "Rendezvous Launch", s);

  indata->rendzs = -1;
}

void SatcalView::vOrbDet()
{
  bool az1flg, az2flg, odm, odp;
  double az1, el1, az2, el2, deltm;
  
  // get the time of launch first
  indata->lnchflg = 1;
	az1flg = vInputDate();
  indata->lnchflg = 0;
  if (az1flg) return;

  az1 = indata->az1;
  el1 = indata->el1;
  az2 = indata->az2;
  el2 = indata->el2;
  deltm = indata->deltm;
  az1flg = (indata->azra1flg==1);
  az2flg = (indata->azra2flg==1);
  odm = indata->odtmatch;
  if (indata->pnstars > 5) odp = true;
  else odp = false;
  
  StOrbDetDialog *dlg = new StOrbDetDialog();
  dlg->setParams(az1, el1, az2, el2, deltm, az1flg, az2flg, odm, odp);
  if(dlg->exec())
   {
    dlg->getParams(az1, el1, az2, el2, deltm, az1flg, az2flg, odm);
    indata->az1 = az1;
    indata->el1 = el1;
    indata->az2 = az2;
    indata->el2 = el2;
    indata->deltm = deltm;
    if (az1flg) indata->azra1flg = 1;
    else indata->azra1flg = 0;
    if (az2flg) indata->azra2flg = 1;
    else indata->azra2flg = 0;
    indata->odtmatch = odm;

    OrbitDet(indata);
    vEditOrb();  // show result
    if (odm)
     { 
      QApplication::setOverrideCursor(Qt::WaitCursor);
      if (SatMatch(indata, outdata)) 
             QMessageBox::information(this, "Find Matching Sats",
                 "*** Unable to open TLE file ***");
      indata->sdfinish = 1;
      QApplication::restoreOverrideCursor();
      if (indata->sdfinish) showtxt(true);
     }; 
   }; 
}

void SatcalView::vSwitch()
{
  int sx, rs;
  
  StSwitchDialog *dlg = new StSwitchDialog();
  dlg->setParams(indata->satidx, indata->relsat, indata->sidxcn);
  if(dlg->exec())
   {
    dlg->getParams(sx, rs);
    indata->satidx = sx;
    indata->relsat = rs;
   };
}

void SatcalView::vStep()
{
 int min;
 double sec;

 StStepDialog *dlg = new StStepDialog();
 min = indata->tstpmin;
 sec = indata->tstpsec;
 dlg->setParams(min, sec);
 if(dlg->exec())
  {
    dlg->getParams(min, sec);
    if (min < 0) min = -min;
    if (sec < 0) sec = -sec;
    if ((min == 0) && (sec == 0)) sec = 0.1;
    indata->tstpmin = min;
    indata->tstpsec = sec;
    indata->tstep=double(indata->tstpmin)/1440.0+indata->tstpsec/86400.0;
  };  
}

void SatcalView::vText()
{
 if (indata->graphflg == 1)
  {
   indata->graphflg = 0;
   resize(buffer->width(),buffer->height());
   if (dsplmd == 2) indata->graphflg = -1;
   this->unsetCursor();
  }
 else
  {
   indata->graphflg = 1;
   if (indata->pastro) resize(buffPPic->width(), buffPPic->height());
   else resize(buffP->width(), buffP->height());
   this->setCursor( Qt::CrossCursor );
   goHome();
  };  

 update();
}

void SatcalView::vLocalPass()
{
  // display local sat info
  QString s;
  char otxt[995];

  LocalInf(indata, otxt);
  s = otxt;
  
  StLocalDialog *dlg = new StLocalDialog();
  dlg->setParams(s, indata->dfsz);
  dlg->exec();
}

void SatcalView::vLocalInf()
{
 // local info via manual entry
  double lat, lng, ht;
  char   svname[31];

  if (indata->skymap) return;

  timsuspd = true;
  // save coordinates
  lat = indata->geolat;
  lng = indata->geolong;
  ht = indata->geoheight;
  strcpy (svname, indata->geoname);

  // get coordinates
  vInputLocation();

  // calculate and display local circumstances
  vLocalPass();

  // restore old coordinates
  indata->geolat = lat;
  indata->geolong = lng;
  indata->geoheight = ht;
  strcpy (indata->geoname, svname);
  timsuspd = false;
}

void SatcalView::vMapDialog()
{
 int mgrid, mgrey, skyprj, mpnight, skymap, ra4d, fz, mapprj;
 bool bld, satsky;
 double skaz, centmer, frequ;

 StMapDialog *dlg = new StMapDialog();
 mgrid = indata->mapbuf.emagrid;
 mgrey = indata->mapbuf.emagrey;
 skyprj = indata->skyprj;
 mpnight = indata->mpnight;
 skymap = indata->skymap;
 satsky = indata->satsky;
 ra4d = indata->ra4dflg;
 skaz = indata->skazimuth;
 mapprj = indata->mapprj;
 if ((mapprj == 3) && (!indata->physmap)) mapprj = 0;
 centmer = indata->centmer;
 frequ = indata->frequ;
 dlg->setParams(mgrid, mgrey, skyprj, mpnight, skymap, ra4d, skaz, satsky);
 dlg->setParamF(indata->mfsz, indata->fbold, mapprj, centmer, frequ,
                indata->physmap);
 if(dlg->exec())
  {
   dlg->getParams(mgrid, mgrey, skyprj, mpnight, skymap, ra4d, skaz, satsky);
   dlg->getParamF(fz, bld, mapprj, centmer, frequ);
   indata->mapbuf.emagrid = mgrid;
   indata->mapbuf.emagrey = mgrey;
   indata->skyprj = skyprj;
   indata->mpnight = mpnight;
   indata->skymap = skymap;
   indata->satsky = satsky;
   indata->ra4dflg = ra4d;
   indata->skazimuth = skaz;
   indata->mfsz = fz;
   indata->fbold = bld;
   indata->mapprj = mapprj;
   if (centmer < 0) centmer += 360.0;
   indata->centmer = centmer;
   if (frequ < 0) frequ = 0;
   indata->frequ = frequ;
   
   vRefresh();
  };  
}

void SatcalView::showtxt(bool tab)
{
  // tab = true for tabular output
  char s[81];
  int i, j, x, y, dx, mxl;
  QString qs;

  dsplmd = 1;
  indata->graphflg = 0;
  this->unsetCursor();

  QFont f ("helvetica", fsz);
  buffer->fill();

  y = 0;

  QPainter painter;
  painter.begin(buffer);
  painter.setFont(f);
  QFontMetrics fm = painter.fontMetrics();
  dx = fm.width("M");
  painter.setPen(QColor(0,0,0));
  int maxlen = GBL_MAXCPL;

  for (i = 0; i < outdata->curlines; ++i)
   {
    y += fm.ascent();
    for (j = 0; j < maxlen; j++) s[j] = outdata->wbuf[i][j];
    qs = s;
    x = 10;
    mxl = qs.length();

    if (tab && (s[maxlen-1] == '~')) // tabular output
     {
      for (j = 0; j < mxl; j++)
       {
        qs = outdata->wbuf[i][j];
        painter.drawText( x, y, qs);
        x += dx;
       };
     }
    else
     {
      for (j = 0; j < mxl; j++)
       {
        qs = outdata->wbuf[i][j];
        painter.drawText( x, y, qs);
        if (outdata->wbuf[i][j] == '\t') x = 10 + j*dx;
        else x += fm.width(outdata->wbuf[i][j]);
       };
     };
   };

   painter.end();

   update();
   goHome();
}

void SatcalView::showSattxt()
{
  char s[81];
  int i, j, x, y, dx, mxl, py;
  QString qs;

  dsplmd = 2;
  this->unsetCursor();

  QFont f ("helvetica", fsz);
  buffer->fill();

  y = 0;

  QPainter painter;
  painter.begin(buffer);
  painter.setFont(f);
  QFontMetrics fm = painter.fontMetrics();
  dx = fm.width("M");
  painter.setPen(QColor(0,0,0));
  int maxlen = GBL_MAXCPL;

  py = 0;
  for (i = 0; i < outdata->curlines; ++i)
   {
    y += fm.ascent();
    for (j = 0; j < maxlen; j++) s[j] = outdata->wbuf[i][j];
    qs = s;
    x = 10;
    mxl = qs.length();

    // record y position of pass info
    if (s[0] != ' ')  // this line contains pass info
     {
      if (py < GBL_MAXPASS)
       {
        epos[py] = y - fm.ascent() - 2;
        py++;
       };
     };

    if (s[maxlen-1] == '~') // tabular output
     {
      for (j = 1; j < mxl; j++)
       {
        qs = outdata->wbuf[i][j];
        painter.drawText( x, y, qs);
        x += dx;
       };
     }
    else
     {
      for (j = 1; j < mxl; j++)
       {
        qs = outdata->wbuf[i][j];
        painter.drawText( x, y, qs);
        if (outdata->wbuf[i][j] == '\t') x = 10 + j*dx;
        else x += fm.width(outdata->wbuf[i][j]);
       };
     };
   };

  painter.end();
  update();
  goHome();
}

void SatcalView::showPassDetails( int y )
 {
  int k, j;
  int irdsv;

  if (indata->iridflg == 3)
   {
    indata->iridflg = 0;
    irdsv = 1;
   }
  else irdsv = 0;

  k = -1;
  if (indata->npass > 0)
   {
    for (j=0; j<indata->npass; j++)
     {
      if (y > epos[j]) k = j;
     };
   };

  if (k < 0) return;

  indata->npsel = k;

  QApplication::setOverrideCursor(Qt::WaitCursor);
  StPassDialog *dlg = new StPassDialog();
  dlg->setParams(indata->dfsz);
  QApplication::restoreOverrideCursor();
  dlg->exec();

  if (irdsv) indata->iridflg = 3;
 }

void SatcalView::vTmCheck(int timon)
{
 if (timon == 0)
  {
   if (indata->timer_on == 1)
    {
     indata->timer_on = 0;
     vCurrent();
    };
  };  
 indata->timer_del = 0;

 if (timon == 1) indata->timer_on = 1;
 if (timon == 2)
  {
   indata->timer_on = 0; 
   indata->timer_del = 2;
   setNewMjd(indata);
   vCurrent(); 
   dltend = indata->lastmjd + indata->duration;
  }; 
   
 if (timon > 0)
  {
   if (indata->graphflg < 1) vRefresh();
  };  
}

void SatcalView::RT_Plot()
 {
  // plot the satellite position and ranging data in Real Time 
  double t;

  if (timsuspd) return;
  
  indata->graphflg = 1;
  t = RT_MJD (indata);
  SatPlot (t, 1);
 }

bool SatcalView::PlotDelta()
 {
  double t;

  if (timsuspd) return true;
  
  t = indata->lastmjd + indata->tstep;
  if (t > dltend) return true;
  SatPlot (t, 0);

  return false;
 }

void SatcalView::vCurrent()
{
  double t;

  if (indata->graphflg == 0)
   {
    indata->graphflg = 1;
    vRefresh();
   };

  t = indata->lastmjd;
  SatPlot (t, 1);
}

void SatcalView::vPlotNext()
{
  // plot the satellite position on next pass 
  int irdsv;
  int dd, mm, yy, deg, mnt, m1;
  double hh, sec;
  double t, ts;
  double ti[3], el[3], az[3];

  if (indata->sidxcn < 1)
   {
    QMessageBox::warning(this, "SatCal", "No satellite selected", 0,  "Quit", 0, 0, 1 );
    return;
   };

  t = indata->lastmjd + indata->tstep;
  ts = t + indata->duration;

  if (indata->iridflg == 3)
   {
    indata->iridflg = 0;
    irdsv = 1;
   }
  else irdsv = 0;

  if (indata->relsat && indata->skymap) indata->relsat = 0;   
  if (nextpass(indata, indata->satidx, t, ts, ti, el, az))
   {
    vRefresh();
    indata->passact = 1;    // mark the pass as currently active
    if (indata->relsat && (indata->sidxcn > 1)) t = ti[0];
    else
     {
      if (indata->skymap && (indata->skyprj == 5)) // automatic azimuth
       {
        SkyAzim(indata, ti[1]);
        vRefresh();
       };
      t = ti[1];
     };
    if (indata->tstep >= (1.0/86400.0))  // set t to exact sec
     {
      caldat(t, dd, mm, yy, hh);
      dms (hh,deg,mnt,sec);
      m1 = int(sec+0.5);
      sec = m1;
      hh = ddd (deg, mnt, sec);
      t = mjd (dd, mm, yy, hh);
     }; 
    SatPlot (t, 1); 
   }
  else
   {
    if (indata->graphflg)
     {
      t = indata->lastmjd;
      SatPlot (t, 1);
     };
    QMessageBox::warning(this, "SatCal", "No suitable pass found", 0,  "Quit", 0, 0, 1 );
   };

  if (irdsv) indata->iridflg = 3;
}

void SatcalView::vPlotLast()
{
  // plot the satellite position on last pass
  int irdsv;
  int dd, mm, yy, deg, mnt, m1;
  double hh, sec;
  double t, ts;
  double ti[3], el[3], az[3];

  if (indata->sidxcn < 1)
   {
    QMessageBox::warning(this, "SatCal", "No satellite selected", 0,  "Quit", 0, 0, 1 );
    return;
   };

  t = indata->lastmjd - indata->tstep;
  ts = t - indata->duration;

  if (indata->iridflg == 3)
   {
    indata->iridflg = 0;
    irdsv = 1;
   }
  else irdsv = 0;

  if (indata->relsat && indata->skymap) indata->relsat = 0; 
  if (lastpass(indata, indata->satidx, t, ts, ti, el, az))
   {
    vRefresh();
    indata->passact = 1;    // mark the pass as currently active
    if (indata->relsat && (indata->sidxcn > 1)) t = ti[0];
    else
     {
      if (indata->skymap && (indata->skyprj == 5)) // automatic azimuth
       {
        SkyAzim(indata, ti[1]);
        vRefresh();
       };
      t = ti[1];
     };
    if (indata->tstep >= (1.0/86400.0))  // set t to exact sec
     {
      caldat(t, dd, mm, yy, hh);
      dms (hh,deg,mnt,sec);
      m1 = int(sec+0.5);
      sec = m1;
      hh = ddd (deg, mnt, sec);
      t = mjd (dd, mm, yy, hh);
     };
    SatPlot (t, 1);
   }
  else
   {
    if (indata->graphflg)
     {
      t = indata->lastmjd;
      SatPlot (t, 1);
     };
    QMessageBox::warning(this, "SatCal", "No suitable pass found", 0,  "Quit", 0, 0, 1 );
   };

  if (irdsv) indata->iridflg = 3;
}

void SatcalView::vForward()
{
  double t;

  if (indata->graphflg == 0)
   {
    indata->graphflg = 1;
    vRefresh();
   };

  t = indata->lastmjd + indata->tstep;
  SatPlot (t, 1);
}

void SatcalView::vBackward()
{
  double t;

  if (indata->graphflg == 0)
   {
    indata->graphflg = 1;
    vRefresh();
   }; 

  t = indata->lastmjd - indata->tstep;
  SatPlot (t, 1);
}

void SatcalView::vNames(bool on)
{
 if (on) indata->namflg = 1;
 else indata->namflg = 0;
 if ((indata->timer_on == 0) && (indata->timer_del == 0))
 vCurrent();
}
  
void SatcalView::vPicSwitch(bool on)
{
 if (on)
  {
   indata->pastro = true;
   indata->graphflg = 1;
  }
 else
  {
   indata->pastro = false;
   if (indata->graphflg > 0) resize(buffP->width(), buffP->height());
  };

 update();
}
  
void SatcalView::vLeft(bool on)
{
 if (on) indata->leftm = 1;
 else indata->leftm = 0;
}

void SatcalView::vMag()
{
  bool ok;
  double res;

  res = QInputDialog::getDouble(this, "Magnitude", "Charact.Length (meters):",
                indata->maglng, 0, 10000.0, 2, &ok);
  if (ok) indata->maglng = res;
}

void SatcalView::vVisibility()
{
  double sunel, satel, yrepoch, occra, occdec, flrmagx;
  int sunmplt, occflg, iridflg, flarflg, flarmod, flaryst, visflg, moonflg;
  bool eclflg, solflar, flrmagf, cylflg;

  sunel = indata->minsunel;
  satel = indata->minsatel;
  yrepoch = indata->yrepoch;
  cylflg = indata->cylflg;
  visflg = indata->visflg;
  moonflg = indata->moonflg;
  sunmplt = indata->sunmplt;
  occflg = indata->occflg;
  iridflg = indata->iridflg;
  flarflg = indata->flarflg;
  flarmod = indata->flarmod;
  flaryst = indata->flaryst;
  eclflg = indata->eclflg;
  solflar = indata->solflar;
  flrmagf = indata->flrmagf;
  flrmagx = indata->flrmagx;
  if (cylflg && (flarflg == 1) && (flarmod < 0))
   {
    occra = indata->cylra;
    occdec = indata->cyldec;
   }
  else
   {
    occra = indata->occra;
    occdec = indata->occdec;
   };

  StVisDialog *dlg = new StVisDialog();
  dlg->setParams(sunel, satel, yrepoch, occra, occdec, visflg, moonflg);
  dlg->setParam2(sunmplt, occflg, iridflg, flarflg, flarmod, flaryst,
                  eclflg, solflar, flrmagf, flrmagx, cylflg);
  if(dlg->exec())
   {
    dlg->getParams(sunel, satel, yrepoch, occra, occdec, visflg, moonflg);
    dlg->getParam2(sunmplt, occflg, iridflg, flarflg, flarmod, flaryst,
                  eclflg, solflar, flrmagf, flrmagx, cylflg);
  
    indata->minsunel = sunel;
    indata->minsatel = satel;
    indata->yrepoch = yrepoch;
    indata->cylflg = cylflg;
    if (cylflg && (flarflg == 1) && (flarmod < 0))
     {
      indata->cylra = occra;
      indata->cyldec = occdec; 
     }
    else
     {   
      indata->occra = occra;
      indata->occdec = occdec;
     }; 
    indata->visflg = visflg;
    indata->moonflg = moonflg;
    indata->sunmplt = sunmplt;
    if (indata->sunmplt)
     {
      if (((indata->occflg == 1) && (occflg == 2))
                || ((indata->occflg == 2) && (occflg == 1))) indata->mrefmjd = 0;
     };
    indata->occflg = occflg;
    indata->iridflg = iridflg;
    indata->flarflg = flarflg;
    indata->flarmod = flarmod;
    indata->flaryst = flaryst;
    indata->eclflg = eclflg;
    indata->solflar = solflar;
    indata->flrmagf = flrmagf;
    indata->flrmagx = flrmagx;
   };  
}

void SatcalView::vRefreshP()
{
 vRefresh();
 if (indata->sidxcn > 0) vCurrent();
}

void SatcalView::vRefresh()
{
  // initialize graphic output
 
  QString qs;
  int wdt, hgt, j, ofz;
  bool sunmplt;
  double t;

  dsplmd = 3;
  this->setCursor( Qt::CrossCursor );
  indata->graphflg = 1;

  wdt = GBL_MAPWIDTH;
  hgt = GBL_MAPHEIGHT;

  if (indata->mapbuf.emagrey)
   {
    if (indata->mapbuf.emagrey > 1) buffM->fill( QColor(0,0,0));
    else buffM->fill( QColor(200,200,200));
   }   
  else buffM->fill( QColor(255,255,255));

  // get origin and scale factors
  origx = wdt / 2;
  origy = hgt / 2;
  sunmplt = indata->sunmplt && ((indata->occflg == 1) || (indata->occflg == 2));

  QPainter painter;
  painter.begin(buffM);
  j = GBL_MAPHEIGHT + 1;

  // plot physical map if needed
  if (indata->physmap && (indata->skymap == 0) && (indata->mapprj == 3)
        && (!sunmplt))
   {
    if(indata->centmer == 0) painter.drawPixmap(0,0,*buffPWM);
    else
     {
      ofz = int(indata->centmer);
      if (ofz < -180) ofz = -180;
      if (ofz > 360) ofz = 0;
      if (ofz < 0) ofz += 360;
      ofz = 2*ofz;  
      painter.drawPixmap(-ofz,0,*buffPWM);
      ofz = 720 - ofz;
      painter.drawPixmap(ofz,0,*buffPWM);
     };
    }; 

  painter.drawPixmap(0,j,*buffoot);

  if (sunmplt)
   {
    xscale = double(origy) * 2.8;
    yscale = xscale;
    if (indata->sidxcn > 0)  // need position of sun or moon for PltSunM
     {
      j = indata->satidx;
      t = indata->lastmjd;
      GetSatPos (t, indata, satps, j, 0);
      indata->mrefmjd = indata->lastmjd;
     };
    PltSunM(painter);
    indata->lstgraph = 1;
   }
  else
   {
    if (indata->skymap)
     {
      if (indata->skyprj)
       {
        origy = hgt;
        xscale = double(origx) / 1.414;
        yscale = double(origy) / 1.414;
       }
      else
       {
        xscale = double(origy) / 1.678;
        yscale = xscale;
       };
      PltSky(painter);
     }
    else
     {
      if ((indata->mapprj == 1) || (indata->mapprj == 2))
       {
        xscale = double(origy) / 1.678;
        yscale = xscale;
       }
      else
       { 
        xscale = double(origx) / 180.0;
        yscale = double(origy) / 90.0;
       }; 
      PlotWorld(painter); 
     };
    indata->lstgraph = 0;
    indata->mrefmjd = indata->lastmjd;
   };

  painter.end();
  buffP->fill(QColor(0,0,0));
  painter.begin(buffP);
  painter.drawPixmap(0,0,*buffM);
  painter.end();

  if (indata->pastro)
   {
    painter.begin(buffPPic);
    painter.drawPixmap(0,0,*buffTPic);
    painter.end();
   };

  update();
}

void SatcalView::PicPlot(double t)
 {
  // plot the satellite position on the loaded picture

  const int tz5 = 4;

  int dd, mm, yy, hh, mn, ss;
  int j, tht, twt, xs, ys, sx, hy;
  double hour, sec;
  double ra0, dec0, ra1, dec1, xds, yds;
  bool visflg;
  static char ptxt[31];
  char dts[31];
  char ntx[31];
  QString qs;
  QFont f;
  QPoint p;
  QPoint p0(0,0);
  QPoint poffset;
  QColor satcolor[GBL_MXSAT];  // color used to represent satellites

  poffset = this->mapFromParent(p0);
  twt = poffset.x();

  if (indata->pnstars < 4) return;  // picture coordinates not initalized

  indata->relsat = 0;
  indata->lastmjd = t;  // remember the last time
  f.setFamily("Helvetica");
  f.setPointSize(indata->mfsz);
  f.setBold(indata->fbold);

  ra0 = indata->rstarra[0];
  dec0 = indata->rstardec[0];

  for (j=0; j<indata->sidxcn; j++)  // just in case
   {
    satcolor[j] = QColor(0,255,0);
   };
  if (indata->mapbuf.emagrey != 2) satcolor[0] = QColor(0,0,0);
  else satcolor[0] = QColor(255,255,255);
  satcolor[1] = QColor(255,0,0);
  satcolor[2] = QColor(0,0,255);
  satcolor[4] = QColor(255,255,0);
  satcolor[5] = QColor(255,0,255);
  satcolor[6] = QColor(0,0,255);
  satcolor[8] = QColor(255,255,0);
  satcolor[9] = QColor(255,0,255);

  sx = indata->satidx;  // assigned to have less to write with index

  QPainter WorldMap;
  WorldMap.begin(buffPPic);
  WorldMap.drawPixmap(0,0,*buffTPic);

  WorldMap.setFont(f);
  QFontMetrics fm = WorldMap.fontMetrics();
  hy = fm.ascent() + 10;
  hy = hy + poffset.y();

  tht = hy;
  caldat(t+indata->tzone/24.0, dd, mm, yy, hour);
  indata->dspyear = yy;
  dms (hour, hh, mn, sec);
  ss = int(sec+0.4999);

  // calculate position of satellite at time t
  for (j=0; j<indata->sidxcn; j++)
	 GetSatPos (t, indata, satps, j, 0);

  /* store data in their positions */
  GetMonth (mm, dts);
  dts[3]=' ';
  if (indata->tstep >= 1.157407407e-05) // if >= 1 sec
   {
    if (ss == 60)
     {
      if (mn < 59)
       { 
        mn++;
        ss = 0;
       }
      else
       {
        if (hh < 24)
         {
          hh++;
          mn = 0;
          ss = 0; 
         };   
       };    
     };
    sprintf(&dts[4],"%2i  %2i:%02i:%02i", dd, hh, mn, ss);
    dts[16]='\0';
   }
  else
   {
    if (sec >= 59.95)
     {
      if (mn < 59)
       {
        mn++;
        sec = 0;
       }
      else
       {
        if (hh < 24)
         {
          hh++;
          mn = 0;
          sec = 0;
         };
       };
     };
    sprintf(&dts[4],"%2i  %2i:%02i:%4.1lf", dd, hh, mn, sec);
    dts[18]='\0';
    decdot(dts);
   };
  strcpy (ptxt, dts);

  qs = ptxt;
  WorldMap.setPen(QColor(255,255,255));
  WorldMap.drawText(twt+10,tht,qs);

  if (indata->flarflg && (indata->flarmod == -1))
   {
    WorldMap.setPen(QColor(255,255,0));
    sprintf(dts, "Y:%7.1lf", indata->occra);
    decdot(dts);
    qs = dts;
    WorldMap.drawText(twt+180,tht,qs);
    sprintf(dts, "P:%7.1lf", indata->occdec);
    decdot(dts);
    qs = dts;
    WorldMap.drawText(twt+290,tht,qs);
   }
  else
   {
    strcpy (dts, "R.A.:");
    prpang (ntx, 1, -2, satps->RA[sx]);
    strcat (dts, ntx);
    qs = dts;
    WorldMap.drawText(twt+180,tht,qs);
    strcpy (dts, "Dec.:");
    prpang (ntx, 1, -1, satps->Dec[sx]);
    strcat (dts, ntx);
    qs = dts;
    WorldMap.drawText(twt+320,tht,qs);
   };

  // output name of currently selected satellite
  if (indata->sidxcn == 0)
   {
    WorldMap.setPen(QColor( 255, 0, 0));
    qs = "*** No Satellite ***";
    WorldMap.drawText(twt+500,tht,qs);
   }
  else
   {
    WorldMap.setPen(satcolor[sx]);
    for (j=0; j<18; j++) dts[j] = ' ';
    strncpy (dts, indata->orbelm.fsname[sx], 15);
    dts[16] = '\0';
    qs = dts;
    WorldMap.drawText(twt+500,tht,qs);
   };

  // mark the satellite position(s) on picture

  f.setPointSize(8);
  f.setBold(false);
  WorldMap.setFont(f);

  for (sx=0; sx < indata->sidxcn; sx++)
   {
    visflg = true;
    if (indata->visflg)
     {
      if (satps->sunang[sx] < 0) visflg = false;
      if (indata->moonflg && (satps->moonang[sx] > 0)) visflg = true;
     };
 
    if ((satps->elev[sx] > 0) && visflg)
     {
      WorldMap.setPen( satcolor[sx] );

      ra1 = DmsDegF(satps->RA[sx])*M_PI/12.0;
      dec1 = DmsDegF(satps->Dec[sx])*M_PI/180.0;

      equplt(ra0, dec0, ra1, dec1, xds, yds);

      dec1 = indata->pltcx[0];
      if (fabs(dec1) < 1.0e-40) dec1 = 1.0e-40;  // just in case
      ra1 = indata->pltcy[1] - indata->pltcy[0] * indata->pltcx[1] / dec1;
      if (fabs(ra1) < 1.0e-40) ra1 = 1.0e-40;  // just in case
      ra1 = (yds - indata->pltcy[0]*(xds-indata->pltcx[2])/dec1 
               - indata->pltcy[2]) / ra1; 
      dec1 = (xds - indata->pltcx[1]*ra1 - indata->pltcx[2]) / dec1; 

      xs = int(dec1+0.5) + indata->pstarx[0];
      ys = int(ra1+0.5) + indata->pstary[0];

      lastspx[sx] = xs;
      lastspy[sx] = ys;

      xs -= 2;
      ys -= 2;
      WorldMap.drawRect(xs, ys, tz5, tz5);
      WorldMap.drawPoint(xs+2,ys+2 ); 

      // plot satellite names
      if (indata->namflg)
       { 
        xs = lastspx[sx];
        ys = lastspy[sx];
        xs -= 2;
        ys -= 2;
        strncpy (dts, indata->orbelm.fsname[sx], 15);
        dts[16] = '\0';
        qs = dts;
        WorldMap.drawText(xs+3, ys, dts);
       };
     }; // if elevation > 0 

   };  // for sx

  update();
 };

void SatcalView::SatPlot(double t, int rngflg)
 {
  // plot the satellite position and ranging data at MJD t
  //  rngflg = 1 if visibility circle is to be plotted, 0 otherwise

  const int tz5 = 4; 

  int dd, mm, yy, hh, mn, ss;
  int j, tht, xs, ys, sx, hy;
  int rlsflg;  // = 1 if relative satellite mode
  int smpflg;  // 1 if Sun/Moon plot; 0 for normal map
  int skyvis;  // 1 if sat illuminated, 0 if not (only for Sky Map)
  double hour, sec, spsel, glatsav, glongsav, gheightsav;
  static char ptxt[31];
  char dts[31];
  char ntx[31];
  QString qs;
  QFont f;
  QPoint p;
  QColor satcolor[GBL_MXSAT];  // color used to represent satellites

  if (indata->pastro)
   {
    PicPlot(t);
    return;
   };

  skyvis = 0;  // just to avoid a compiler warning
  glatsav = indata->geolat;
  glongsav = indata->geolong;
  gheightsav = indata->geoheight;

  indata->lastmjd = t;  // remember the last time
  sx = indata->satidx;  // assigned to have less to write with index

  if (indata->satsky && (indata->skymap == 1))
   {
    GetSatPos (t, indata, satps, sx, 0);
    indata->geolat = satps->lat[sx];
    indata->geolong = satps->lng[sx];
    indata->geoheight = satps->height[sx] * 1000.0;
   };

  f.setFamily("Helvetica");
  f.setPointSize(indata->mfsz);
  f.setBold(indata->fbold);

  if (indata->sunmplt && ((indata->occflg == 1) || (indata->occflg == 2)))
	 smpflg = 1;
  else smpflg = 0;

  if (smpflg)
   {
    if ((indata->lstgraph == 0) ||
          (fabs(indata->mrefmjd - t) > 0.01)) vRefresh(); // Sun plot mode
   }
  else
   {
    if (indata->lstgraph == 1) vRefresh(); // standard map mode
    else
     {
      if (indata->mpnight && (fabs(indata->mrefmjd - t) > 0.01)) vRefresh();
      if (indata->skymap)
       {
        sec = 0.004;  // 5.7 min
        if (indata->skyprj)
         {
          if (indata->mpnight) sec = 0.001;  // 1.4 min
          else sec = 0.002;
         }
        else
         {
          if (indata->mpnight) sec = 0.002;
         };
        if(indata->satsky || (fabs(indata->mrefmjd - t) > sec)) vRefresh();
       }; // skymap
     };
   };

  if (indata->relsat && indata->skymap) indata->relsat = 0;   
  if (indata->relsat && (indata->sidxcn > 1) && (indata->occflg == 0)
                                    && (indata->flarflg == 0)) rlsflg = 1;
  else rlsflg = 0;

  for (j=0; j<indata->sidxcn; j++)  // just in case
   {
    satcolor[j] = QColor(0,255,0);
   };
  if (indata->mapbuf.emagrey != 2) satcolor[0] = QColor(0,0,0);
  else satcolor[0] = QColor(255,255,255);
  satcolor[1] = QColor(255,0,0);
  satcolor[2] = QColor(0,0,255);
  satcolor[4] = QColor(255,255,0);
  satcolor[5] = QColor(255,0,255);
  satcolor[6] = QColor(0,0,255);
  satcolor[8] = QColor(255,255,0);
  satcolor[9] = QColor(255,0,255);

  tht = GBL_MAPHEIGHT + 1;

  QPainter WorldMap;
  WorldMap.begin(buffM);
  WorldMap.drawPixmap(0,tht,*buffoot);
  WorldMap.setFont(f);
  QFontMetrics fm = WorldMap.fontMetrics();
  hy = fm.ascent() + 5;

  tht += hy;
  caldat(t+indata->tzone/24.0, dd, mm, yy, hour);
  indata->dspyear = yy;
  dms (hour, hh, mn, sec);
  ss = int(sec+0.4999);

  // calculate position of satellite at time t
  for (j=0; j<indata->sidxcn; j++)
	 GetSatPos (t, indata, satps, j, rngflg);

  /* store data in their positions */
  GetMonth (mm, dts);
  dts[3]=' ';
  if (indata->tstep >= 1.157407407e-05) // if >= 1 sec
   {
    if (ss == 60)
     {
      if (mn < 59)
       { 
        mn++;
        ss = 0;
       }
      else
       {
        if (hh < 24)
         {
          hh++;
          mn = 0;
          ss = 0; 
         };   
       };    
     };
    sprintf(&dts[4],"%2i  %2i:%02i:%02i", dd, hh, mn, ss);
    dts[16]='\0';
   }
  else
   {
    if (sec >= 59.95)
     {
      if (mn < 59)
       {
        mn++;
        sec = 0;
       }
      else
       {
        if (hh < 24)
         {
          hh++;
          mn = 0;
          sec = 0;
         };
       };
     };
    sprintf(&dts[4],"%2i  %2i:%02i:%4.1lf", dd, hh, mn, sec);
    dts[18]='\0';
    decdot(dts);
   };
  strcpy (ptxt, dts);

  qs = ptxt;
  WorldMap.setPen(QColor(255,255,255));
  WorldMap.drawText(10,tht,qs);

  if (satps->height[sx] < 0) WorldMap.setPen(QColor (0, 0, 0));
  strcpy (dts, "Lat  :");
  prpang (ntx, indata->ra4dflg, indata->dmsflg, satps->lat[sx]);
  strcat (dts, ntx);
  qs = dts;
  WorldMap.drawText(150,tht,qs);

  tht += 22;
  sprintf(dts, "TimeZone: %g", indata->tzone);
  decdot(dts);
  qs = dts;
  WorldMap.drawText(10,tht,qs);
  strcpy (dts, "Lng :");
  prpang (ntx, indata->ra4dflg, indata->dmsflg, satps->lng[sx]);
  strcat (dts, ntx);
  qs = dts;
  WorldMap.drawText(150,tht,qs);
  if (rlsflg == 0)
   {
    sprintf(dts, "ObsSun:%7.1lf", satps->obssun);
    decdot(dts);
    qs = dts;
    WorldMap.drawText(590,tht,qs);
   };

  tht += 22;
  if (indata->timer_on) WorldMap.drawText(10,tht,"R/T");
  else
   {
    sprintf(dts, "Year:%5i", indata->dspyear);
    qs = dts;
    WorldMap.drawText(10,tht,qs);
   };  
  sprintf(dts, "Height:%8.1lf km", satps->height[sx]);
  decdot(dts);
  qs = dts;
  WorldMap.drawText(150,tht,qs);
  if (((indata->maglng > 0) && (rlsflg == 0))
                 || ((indata->occflg > 0) && (satps->occlat[sx] < 100.0))
                 || ((indata->flarflg > 0) && (indata->flarmod > 0)
                 && (satps->occlat[sx] < 100.0)))
   {
    if (satps->sunang[sx] < 0)
     {
      if (indata->moonflg && (satps->moonang[sx] > 0)) WorldMap.setPen(QColor (255, 255, 0));
      else WorldMap.setPen(QColor (0, 0, 0));
     };
    if (((indata->occflg > 0) && (satps->occlat[sx] < 100.0))
                      || ((indata->flarflg > 0) && (indata->flarmod > 0)
                      && (satps->occlat[sx] < 100.0)))
     {
      if (indata->occflg > 0) sprintf(dts, "dst:%8.1lf km", satps->mag);
      else
       {
        if (indata->flrmagf && (indata->iridflg < 3) && (indata->maglng > 0))
	            sprintf(dts, "Mag:%4.1lf", satps->mag);
        else sprintf(dts, "FlrAng:%7.2lf", indata->flrang);
       };  
     }
    else sprintf(dts, "Mag:%4.1lf", satps->mag);
    decdot(dts);
    qs = dts;
    WorldMap.drawText(590,tht,qs);
   }
  else
   {
    if ((indata->iridflg == 3) && (satps->occlat[sx] < 100.0))
                     sprintf(dts, "FlrAng:%7.2lf", indata->flrang);
    else
     {
      if (indata->frequ == 0) 
       {
        if(indata->cylflg && (satps->occlat[sx]<100.0) && (indata->flarflg>0)) 
                sprintf(dts, "CylAng:%7.2lf", indata->flrang);
        else sprintf(dts, "Rate:%7.3lf km/s", satps->rrate[sx]);
       } 
      else
       {
        sec = satps->rrate[sx];
        sec = sec / 299792.458;
        if (fabs(sec) > 0.99) sec = 0.99; // shouldn't happen!
        sec = (1.0-sec)/(1.0+sec);
        hour = sqrt(sec);
        hour *= indata->frequ;
        hour = hour - indata->frequ;
        sprintf(dts, "Hz: %g", hour);
       };    
     };   
    decdot(dts);
    qs = dts;
    WorldMap.drawText(590,tht,qs);
   };

  // darken sky output if satellite not sunlit, yellow if moonlit
  if ((satps->sunang[sx] < 0) && (rlsflg == 0))
   {
    if (indata->moonflg && (satps->moonang[sx] > 0)) WorldMap.setPen(QColor (255, 255, 0));
    else WorldMap.setPen(QColor (0, 0, 0));
   }
  else WorldMap.setPen(QColor (255, 255, 255));
  if (satps->height[sx] < 0) WorldMap.setPen(QColor (0, 0, 0));
  tht = GBL_MAPHEIGHT + hy + 1;
  if (rlsflg)
   {
    strcpy (dts, "Lat :");
    prpang (ntx, indata->ra4dflg, indata->dmsflg, satps->RA[sx]);
    strcat (dts, ntx);
    qs = dts;
    WorldMap.drawText(300,tht,qs);
    tht += 22;
    strcpy (dts, "Lng :");
    prpang (ntx, indata->ra4dflg, indata->dmsflg, satps->Dec[sx]);
    strcat (dts, ntx);
    qs = dts;
    WorldMap.drawText(300,tht,qs);
    tht += 22;
    sprintf(dts, "Hgt:%8.1lf km", satps->sunang[sx]);
    decdot(dts);
   }	  
  else
   {
    if (indata->occflg || indata->flarflg)
     {
      if (satps->occlat[sx] < 100.0)
       {
        if (indata->occflg || (indata->flarflg && (indata->flarmod > 0)))
         {
          strcpy (dts, "Lat :");
          prpang (ntx, indata->ra4dflg, indata->dmsflg, satps->occlat[sx]);
          strcat (dts, ntx);
          qs = dts;
          WorldMap.drawText(300,tht,qs);
          tht += 22;
          strcpy (dts, "Lng :");
          prpang (ntx, indata->ra4dflg, indata->dmsflg, satps->occlng[sx]);
          strcat (dts, ntx);
          qs = dts;
          WorldMap.drawText(300,tht,qs);
         }
        else
         {
          if (indata->flarmod == -1)
           {
            sprintf(dts, "Y:%7.1lf", indata->occra);
            decdot(dts);
            qs = dts;
            WorldMap.drawText(300,tht,qs);
            tht += 22;
            sprintf(dts, "P:%7.1lf", indata->occdec);
            decdot(dts);
            qs = dts;
            WorldMap.drawText(300,tht,qs);
           }
          else
           {
            if (indata->solflar) qs = "Solar Panel";
            else
             { 
              strcpy (dts, "R.A.:");
              prpang (ntx, indata->ra4dflg, -2, indata->occra);
              strcat (dts, ntx);
              qs = dts;
             };
            WorldMap.drawText(300,tht,qs);
            tht += 22;
            if (indata->solflar) strcpy (dts, "Ang.:");
            else strcpy (dts, "Dec.:");
            prpang (ntx, indata->ra4dflg, -1, indata->occdec);
            strcat (dts, ntx);
            qs = dts;
            WorldMap.drawText(300,tht,qs);
           };
         };  // occflg
       }
      else
       {
        qs = "No Intersection";
        WorldMap.drawText(300,tht,qs);
        tht += 22;
       };
      tht += 22;
      if (indata->occflg == 1) strcpy (dts, "Sun Transit");
      if (indata->occflg == 2) strcpy (dts, "Moon Transit");
      if (indata->occflg == 3) strcpy (dts, "Occultation");
      if (indata->flarflg)
       {
        if (indata->flarmod>0) strcpy (dts, "Flare Location");
        else strcpy (dts, "Flare Angle");
       };
      WorldMap.setPen(QColor (255, 0, 0));
     }
    else
     {
      if (indata->eclflg)  // solar eclipse check
       {
        sprintf(dts, "S/M Ang.:%7.2lf", indata->eclang);
        decdot(dts);
        qs = dts;
        WorldMap.drawText(300,tht,qs);
        tht += 22;
        if (indata->ecltyp > 0)
         {
          if (indata->maglng > 0)
           {
            sprintf(dts, "Ecl.Pow:%6.2lf", 100.0*indata->eclpow);
            decdot(dts);
            strcat (dts, "%");
           }
          else sprintf(dts, "Ecl.Mag.:%7.3lf", indata->eclmag);
          decdot(dts);
          qs = dts;
          WorldMap.drawText(300,tht,qs);
          strcpy (dts, "Partial Ecl.");
          if (indata->ecltyp == 2) strcpy (dts, "Total Ecl.");
          if (indata->ecltyp == 3) strcpy (dts, "Annular Ecl.");
         }
       else strcpy (dts, "No Eclipse");          
       tht += 22;
       WorldMap.setPen(QColor (255, 0, 0));
      }
     else  // this is actually the standard case
      { 
       strcpy (dts, "R.A.:");
       prpang (ntx, indata->ra4dflg, -2, satps->RA[sx]);
       strcat (dts, ntx);
       qs = dts;
       WorldMap.drawText(300,tht,qs);
       tht += 22;
       strcpy (dts, "Dec.:");
       prpang (ntx, indata->ra4dflg, -1, satps->Dec[sx]);
       strcat (dts, ntx);
       qs = dts;
       WorldMap.drawText(300,tht,qs);
       tht += 22;
       sprintf(dts, "SatSun:%7.1lf", satps->sunang[sx]);
       decdot(dts);
      }; 
    };
   };
  qs = dts;
  WorldMap.drawText(300,tht,qs);

  // darken tracking data if satellite below the horizon
  tht = GBL_MAPHEIGHT + hy + 1;
  spsel = satps->elev[sx];
  spsel = ElvRefr(indata, spsel);
  if ((spsel < 0) && (rlsflg == 0)) WorldMap.setPen(QColor (0, 0, 0));
  else
   {
    WorldMap.setPen(QColor (255, 255, 255));
    if (rlsflg && (cksatok(indata, satps, sx) == 0)) WorldMap.setPen(QColor (0, 0, 0));
   };
  sprintf(dts, "Azim.:%7.2lf", satps->azim[sx]);
  decdot(dts);
  qs = dts;
  WorldMap.drawText(430,tht,qs);
  tht += 22;
  sprintf(dts, "Elev.:%7.2lf", spsel);
  decdot(dts);
  qs = dts;
  WorldMap.drawText(430,tht,qs);
  tht += 22;
  sprintf(dts, "Range:%8.1lf km", satps->range[sx]);
  decdot(dts);
  qs = dts;
  WorldMap.drawText(430,tht,qs);

  // output name of currently selected satellite
  tht = GBL_MAPHEIGHT + hy + 1;
  if (indata->sidxcn == 0)
   {
    WorldMap.setPen(QColor( 255, 0, 0));
    qs = "*** No Satellite ***";
    WorldMap.drawText(590,tht,qs);
   }
  else
   {
    QBrush bs(Qt::SolidPattern);
    if (indata->mapbuf.emagrey != 2) bs.setColor( QColor( 255,255,255 ) );
    else bs.setColor( QColor( 0, 0, 0 ) );
    WorldMap.setBackgroundMode(Qt::OpaqueMode);
    WorldMap.setBackground(bs);
    WorldMap.setPen(satcolor[sx]);
    for (j=0; j<18; j++) dts[j] = ' ';
    strncpy (dts, indata->orbelm.fsname[sx], 15);
    dts[16] = '\0';
    qs = dts;
    WorldMap.drawText(590,tht,qs);
    if (rlsflg)  // target satellite
     {
      sx++;
      if (sx >= indata->sidxcn) sx = 0;
      tht += 22;
      WorldMap.setPen(satcolor[sx]);
      for (j=0; j<18; j++) dts[j] = ' ';
      strncpy (dts, indata->orbelm.fsname[sx], 15);
      dts[16] = '\0';
      qs = dts;
      WorldMap.drawText(590,tht,qs);
     };
     WorldMap.setBackgroundMode(Qt::TransparentMode);
   };

  // mark the satellite position(s) on world map and save old map
  WorldMap.end();
  buffP->fill(QColor(0,0,0));
  WorldMap.begin(buffP);
  WorldMap.drawPixmap(0,0,*buffM);

  f.setPointSize(8);
  f.setBold(false);
  WorldMap.setFont(f);

  for (sx=0; sx < indata->sidxcn; sx++)
   {
    WorldMap.setPen( satcolor[sx] );
    if (indata->skymap)
     {
      skyvis = 1;
      if (indata->visflg)
       {
        if (satps->sunang[sx] < 0)
         {
          skyvis = 0;
          if (indata->moonflg && (satps->moonang[sx] > 0)) skyvis = 1;
         };
       };
     };

    xs = 0; 
    if (indata->occflg || (indata->flarflg && (indata->flarmod > 0)))
     {
      if (smpflg) p = PxCdSun (sx, indata->occflg);
      else
       {
        if (indata->skymap)
         {
          if (skyvis) p = PxSky(satps->elev[sx],satps->azim[sx]);
          else xs = indata->mapwidth + 3;
         }
        else p = PxCoord(satps->occlat[sx], satps->occlng[sx]);
       };
     }
    else
     {
      if (indata->skymap)
       {
        if (skyvis) p = PxSky(satps->elev[sx],satps->azim[sx]);
        else xs = indata->mapwidth+3;
       }
      else p = PxCoord(satps->lat[sx], satps->lng[sx]);
     };

    if (xs == 0)
     { 
      xs = p.x();
      ys = p.y();
     }
    else ys = 0;
     
    lastspx[sx] = xs;
    lastspy[sx] = ys;

    xs -= 2;
    ys -= 2;
    if (xs <= indata->mapwidth)
     {
      if (indata->skymap)
       {
        if (!(indata->satsky && (sx == indata->satidx)))
         { 
          WorldMap.drawRect(xs, ys, tz5, tz5);
          WorldMap.drawPoint(p);
         };
       }   
      else
       {
        WorldMap.drawEllipse(xs, ys, tz5, tz5);
        WorldMap.drawPoint(p);
       };
     };

    // plot satellite visibility circle
    if (rngflg && (indata->occflg == 0) && (indata->flarflg == 0)
                                        && (indata->skymap == 0))
     {
      for (j=0; j<satps->nsatvp; j++)
       {
        p=PxCoord(satps->satvlat[sx][j]*180.0/M_PI,satps->satvlng[sx][j]*180.0/M_PI);
        WorldMap.drawPoint(p);
       };
     };

    // plot satellite names
    if (indata->namflg && (!(indata->satsky && (sx == indata->satidx) && (indata->skymap == 1))))
     { 
      xs = lastspx[sx];
      ys = lastspy[sx];
      xs -= 2;
      ys -= 2;
      strncpy (dts, indata->orbelm.fsname[sx], 15);
      dts[16] = '\0';
      qs = dts;
      if (xs <= indata->mapwidth) WorldMap.drawText(xs, ys, dts);
     }; 

   };  // for sx

  // mark sub-solar point
  if (indata->sidxcn && (smpflg == 0) && (indata->skymap == 0))
   {
    p = PxCoord(satps->sunlat, satps->sunlng);
    xs = p.x();
    ys = p.y();
    xs -= 2;
    ys -= 2;
    WorldMap.setPen (satcolor[4]);
    WorldMap.drawPoint(xs, ys);
    ys += 2;
    WorldMap.drawPoint(xs, ys);
    ys += 2;
    WorldMap.drawPoint(xs, ys);
    xs += 1;
    ys -= 3;
    WorldMap.drawPoint(xs, ys);
    ys += 1;
    WorldMap.drawPoint(xs, ys);
    ys += 1;
    WorldMap.drawPoint(xs, ys);
    xs += 1;
    WorldMap.drawPoint(xs, ys);
    ys += 1;
    WorldMap.drawPoint(xs, ys);
    ys -= 4;
    WorldMap.drawPoint(xs, ys);
    ys += 1;
    WorldMap.drawPoint(xs, ys);
    ys += 1;
    WorldMap.setPen (satcolor[0]);
    WorldMap.drawPoint(xs, ys);      // center
    xs += 1;
    WorldMap.setPen (satcolor[4]);
    WorldMap.drawPoint(xs, ys);
    ys += 1;
    WorldMap.drawPoint(xs, ys);
    ys -= 2;
    WorldMap.drawPoint(xs, ys);
    xs += 1;
    ys -= 1;
    WorldMap.drawPoint(xs, ys);
    ys += 2;
    WorldMap.drawPoint(xs, ys);
    ys += 2;
    WorldMap.drawPoint(xs, ys);
   };	  

  WorldMap.end();

  // mark the satellite positions if a trace is wanted.
  if (indata->mpnight == 0)
   {
    WorldMap.begin(buffM);

    for (sx=0; sx < indata->sidxcn; sx++)
     {
      xs = lastspx[sx];
      ys = lastspy[sx];
      if (xs <= indata->mapwidth)
       {
        WorldMap.setPen (satcolor[sx]);
        WorldMap.drawPoint(xs, ys);
       };
     };  
    WorldMap.end();
   }; // indata->mpnight 

  if (indata->satsky && (indata->skymap == 1))
   {
    indata->geolat = glatsav;
    indata->geolong = glongsav;
    indata->geoheight = gheightsav;
   };

  update();
 }
    
void SatcalView::getMapPos(int x, int y)
 {
  // display local pass details via mouse click
  int sx;
  double lat, lng, ht, az1, az2, el1, el2;
  QPoint p;

  p.setX(x);
  p.setY(y);

  timsuspd = true;

  // save coordinates
  lat = indata->geolat;
  lng = indata->geolong;
  ht = indata->geoheight;
  az1 = indata->az1;
  az2 = indata->az2;
  el1 = indata->el1;
  el2 = indata->el2;

  // get clicked coordinates
  if (indata->skymap)
   {
    if (indata->satsky == 1)
     {
      sx = indata->satidx;  
      indata->geolat = satps->lat[sx];
      indata->geolong = satps->lng[sx];
      indata->geoheight = satps->height[sx] * 1000.0;
     };
    SkyPx(p);
   }
  else
   {
    CoordPx(p, indata->geolat, indata->geolong);
    indata->geoheight = 0;
   };

  // calculate and display local circumstances
  vLocalPass();

  // restore old coordinates
  indata->geolat = lat;
  indata->geolong = lng;
  indata->geoheight = ht;
  indata->az1 = az1;
  indata->az2 = az2;
  indata->el1 = el1;
  indata->el2 = el2;
  timsuspd = false;
 }

void SatcalView::PlotWorld(QPainter &WorldMap)
 {
  // Draw the grid of coordinates and the outlines of continents 
  int i, j, l3;
  int wdt, hgt;
  double dt, lat, lng, dl, ps, bt, c2, s;
  Sun200 sun;
  Vec3 rs, gps, r2;
  QColor tc;
  QPoint p, p0, p2;
  const int tz5 = 4;  // = 5 for Qt3 system 

  // show night part of Earth

  if (indata->mpnight)
   {
    if (indata->mapbuf.emagrey == 0) tc = QColor(223, 223, 223);
    if (indata->mapbuf.emagrey == 1) tc = QColor(150, 150, 150);
    if (indata->mapbuf.emagrey == 2) tc = QColor(65, 65, 65);
    WorldMap.setPen( tc );

    dt = julcent (indata->lastmjd);
    rs = sun.position(dt);
    rs = eclequ(dt,rs);
    rs = vnorm(rs);

    for (j= -360; j<=360; j++)
     {
      lng = 0.5*double(j); // find limiting lat of darkness for this long
      gps = GeoPos (indata->lastmjd, 0, 0, lng*0.0174533,0);
      gps = vnorm(gps);
      r2 = rs;
      r2[2] = 0;
      r2 = vnorm(r2);
      dl = dot(r2,gps);
      if (fabs(dl) <= 1.0) dl = acos(dl);
      else dl = 0;
      ps = fabs(rs[2]);
      if (fabs(ps) <= 1.0) ps = acos(ps);
      else ps = 0;
      bt = sin(ps) * sin(dl);
      bt = asin(bt);
      c2 = cos((dl - bt) * 0.5);
      if (fabs(c2) < 1e-10) c2 = 1.74533e-07;
      c2 = (tan((M_PI*0.5 + ps)*0.5) * cos((dl + bt)*0.5))/c2;
      c2 = 2.0 * atan(c2);
      c2 = 90.0 - 180*c2/M_PI;
      l3 = int(2.0*c2);

      if (rs[2] > 0)
       {
        for (i= -180; i<=l3; i++)
         {
          lat = 0.5*double(i);
          p = PxCoord(lat, lng);
          if (p.x() <= indata->mapwidth) WorldMap.drawPoint(p);
         };
       }
      else
       {
        l3 = -l3;
        for (i= 180; i>=l3; i--)
         {
          lat = 0.5*double(i);
          p = PxCoord(lat, lng);
          if (p.x() <= indata->mapwidth) WorldMap.drawPoint(p);
         };
       };
     }; // for j -360
   };
  // end of night part of Earth

  wdt = GBL_MAPWIDTH;
  hgt = GBL_MAPHEIGHT;

  // draw 10 deg lines
  WorldMap.setPen(QColor( 130, 160, 180 ));

  if ((indata->mapprj == 0) || (indata->mapprj == 3))
   {
    if (indata->mapbuf.emagrid == 2)
     {
      for (j= -170; j<=180; j+=10)
       {
        s = double(j);
        p = PxCoord(90.0,s);
        i = p.x();
        WorldMap.drawLine(i,0,i,hgt);
       };
      for (j= -80; j<90; j+=10)
       {
        s = double(j);
        p = PxCoord(s,-180.0);
        i = p.y();
        WorldMap.drawLine(0,i,wdt,i);
       };
      WorldMap.setPen(QColor( 30, 240, 120 ));
     };

    // draw 30 deg lines
    if (indata->mapbuf.emagrid > 0)
     {
      for (j= -150; j<=180; j+=30)
       {
        s = double(j);
        p = PxCoord(90.0,s);
        i = p.x();
        WorldMap.drawLine(i,0,i,hgt);
       };
      for (j= -60; j<90; j+=30)
       {
        s = double(j);
        p = PxCoord(s,-180.0);
        i = p.y();
        WorldMap.drawLine(0,i,wdt,i);
       };
     };
   }
  else
   {
    // draw 30 deg lines stereographic
    if (indata->mapbuf.emagrid > 0)
     {
      if (indata->mapprj == 1)
       {
        p0 = PxCoord(90.0,0);
        c2 = 10.0;
       } 
      else
       {
        p0 = PxCoord(-90.0,0);
        c2 = -10.0;
       }; 
      for (j= -180; j<180; j+=30)
       {
        s = double(j);
        p = PxCoord(c2,s);
        WorldMap.drawLine(p0,p);
        // the following is to avoid plotting out of the field
        if (indata->mapprj == 1)
         {
          for (i=9; i>= 0; i--)
           {
            p2 = p;
            p = PxCoord(double(i),s);
            if (p.y() < indata->mapwidth) WorldMap.drawLine(p2,p);
           };  
         }
        else
         {
          for (i=-9; i<= 0; i++)
           {
            p2 = p;
            p = PxCoord(double(i),s);
            if (p.y() < indata->mapwidth) WorldMap.drawLine(p2,p);
           };
         };     
       };
  
      if (indata->mapprj == 1) s = 60.0;
      else s = -60.0;
      p0 = PxCoord(s,-180.0);
      for (j= -175; j<=180; j+=5)
       {
        p = PxCoord(s,double(j));
        WorldMap.drawLine(p0,p);
        p0 = p;
       };
      if (indata->mapprj == 1) s = 30.0;
      else s = -30.0;
      p0 = PxCoord(s,-180.0);
      for (j= -175; j<=180; j+=5)
       {
        p = PxCoord(s,double(j));
        WorldMap.drawLine(p0,p);
        p0 = p;
       };
     };
   };        

  if (indata->mapbuf.emagrid == 0)  // right border for map
   {
    wdt = wdt - 1;
    WorldMap.drawLine(wdt,0,wdt,hgt);
   };
   
  // draw the continents
  if ((indata->mapprj < 3) || (!indata->physmap) || (indata->mpnight))
          MapLand(WorldMap);

  WorldMap.setPen( QColor( 0, 255, 255 ) );
  WorldMap.setBrush(Qt::SolidPattern);
  WorldMap.setBrush(QColor(0, 255, 255));
  p = PxCoord(indata->geolat, indata->geolong);
  i = p.x();
  j = p.y();
  i -= 2;
  j -= 2;
  if (p.x() <= indata->mapwidth)
    WorldMap.drawEllipse(i, j, tz5, tz5);
  WorldMap.setBrush(Qt::NoBrush);
 }

void SatcalView::PltSunM(QPainter &WorldMap)
 {
  /* Draw the Sun or Moon */
  int k, j;
  double a, b, c, d, s, r;
  QPoint p, p2;
  QColor tc;

  const double dgrad = M_PI / 180.0;

  if (indata->mapbuf.emagrey != 2) tc = QColor(0,0,0);
  else tc = QColor(255,255,255);
  WorldMap.setPen(tc);

  if ((indata->occflg == 1) || (indata->sidxcn == 0))  // Sun plot
   {
    r = 0.265 * yscale;
    s = -r - 0.5;
    k = int(s);
    k += origy;
    p.setY(k);
    p.setX(origx);

    for (j = 5; j <= 360; j += 5)
     {
      a = j;
      a *= dgrad;
      s = -r * cos(a);
      if (s < 0) s -= 0.5;
      else s += 0.5;
      k = int(s);
      k += origy;
      p2.setY(k);
      s = r * sin(a);
      if (s < 0) s -= 0.5;
      else s += 0.5;
      k = int(s);
      k += origx;
      p2.setX(k);
      WorldMap.drawLine(p, p2);
      p = p2;
     };
   }
  else   // Moon plot
   {
    r = satps->omoonrd * yscale;

    // position angle of terminator
    b = satps->obssun * dgrad;
    c = cos(b);
    a = satps->osunaz - satps->omoonaz;
    s = c * sin(a);
    c = cos(satps->omoonel) * sin(b) - sin(satps->omoonel) * c * cos(a);
    if ((s == 0) && (c == 0)) b = 0;
    else b = atan2 (s, c);
    b -= M_PI * 0.5;

    // plot circumference of moon
    for (j = 0; j <= 360; j += 5)
     {
      a = j;
      a = a * dgrad + b;
      s = -r * cos(a);
      if (s < 0) s -= 0.5;
      else s += 0.5;
      k = int(s);
      k += origy;
      p2.setY(k);
      s = r * sin(a);
      if (s < 0) s -= 0.5;
      else s += 0.5;
      k = int(s);
      k += origx;
      p2.setX(k);
      if (j < 5) p = p2;
      else
       {
        if ((j <= 180) && (satps->moonphs > -0.995)) WorldMap.drawLine(p, p2);
        else
         {
          if (indata->mapbuf.emagrey == 2) WorldMap.setPen(QColor(30,240,120));
          WorldMap.drawPoint(p2);
          WorldMap.setPen(tc);
         };
        p = p2;   
       };
     };

    // plot terminator
    if (satps->moonphs > -0.995)  // not needed at New Moon
     {
      c = cos(b);
      b = sin(b);
      for (j = 0; j <= 180; j += 5)
       {
        a = j;
        a = a * dgrad;
        s = r * cos(a);
        d = -r * sin(a) * satps->moonphs;
        a = d * c + s * b;
        s = d * b - s * c;
        if (s < 0) s -= 0.5;
        else s += 0.5;
        k = int(s);
        k += origy;
        p2.setY(k);
        if (a < 0) a -= 0.5;
        else a += 0.5;
        k = int(a);
        k += origx;
        p2.setX(k);
        if (j >= 5) WorldMap.drawLine(p, p2);
        p = p2;
       };
     };
   };

  // plot position of celestial north
  if ((indata->sidxcn > 0) && (indata->mapbuf.emagrid > 0))
   {
    b = indata->geolat * dgrad;
    if (indata->occflg == 1)
     {
      a = satps->obssun * dgrad;
      c = satps->osunaz;
     }
    else
     {
      a = satps->omoonel;
      c = satps->omoonaz;
     };
    if ((c > M_PI) || (c < 0)) k = 1;
    else k = 0;
    c = sin(a)*sin(b) + cos(a)*cos(b)*cos(c);
    s = 1.0 - c*c;
    if (s < 0) s = 0;
    else s = sqrt(s);  // sin(c)
    s *= cos(a);
    if (fabs(s) > 1e-40)
     {
      s = (sin(b) - sin(a)*c) / s;
      if (fabs(s) <= 1.0) s = -acos(s);
      else s = 0;
      if (k) s = -s;
     }
    else s = 0;

    WorldMap.setPen( QColor( 200, 100, 50 ) );
    p.setX(origx);
    p.setY(origy);
    b = -r*0.35 * cos(s);
    if (b < 0) b -= 0.5;
    else b += 0.5;
    k = int(b);
    k += origy;
    p2.setY(k);
    b = r*0.35 * sin(s);
    if (b < 0) b -= 0.5;
    else b += 0.5;
    k = int(b);
    k += origx;
    p2.setX(k);
    WorldMap.drawLine(p, p2);
    WorldMap.setPen(QColor( 200, 100, 50));
    WorldMap.drawText(p2,"N");
   };
 }

QPoint SatcalView::PxCdSun (int sax, int occflg)
 {
  // get pixel coordinates for the Sun/Moon Map for
  //  satellite index sax from satps
  //  RETURN: the pixel coordinates as a point

  QPoint p(0,0);
  double s, l1, b1;
  int k;

  s = M_PI / 180.0;

  // get projection onto plane as diff between sun and satellite
  if (occflg == 1)
   {
    l1 = satps->osunaz;
    b1 = satps->obssun;
   }
  else
   {
    l1 = satps->omoonaz;
    b1 = satps->omoonel / s;
   };
  l1 /= s;
  if (l1 < 0) l1 += 360.0;
  l1 = satps->azim[sax] - l1;
  if (l1 < -180.0) l1 += 360.0;
  if (l1 > 180.0) l1 -= 360.0;
  l1 = l1 * cos(b1*s);
  b1 = satps->elev[sax] - b1;

  // limit map
  if ((fabs(b1) > 5.0) || (fabs(l1) > 5.0))
   {
    p.setX(0);
    p.setY(-10);

    return p;
   };

  // latitude
  s = -b1 * yscale;
  if (s < 0) s -= 0.5;
  else s += 0.5;
  k = int(s);
  k += origy;
  p.setY(k);

  // longitude
  s = l1 * xscale;
  if (s < 0) s -= 0.5;
  else s += 0.5;
  k = int(s);
  k += origx;
  p.setX(k);

  if (p.y() > GBL_MAPHEIGHT) p.setY(-10);

  return p;
 }
  
QPoint SatcalView::PxCoord (double lat, double lng)
 {
  /* get pixel coordinates for the World Map from the
     geographic latitude lat and the longitude lng
     (both in decimal degrees)

     RETURN: the pixel coordinates as a point
   */
  const double dgrad = M_PI / 180.0;
  QPoint p;
  double x, y, s;
  int k;

  if ((indata->mapprj == 0) || (indata->mapprj == 3))  // normal world map
   {  
    // latitude
    s = -lat * yscale;
    if (s < 0) s -= 0.5;
    else s += 0.5;
    k = int(s);
    k += origy;
    p.setY(k);

    if (indata->centmer != 0)
     {
      lng -= indata->centmer;
      if (lng > 180.0) lng -= 360.0;
      if (lng < -180.0) lng += 360.0;
     }; 

    // longitude
    if (lng > 180.0) lng -= 360.0;
    if (lng < -180.0) lng += 360.0;
    s = lng * xscale;
    if (s < 0) s -= 0.5;
    else s += 0.5;
    k = int(s);
    k += origx;
    p.setX(k);
   }
  else  // stereographic projection
   { 
    if (indata->centmer != 0)
     {
      lng -= indata->centmer;
      if (lng > 180.0) lng -= 360.0;
      if (lng < -180.0) lng += 360.0;
     };

    if (indata->mapprj == 2) // southern hemisphere
     {
      lat = -lat;
      lng = -lng;
     };   

    if ((lat < 0) || (lat >90.1))
     {
      p.setX(indata->mapwidth + 3);
      p.setY(indata->mapheight + 3);
      return p;
     };
    y = lat * dgrad;
    x = lng * dgrad;
    s = 2.0 * cos(y) / (1.0 + sin(y));
    y = s * cos(x);
    x = -s * sin(x);

    // latitude
    s = -y * yscale;
    if (s < 0) s -= 0.5;
    else s += 0.5;
    k = int(s);
    k += origy;
    p.setY(k);

    // longitude
    s = x * xscale;
    if (s < 0) s -= 0.5;
    else s += 0.5;
    k = int(s);
    k += origx;
    p.setX(k);

    if (p.x() > GBL_MAPWIDTH) p.setX(indata->mapwidth + 3);
    if (p.y() > GBL_MAPHEIGHT)
     {
      p.setY(indata->mapwidth + 3);
      p.setX(indata->mapwidth + 3);
     };   
   }; 

  return p;
 }

void SatcalView::CoordPx (QPoint& p, double& lat, double& lng)
 {
  /* get coordinates of geographic latitude lat and
     longitude lng (both in decimal degrees)
     from pixel p of the World Map
  */
  const double dgrad = M_PI / 180.0;
  double s, s2, az1, el1;
  int k;

  // latitude
  k = p.y() - origy;
  s = k;
  lat = -s / yscale;

  // longitude
  k = p.x() - origx;
  s = k;
  lng = s / xscale;

  if ((indata->mapprj == 1) || (indata->mapprj == 2)) // stereographic projection
   {  
    if ((lng == 0) && (lat == 0))
     {
      az1 = 0;
      el1 = 90;
     }
    else
     {
      lng = -lng;
      s = atan2(lng, lat);
      az1 = s / dgrad;
      s2 = sin(s);
      if (fabs(s2) > 1e-5) s = s2;
      else
       {
        s = cos(s);
        lng = lat;
       };
      s = 0.5 * lng / s;
      lat = (s*s - 1.0) / (s*s + 1.0);
      s = s*s / (s*s + 1.0);
      lat = s*s - lat;
      if (lat < 0) el1 = -100.0;
      else
       {
        lng = -s + sqrt(lat);
        lng = asin(lng);
        el1 = lng / dgrad;
        if (el1 < 0) el1 = -100.0;
       };
     };
    lng = az1;
    lat = el1;

    if ((indata->mapprj == 2) && (lat != -100.0))
     {
      lng = -lng;
      lat = -lat; 
     };    
    if (lng > 180.0) lng -= 360.0;
    if (lng < -180.0) lng += 360.0;
   };

  if (indata->centmer != 0)
   {
    lng += indata->centmer;
    if (lng > 180.0) lng -= 360.0;
    if (lng < -180.0) lng += 360.0;
   };
 }

void SatcalView::MapLand(QPainter &WorldMap)
 {
  /* Draw the continents and major islands
     The geographic coordinates are defined in an array geo
     with odd index latitude and the next even indes longitude.
     The coordinates are stored as integers in tens of degrees
     (thus 35.2 degrees will be stored as 352).
     West and South are negative.
   */
  const int gx = 3598;
  int j, k;
  int lon;
  double lat, lng;
  QPoint p1, p2;

  int geo[gx] = {
          // Australia  206
	  1111, 0,
	  -200, 1200,  -200, 1183,  -204, 1180,  -210, 1175,
	  -210, 1175,  -207, 1169,  -218, 1160,  -222, 1145,
	  -226, 1140,  -226, 1137,  -222, 1136,  -227, 1130,
	  -234, 1134,  -241, 1128,  -260, 1140,  -267, 1135,
	  -260, 1128,  -270, 1130,  -265, 1125,  -275, 1140,
	  -280, 1140,  -295, 1150,  -305, 1150,  -325, 1158,
	  -335, 1155,  -337, 1150,  -342, 1150,  -348, 1159,
	  -350, 1165,  -350, 1177,  -337, 1201,  -333, 1224,
	  -335, 1226,  -335, 1238,  -328, 1240,  -322, 1265,
	  -317, 1300,  -315, 1317,  -320, 1338,  -350, 1360,
	  -325, 1375,  -353, 1370,  -351, 1377,  -332, 1376,
	  -348, 1378,  -360, 1367,  -357, 1378,  -375, 1397,
	  -383, 1405,  -388, 1438,  -377, 1448,  -395, 1462,
	  -375, 1477,  -375, 1495,  -372, 1500,  -365, 1500,
	  -350, 1510,  -338, 1523,  -328, 1525,  -300, 1530,
	  -277, 1538,  -266, 1528,  -245, 1527,  -252, 1525,
	  -236, 1509,  -225, 1505,  -225, 1495,  -215, 1487,
	  -182, 1466,  -170, 1460,  -165, 1456,  -150, 1452,
	  -142, 1446,  -145, 1435,  -125, 1430,  -110, 1423,
	  -113, 1420,  -137, 1418,  -150, 1420,  -176, 1410,
	  -176, 1400,  -147, 1355,  -123, 1372,  -120, 1362,
	  -123, 1350,  -117, 1321,  -124, 1325,  -122, 1312,
	  -125, 1306,  -140, 1292,  -150, 1295,  -145, 1277,
	  -133, 1270,  -146, 1253,  -153, 1245,  -166, 1243,
	  -165, 1232,  -178, 1230,  -168, 1225,  -175, 1223,
	  -180, 1220,  -200, 1200,
	  // Tasmania   20
	  1111, 0,
	  -405, 1445,  -440, 1460,  -442, 1470, -438, 1473,
	  -430, 1475,  -425, 1479,  -405, 1475, -415, 1469,
	  -405, 1445,
	  // New Zealand  174
	  1111, 0,    // North Island
	  -344, 1726,  -352, 1731,  -353, 1731,  -369, 1743,
	  -377, 1746,  -381, 1747,  -382, 1745,  -383, 1746,
	  -390, 1744,  -392, 1739,  -394, 1737,  -396, 1740,
	  -397, 1744,  -399, 1747,  -399, 1750,  -404, 1752,
	  -411, 1747,  -413, 1746,  -416, 1748,  -415, 1753,
	  -417, 1753,  -413, 1760,  -406, 1764,  -405, 1766,
	  -396, 1771,  -395, 1768,  -391, 1772,  -391, 1778,
	  -393, 1778,  -391, 1780,  -390, 1779,  -387, 1779,
	  -386, 1782,  -384, 1784,  -380, 1784,  -376, 1786,
	  -375, 1780,  -380, 1773,  -376, 1762,  -365, 1754,
	  -372, 1754,  -372, 1753,  -369, 1752,  -367, 1747,
	  -363, 1748,  -358, 1745,  -358, 1746,  -352, 1743,
	  -350, 1738,  -350, 1734,  -345, 1730,  -344, 1726,
	  1111, 0,     // South Island
	  -405, 1726,  -409, 1721,  -414, 1721,  -418, 1714,
	  -425, 1712,  -430, 1706,  -440, 1687,  -440, 1684,
	  -457, 1665,  -460, 1665,  -463, 1666,  -463, 1676,
	  -466, 1683,  -465, 1687,  -467, 1690,  -465, 1698,
	  -459, 1704,  -458, 1708,  -456, 1706,  -449, 1712,
	  -444, 1712,  -438, 1727,  -438, 1732,  -436, 1730,
	  -436, 1726,  -429, 1734,  -417, 1743,  -414, 1741,
	  -410, 1742,  -407, 1739,  -413, 1732,  -407, 1730,
	  -405, 1726,
	  // Africa   250  (cum 650)
	  1111, 0,
	  364, -55,  360, -60,  340, -70,  332, -85,  318, -97,
	  305, -95,  285, -110,  280, -130,  270, -140,  245, -150,
	  240, -160,  210, -170,  200, -165,  192, -168,  158, -170,
	  142, -175,  134, -170,  122, -165,  80, -130,  73, -125,
	  42, -77,  50, -45,  45, -26,  60, 35,  55, 42,
	  45, 50,  45, 90,  40, 90,  35, 97,  15, 92,
	  8, 95,  2, 92,  -4, 90,  -58, 123,  -62, 125,
	  -92, 130,  -92, 127,  -128, 131,  -140, 125,  -158, 118,
	  -180, 116,  -227, 145,  -245, 145,  -250, 150,  -260, 155,
	  -275, 160,  -285, 165,  -323, 180,  -330, 175,  -345, 185, // cape argulhas
	  -350, 200,  -345, 208,  -343, 215,  -338, 222,  -340, 250,
	  -335, 265,  -311, 302,  -290, 318,  -285, 323,  -255, 330,
	  -245, 360,  -222, 355,  -200, 350,  -175, 370,  -160, 400,
	  -150, 405,  -103, 404,  -76, 386,  -65, 390,  -55, 382,
	  -45, 390,  -44, 396,  -25, 402,  -25, 408,  20, 445,
	  25, 460,  50, 482,  91, 506,  118, 510,  106, 440,
	  112, 425,  116, 420,  120, 428,  130, 422,  152, 392,
	  176, 383,  183, 374,  223, 370,  231, 352,  240, 355,
	  297, 322,   1111, 0,  // suez  (186)
	  318, 320,  322, 307,  310, 290,  318, 258,  316, 248,
	  320, 248,  325, 227,  328, 225,  331, 220,  320, 200,
	  306, 200,  302, 190,  312, 170,  317, 155,  322, 152,
	  330, 136,  328, 120,  340, 100,  352, 112,  360, 105,
	  370, 110,  368, 104,  372, 100,  370, 88,  372, 62,
	  367, 52,  370, 40,  365, 20,  350, -25,  350, -38,
	  355, -55, 364, -55,    // (250)
	  // Madagascar  48  (cum 698)
	  1111, 0,
	  -200, 445,  -217, 440,  -225, 435,  -234, 438,  -250, 440,
	  -257, 445,  -263, 450,  -255, 472,  -177, 492,  -172, 490,
	  -160, 496,  -155, 496,  -158, 504,  -152, 507,  -128, 500,
	  -120, 490,  -123, 485,  -132, 480,  -137, 475,  -150, 468,
	  -160, 440,  -170, 435,  -200, 445,
	  // Lake Victoria  16
	  1111, 0,
	  0, 330,  -15, 325,  -35, 328,  -35, 340,  -5, 345,
	  3, 340,  0, 330,
	  // Lake Tanganyika  16
	  1111, 0,
	  -40, 292,  -55, 292,  -75, 306,  -80, 315,  -60, 300,
	  -50, 297,  -40, 292,
	  // Lake Malawi   18  (cum 748)
	  1111, 0,
	  -96, 340,  -148, 346,  -145, 350, -123, 347,  -120, 344,
	  -117, 350,  -100, 345, -96, 340,
	  // South America  340  (cum 1088)
	  1111, 0,
	  90, -796,  85, -790,  77, -775,  72, -772,  60, -772,
	  58, -775,  42, -772,  40, -770,  28, -777,  25, -780,
	  20, -777,  12, -805,  -5, -810,  -10, -810,  -10, -816,
	  -24, -815,  -28, -804,  -25, -795,  -30, -800,  -44, -820,
	  -60, -820,  -70, -800,  -134, -763,  -142, -763,  -152, -752,
	  -174, -715,  -185, -705,  -215, -701,  -232, -705,  -232, -708,
	  -235, -708,  -235, -705,  -245, -708,  -254, -708,  -257, -710,
	  -270, -710,  -270, -713,  -275, -710,  -285, -715,  -293, -715,
	  -300, -715,  -303, -717,  -325, -715,  -370, -732,  -372, -735,
	  -395, -733,  -400, -737,  -410, -740,  -417, -736,  -418, -740,
	  // isla chiloe
	  -434, -745,  -435, -736,  -418, -732,  -420, -725,  -438, -732,
	  -437, -743,  -455, -745,  -468, -755,  -470, -740,  -480, -755,
	  -508, -755,  -505, -750,  -515, -754,  -513, -742,  -520, -750,
	  -530, -730,  -528, -748,  -540, -735,  -545, -720,  -552, -707,
	  -550, -700,  -553, -702,  -558, -681,  -552, -685,  -553, -672,
	  -550, -670,  -549, -655,  -546, -650,  -546, -660,  -540, -675,
	  -526, -687,  -527, -695,  -522, -693,  -523, -685,  -510, -692,
	  -505, -690,  -500, -678,  -492, -675,  -480, -658,  -470, -658,
	  -470, -669,  -460, -675,  -452, -670,  -450, -655,  -435, -652,
	  -427, -635,  -421, -635,  -422, -650,  -407, -650,  -410, -628,
	  -406, -623,  -389, -624,  -389, -590,  -380, -576,  -364, -567,
	  -360, -570,  -352, -570,  -345, -588,  -342, -585,  -351, -562,
	  -352, -548,  -345, -537,  -286, -488,  -274, -484,  -272, -486,
	  -258, -485,  -240, -463,  -230, -430,  -230, -420,  -224, -416,
	  -215, -410,  -192, -394,  -160, -382,  -130, -382,  -130, -373,
	  -70, -350,  -53, -354,  -50, -370,  -27, -400,  -28, -418,
	  -24, -432,  -28, -440,  -20, -447,  -8, -475,  -16, -482,
	  -5, -477,  0, -500,  22, -500,  41, -515,  55, -526,  // Kourou
	  68, -550,  65, -573,  80, -600,  80, -610,  90, -615,
	  100, -620,  115, -614,  110, -640,  104, -648,  108, -660,
	  108, -677,  118, -677,  120, -685,  120, -693, 125, -693,
	  125, -705,  120, -705,  115, -720,  120, -723,  125, -717,
	  120, -727, 120, -740, 115, -740,  115, -750, 110, -758,
	  90, -760,  80, -775,  85, -780,  90, -790,  // Portobello Panama
	  // Antarctica   174  (cum 1262)
	  1111, 0,
	  -778, 1800,  -770, 1785,  -770, 1700,  -770, 1660,  -775, 1650,
	  -754, 1640,  -740, 1655,  -725, 1700,  -720, 1700,  -710, 1650,
	  -700, 1600,  -677, 1545,  -680, 1520,  -670, 1440,  -665, 1355,
	  -665, 1300,  -670, 1300,  -670, 1245,  -675, 1180,  -660, 1135,
	  -670, 1090,  -662, 1010,  -667, 1000,  -667, 900,  -675, 815,
	  -678, 780,  -694, 750,  -680, 700,  -676, 697,  -674, 600,
	  -662, 550,  -667, 500,  -675, 500,  -668, 485,  -675, 470,
	  -675, 455,  -687, 400,  -700, 390,  -680, 345,  -690, 330,
	  -705, 245,  -700, 135,  -700, 0,  -710, -90,  -718, -130,
	  -722, -125, -730, -190,  -740, -210,  -745, -250,  -758, -250,
	  -774, -320,  -776, -350,  -782, -355,  -780, -450,  -750, -610,
	  -728, -600,  -700, -615,  -667, -610,  -640, -560,  -630, -570,
	  -645, -635,  -640, -640,  -648, -640,  -660, -660,  -670, -670,
	  -698, -683,  -682, -705,  -690, -727,  -720, -750,  -728, -730,
	  -725, -790,  -738, -820,  -724, -950,  -720, -955,  -724, -1040,
	  -750, -1015,  -740, -1125,  -730, -1235,  -727, -1270,  -740, -1280,
	  -750, -1390,  -768, -1500,  -774, -1540,  -770, -1580,  -780, -1640,
	  -778, -1800,
	  // Europe West and South  [238]  (cum 1490)
	  1111, 0,          //  Atlantic and Mediterranean coast 132
	  540, 80,  530, 50,  525, 40,  515, 20,  502, 22,
	  492, -4,  497, -22,  480, -20,  480, -45,  475, -45,
	  470, -25,  465, -20,  435, -20,  436, -78,  430, -84,
	  416, -80,  380, -91,  377,  -85,  373, -85,  373, -75,
	  367, -58,  368, -52,  370, -45,  370, -23,  380, 0,
	  390, -5,  417, 17,  422, 30,  428, 30,  430, 50,
	  427, 65,  442, 85,  438, 103,  426, 106,  400, 150,
	  380, 160,  376, 155,  376, 161,  395, 170,  405, 173,
	  400, 176,  412, 170,   422, 167,  425, 140,  430, 130,
	  438, 125,  456, 125,  458, 130,  450, 135,  455, 140,
	  435, 155,  420, 190,  405, 190,  377, 210,  375, 210,
	  370, 218,  365, 227,  375, 225,  375, 230,  377, 240,
	  405, 225,  402, 230,  412, 250,  410, 268,  400, 264,  // Agean
	  1111, 0,         // Black Sea  48
	  410, 268,  415, 283,  415, 290,  418, 290,  422, 280,
	  427, 275,  465, 306,  460, 330,  455, 325,  444, 337,
	  455, 365,  460, 350,  470, 380,  455, 373,  450, 368,
	  425, 415,  420, 415,  413, 400,  416, 370,  422, 350,
	  421, 327,  415, 310,  415, 290,
	  1111, 0,        // Asia Minor  26
	  415, 283,  410, 295,  404, 264,  367, 273,  362, 300,
	  365, 307,  360, 325,  370, 358,  350, 355,  317, 345,
	  315, 330,  318, 320,
	  1111, 0,        // Caspian Sea   22
	  450, 470,  400, 500,  374, 490,  370, 540,  400, 540,
	  450, 505,  455, 525,  470, 525,  470, 500,  450, 470,
	  //  Europe North   160  (cum  1650)
	  1111, 0,   // 160
	  540, 80,  555, 80,  555, 75,  570, 76,  574, 105,
	  570, 102,  570, 106,  560, 100,  558, 107,  550, 100,
	  540, 110,  545, 135,  535, 145,  548, 178,  540, 178,
	  550, 200,  550, 210,  570, 210,  575, 222,  570, 226,
	  578, 240,  575, 220,  580, 220,  590, 230,  600, 285,
	  605, 280,  600, 223,  632, 220,  650, 250,  660, 240,
	  660, 220,  645, 220,  625, 175,  610, 175,  600, 182,
	  580, 170,  565, 160,  565, 145,  555, 145,  555, 128,
	  590, 107,  577, 75,  577, 66,  580, 55,  582, 60,
	  585, 50,  622, 50,  640, 100,  678, 158,  670, 125,
	  690, 160,  700, 180,  705, 220,  712, 220,  717, 255,
	  700, 250,  715, 275,  705, 310,  700, 295,  677, 405,
	  670, 410,  667, 404,  664, 377,  670, 340,  663, 350,
	  640, 350,  630, 370,  650, 365,  645, 400,  660, 400,
	  667, 423,  665, 440,  680, 440,  678, 456,  673, 465,
	  675, 450,  670, 456,  670, 472,  675, 476,   // indiga
	  //  Sjaeland  12
	  1111, 0,
	  565, 113,  548, 110,  545, 120, 566, 120, 565, 113,
	  //  Scicily   10
	  1111, 0,
	  375, 123,  365, 150,  376, 155,  375, 123,
	  //  Sardinia   14
	  1111, 0,
	  415, 76,  384, 77,  386, 95,  406, 95,  412, 90,
	  415, 76,
	  //  Corsica   12
	  1111, 0,
	  427, 90,  425, 78,  420, 80,  418, 85,  427, 90,
	  //  Cyprus  14
	  1111, 0,
	  352, 322,  346, 325,  350, 335,  357, 340,  355, 325,
	  352, 322,
	  //  Crete   12
	  1111, 0,
	  355, 230,  350, 265,  355, 265,  360, 230,  355, 230,
	  //  England   60   (cum 1784)
	  1111, 0,
	  500, -60,  505, -40,  510, -40,  510, 0,  515, 15,
	  518, 10,  527, 20,  525, 0,  528, 4,  565, -24,
	  565, -28,  577, -20,  576, -40,  580, -26,  580, -50,
	  575, -70,  576, -60,  573, -65,  560, -46,  548, -50,
	  550, -27,  542, -30,  530, -27,  530, -45,  525, -40,
	  520, -50,  520, -27,  516, -40,  500, -60,
	  //  Ireland   28
	  1111, 0,
	  540, -100,  530, -100,  530, -85,  522, -106,  518, -80,
	  524, -70,  527, -67,  532, -70,  540, -58,  555, -61,
	  555, -75,  542, -78,  540, -100,
	  //  Iceland  36   (cum 1848)
	  1111, 0,
	  636, -230,  630, -180,  640, -145,  658, -140,  667, -160,
	  660, -177,  665, -185,  660, -190,  665, -200,  654, -220,
	  665, -230,  657, -245,  655, -225,  650, -240,  645, -227,
	  640, -225, 636, -230,
	  //  Arabia  44
	  1111, 0,
	  297, 322,  275, 340,  290, 350,  278, 348,  170, 420,
	  125, 430,  122, 440,  160, 520,  165, 524,  170, 550,
	  190, 585,  225, 595,  250, 560,  240, 540,  240, 515,
	  295, 490,  300, 500,  260, 540,  270, 560,  250, 575,
	  250, 662,
	  //  South Asia  194  (cum 2086)
	  240, 675,  225, 700,  220, 690,  208, 705,  218, 725,
	  186, 725,  160, 735,  80, 775,  94, 785,  102, 790,
	  102, 797,  120, 797,  136, 805,  150, 800,  165, 810,
	  167, 823,  172, 823,  176, 830,  195, 853,  200, 862,
	  210, 870,  216, 868,  221, 920,  203, 926,  200, 930,
	  195, 937,  193, 933,  183, 940,  175, 945,  160, 940,
	  158, 954,  170, 970,  125, 984,  117, 980,  113, 985,
	  80, 980,  65, 1005,  45, 1005,  15, 1040,  15, 1045,
	  30, 1040,  60, 1040,  80, 1000,  90, 1000,  90, 990,
	  138, 1000,  138, 1010,  130, 1010,  130, 1020,  120, 1030,
	  108, 1030,  100, 1050,  87, 1050,  112, 1090,  155, 1085,
	  190, 1060,  205, 1070,  210, 1095,  200, 1100,  193, 1085,
	  180, 1085,  180, 1100,  200, 1105,  206, 1100,  215, 1135,
	  220, 1135,  215, 1140,  227, 1166,  250, 1195,  260, 1195,
	  278, 1207,  300, 1224,  302, 1205,  310, 1217,  345, 1202,
	  350, 1193,  365, 1200,  375, 1230,  375, 1205,  370, 1190,
	  375, 1190,  375, 1175,  390, 1175,  390, 1190,  407, 1215,
	  395, 1215,  400, 1240,  395, 1250,  378, 1247,  375, 1265,
	  344, 1265,  350, 1295,  375, 1290,  400, 1275,  407, 1300,
	  420, 1300,  427, 1323,       //  Vladivostok
	  //  Russian Pacific coastline   110  (cum 2196)
	  423, 1330,  430, 1345,  478, 1400,  523, 1410,  525, 1420,
	  530, 1415,  532, 1420,  542, 1395,  542, 1380,  535, 1380,
	  540, 1380,  535, 1370,  540, 1370,  548, 1352,  592, 1455,
	  595, 1500,  590, 1533,  584, 1520,  590, 1550,  598, 1545,
	  610, 1557,  620, 1570,  622, 1600,  608, 1600,  620, 1630,
	  625, 1630,  625, 1645,  610, 1632,  604, 1620,  575, 1575,
	  575, 1565,  570, 1565,  560, 1555,  510, 1567,  515, 1575,
	  520, 1580,  525, 1580,  525, 1600,  540, 1600,  545, 1623,
	  565, 1620,  565, 1630,  575, 1629,  575, 1624,  600, 1630,
	  606, 1660,  600, 1660,  606, 1670,  606, 1690,  600, 1700,
	  625, 1772,  623, 1792,  645, 1775,  645, 1780,  650, 1800,
	  //  Artic Asian coastline   160  (cum 2356)
	  1111, 0,
	  675, 476,  685, 545,  680, 540,  685, 590,  680, 600,
	  685, 610,  705, 580,  707, 590,  698, 605,  697, 640,
	  678, 670,  685, 680,  693, 670,  720, 670,  730, 695,
	  730, 725,  665, 725,  665, 740,  720, 740,  725, 750,
	  715, 750,  715, 770,  720, 760,  723, 780,  720, 825,
	  725, 810,  735, 810,  740, 860,  750, 805,  765, 950,
	  760, 970,  770, 1010,  775, 1010,  780, 1045,  770, 1060,
	  773, 1110,  760, 1140,  735, 1060,  740, 1130,  740, 1190,
	  735, 1190,  730, 1230,  737, 1230,  735, 1280,  715, 1290,
	  710, 1315,  720, 1325,  715, 1330,  720, 1400,  725, 1390,
	  727, 1410,  720, 1500,  715, 1520,  715, 1575,  705, 1600,
	  695, 1595,  695, 1670,  700, 1680,  685, 1690,  685, 1710,
	  700, 1705,  698, 1760,  682, 1800,
	  1111, 0,
	  682, -1800,  670, -1750,  665, -1745,  670, -1745,  670, -1720,
	  665, -1700,  660, -1710,  660, -1725,  640, -1725,  650, -1758,
	  660, -1760,  660, -1780,  665, -1780,  665, -1795,  650, -1800,
	  //  Lake Baykal    18
	  1111, 0,
	  520, 1030,  520, 1050,  555, 1090,  555, 1098,  535, 1090,
	  520, 1070,  515, 1040,  520, 1030,
	  //  Sakhalin       20
	  1111, 0,
	  535, 1420,  460, 1425,  465, 1430,  460, 1435,  475, 1430,
	  490, 1430,  490, 1440,  544, 1425,  535, 1420,
	  //  Ostrov Kotel'ny   18
	  1111, 0,
	  765, 1380,  753, 1375,  745, 1390,  750, 1440,  705, 1450,
	  767, 1410,  760, 1410,  765, 1380,
	  //  Svernaya Semlya    22
	  1111, 0,
	  810, 950,  800, 910,  785, 950,  785, 1000,  776, 1000,
	  780, 1050,  790, 1020,  785, 1000,  800, 1000,  810, 950,
	  //  Novaya Semlya     24   (cum 2458)
	  1111, 0,
	  770, 680,  765, 600,  750, 560,  720, 520,  710, 540,
	  710, 575,  720, 555,  725, 560,  750, 600,  765, 680,
	  770, 680,
	  //  Ceylon      14
	  1111, 0,
	  96, 800,  65, 800,  60, 810,  65, 820,  90, 810,  96, 800,
	  //  Taiwan    14
	  1111, 0,
	  235, 1200,  220, 1205,  220, 1210,  245, 1220,  240, 1215,
	  235, 1200,
	  //  Japan    76   (cum 2562)
	  1111, 0,    // Kyushu
	  340, 1308,  338, 1295,  308, 1300,  310, 1315,  330, 1320,
	  340, 1308,
	  1111, 0,    // Honshu
	  340, 1308,  345, 1350,  340, 1346,  335, 1353,  345, 1360,
	  350, 1405,  395, 1415,  410, 1410,  405, 1400,  390, 1395,
	  380, 1390,  370, 1360,  350, 1355,  350, 1330,  340, 1308,
	  1111, 0,    // Hokkaido
	  415, 1410,  410, 1400,  420, 1400,  425, 1415,  452, 1420,
	  430, 1450,  415, 1430,  420, 1410, 415, 1410,
	  1111, 0,    // Shikoku
	  335, 1325,  330, 1330,  335, 1340,  335, 1325,
	  //  Sumatra    26
	  1111, 0,
	  55, 950,  15, 990,  2, 990,  -60, 1040,  -60, 1050,
	  -30, 1050,  -28, 1045,  -10, 1045,  -10, 1035,  2, 1035,
	  50, 970,  55, 950,
	  //  Java       16
	  1111, 0,
	  -60, 1055,  -70, 1050,  -85, 1140,  -80, 1140,  -70, 1105,
	  -75, 1103,  -60, 1055,
	  //  Borneo    28
	  1111, 0,
	  15, 1090,  0, 1090,  -35, 1100,  -45, 1145,  -40, 1155,
	  5, 1180,  5, 1190,  40, 1180,  50, 1190,  68, 1170,
	  25, 1110,  15, 1110,  15, 1090,
	  //  Celebes    40
	  1111, 0,
	  7, 1200,  -40, 1190,  -40, 1195,  -60, 1195,  -60, 1204,
	  -35, 1205,  -35, 1210,  -50, 1215,  -50, 1220,  -20, 1210,
	  -5, 1230,  -15, 1210,  0, 1200,  5, 1205,  5, 1240,
	  15, 1250,  10, 1240,  10, 1207,  7, 1200,
	  //  New Guinea   52  (cum 2724)
	  1111, 0,
	  -10, 1310,  -25, 1338,  -30, 1320,  -40, 1330,  -40, 1335,
	  -35, 1340,  -60, 1370,  -75, 1375,  -80, 1367,  -80, 1400,
	  -90, 1404,  -90, 1430,  -70, 1435,  -70, 1455,  -100, 1470,
	  -105, 1504,  -60, 1465,  -60, 1470,  -50, 1450,  -45, 1450,
	  -18, 1380,  -35, 1350,  -8, 1340,  -2, 1320,  -10, 1310,
	  //  Timor    14
	  1111, 0,
	  -100, 1240,  -100, 1245,  -85, 1270,  -85, 1250,  -95, 1240,
	  -100, 1240,
	  //  Philippines   66   (cum 2804)
	  1111, 0,
	  190, 1204,  160, 1204,  162, 1200,  123, 1210,  140, 1215,
	  130, 1240,  140, 1240,  150, 1220,  155, 1215,  175, 1225,
	  190, 1225,  190, 1204,
	  1111, 0,
	  117, 1220,  103, 1220,  105, 1225,  100, 1220,  90, 1230,
	  110, 1230,  117, 1220,
	  1111, 0,
	  98, 1260,  80, 1240,  85, 1240,  75, 1220,  80, 1240,
	  60, 1240,  55, 1255,  70, 1255,  60, 1260,  83, 1265,
	  98, 1260,
	  //  Hawaii       34  (cum 2838)
	  1111, 0,
	  202, -1559,  197, -1561,  189, -1556,  195, -1548,  202, -1559,
	  1111, 0,
	  210, -1566,  206, -1554,  207, -1560,  210, -1556,
	  1111, 0,
	  216, -1583,  213, -1582,  212, -1577,  217, -1580,  216, -1583,
	  // Puerto Rico  14
	  1111, 0,
	  185, -672,  179, -672,  180, -658,  182, -656,  184, -656,
	  185, -672,
	  // Haiti    28
	  1111, 0,
	  185, -740,  180, -736,  182, -720,  176, -715,  183, -705,
	  182, -703,  184, -686,  186, -686,  197, -705,  194, -716,
	  197, -734,  185, -726,  185, -740,
	  // Cuba  24
	  1111, 0,
	  197, -776,  200, -740,  230, -797,  230, -825,  225, -845,
	  212, -845,  217, -820,  210, -790,  205, -780,  202, -766,
	  197, -776,
	  // Jamaica    12   (cum 2916)
	  1111, 0,
	  182, -785,  175, -770,  180, -765,  185, -780,  182, -785,
	  //  North America  326
	  1111, 0,
	  90, -796,  80, -800,  80, -820,  85, -830,  82, -845,
	  100, -852,  95, -850,  110, -860,  130, -870,  145, -920,
	  160, -955,  154, -965,  200, -1057,  215, -1050,  308, -1135,
	  310, -1140,  300, -1140,  235, -1100,  270, -1150,  270, -1145,
	  330, -1180,  335, -1200,  375, -1230,  375, -1225,  380, -1230,
	  403, -1250,  470, -1250,  480, -1255,  505, -1285,  515, -1285,
	  540, -1300,  545, -1340,  600, -1400,  600, -1450,  606, -1465,
	  606, -1496,  600, -1470,  594, -1525,  600, -1525,  582, -1520,
	  577, -1525,  570, -1520,  565, -1550,  570, -1555,  550, -1600,
	  535, -1655,  552, -1640,  575, -1580,  582, -1580,  582, -1620,
	  600, -1620,  600, -1650,  620, -1665,  640, -1650,  645, -1618,
	  645, -1665,  655, -1690,  667, -1640,  662, -1640,  662, -1630,
	  670, -1630,  687, -1665,  690, -1640,  700, -1630,  715, -1570,
	  700, -1460,  700, -1420,  685, -1355,  695, -1350,  702, -1290,
	  692, -1260,  700, -1250,  692, -1245,  697, -1235,  680, -1150,
	  675, -1155,  675, -1100,  667, -1080,  685, -1080,  675, -970,
	  684, -935,  700, -960,  745, -960,  745, -900,  730, -945,
	  685, -900,  690, -895,  670, -870,  670, -865,  700, -860,
	  710, -900,  740, -900,  740, -785,  700, -680,  695, -700,
	  667, -610,  640, -640,  667, -680,  630, -650,  620, -655,
	  650, -780,  670, -720,  700, -780,  700, -820,  670, -810,
	  660, -860,  640, -800,  640, -870,  650, -880,  610, -940,
	  570, -930,  545, -825,  513, -800,  520, -790,  545, -800,
	  555, -770,  630, -780,  630, -740,  610, -700,  588, -700,
	  588, -660,  600, -650,  520, -570,  500, -570,  470, -520,
	  480, -590,  515, -575,  500, -600,  500, -670,  460, -640,
	  450, -610,  437, -655,  450, -645,  430, -705,  410, -700,
	  402, -740,  360, -760,  350, -755,  310, -810,  300, -810,
	  280, -800,  250, -802,  250, -810,  282, -835,  300, -833,
	  300, -900,  290, -900,  290, -920,  298, -935,  290, -970,
	  260, -970,  210, -980,  195, -965,  195, -910,  210, -900,
	  212, -875,  160, -885,  165, -850,  150, -840,  100, -840,
	  95, -820,  90, -790,
	  //  Vancouver Island / Puget Sound     10
	  1111, 0,
	  485, -1255,  480, -1240,  500, -1250,  505, -1285,
	  //  Cook Inlet    6
	  1111, 0,
	  600, -1525,  628, -1500,
	  //  Baffin Island Sounds   28  (cum 3286)
	  1111, 0,
	  700, -860,  700, -820,
	  1111, 0,
	  740, -850,  720, -850, 710, -845,
	  1111, 0,
	  740, -803,  730, -803,  730, -760,
	  1111, 0,
	  630, -698,  625, -655,
	  //  Great Lakes   76  (cum 3362)
	  1111, 0,
	  468, -920,  490, -880,  490, -850,  465, -845,  468, -920,
	  1111, 0,
	  459, -870,  430, -880,  415, -875,  420, -865,  440, -865,
	  460, -850, 459, -870,
	  1111, 0,
	  457, -847,  455, -835,  435, -835,  440, -830,  430, -825,
	  432, -818,  450, -815,  445, -800,  460, -810,  462, -838,
	  457, -847,
	  1111, 0,
	  420, -835,  415, -838,  413, -820,  427, -790,  420, -835,
	  1111, 0,
	  432, -798,  435, -763,  440, -765,  438, -790,  432, -798,
	  //  Greenland   78  (cum 3440)
	  1111, 0,
	  765, -700,  757, -590,  720, -560,  715, -520,  710, -520,
	  715, -540,  690, -535,  695, -510,  677, -510,  675, -530,
	  650, -530,  625, -495,  600, -430,  650, -400,  670, -330,
	  680, -250,  700, -220,  705, -250,  715, -250,  705, -230,
	  705, -210,  730, -220,  735, -200,  795, -200,  795, -180,
	  820, -102,  820, -300,  825, -200,  840, -350,  825, -500,
	  820, -600,  800, -700,  800, -650,  790, -660,  780, -730,
	  775, -680,  770, -705,  765, -700,
	  //   Ellesmere Island   58
	  1111, 0,
	  800, -970,  820, -940,  830, -750,  830, -700,  825, -610,
	  795, -700,  765, -770,  760, -900,  755, -895,  760, -810,
	  745, -800,  745, -920,  770, -950,  770, -900,  775, -880,
	  770, -850,  780, -880,  775, -910,  800, -970,
	  1111, 0,
	  820, -920,  810, -880,  810, -840,  815, -800,
	  1111, 0,
	  780, -870,  785, -850,  805, -880,
	  //   Melville Island    32
	  1111, 0,
	  760, -1220,  745, -1130,  750, -1060,  765, -1070,  765, -1080,
	  770, -1100,  755, -1090,  755, -1110,  770, -1165,  775, -1175,
	  760, -1220,
	  1111, 0,
	  755, -1180,  770, -1165,  760, -1200,
	  //   Victoria Island   68   (cum 3598)
	  1111, 0,
	  720, -1260,  710, -1235,  715, -1205,  723, -1200,  725, -1180,
	  720, -1180,  720, -1160,  715, -1180,  703, -1110,  700, -1170,
	  690, -1140,  675, -1140,  680, -1090,  690, -1070,  680, -1020,
	  695, -1020,  695, -1008,  715, -1050,  740, -1050,  740, -1070,
	  720, -1070,  720, -1075,  730, -1100,  725, -1100,  735, -1150,
	  740, -1180,  737, -1200,  745, -1215,  745, -1255,  720, -1260,
	  1111, 0,
	  725, -1190,  735, -1150,
	  };

  WorldMap.setPen(QColor( 200, 100, 50 ));
  k = gx;
  lon = 0;
  for (j=0; j<k; j+=2)
   {
    if (geo[j] == 1111) lon = 0;
    else
     {
      lat = double(geo[j]) * 0.1;
      lng = double(geo[j+1]) * 0.1;
      if (!lon) p1 = PxCoord(lat,lng);
      else
       {
        p2 = PxCoord(lat,lng);
        if (abs((p1.x() - p2.x())) < 200)
        WorldMap.drawLine(p1, p2);
        p1 = p2;
       };
      lon = 1;
     };
   };

  // mark equator for stereographic projection
  if ((indata->mapprj == 1) || (indata->mapprj == 2))
   {
    WorldMap.setPen(QColor( 130, 160, 180 ));
    lat = 0;
    k = 0;
    p1 = PxCoord(0, 0.0001);
    for (j=0; j<360; j++)
     {
      lng = double(j);
      p2 = PxCoord(lat, lng);
      if ((p2.x() > indata->mapwidth)
               || (p2.y() > indata->mapheight)) k = 0;
      else
       {
        if (k) WorldMap.drawLine(p1, p2);
        k = 1;
        p1 = p2;
       };
     };
   };  // mapprj
 }

QPoint SatcalView::PxSky (double elev, double azim)
 {
  // get pixel coordinates for the Sky Map from the
  //  elevation elev and the azimuth
  //  (both in decimal degrees)
  //  RETURN: the pixel coordinates as a point
  const double dgrad = M_PI / 180.0;
  QPoint p;
  double x, y, s, horz;
  int k, j;

  horz = 0.0;
  if (indata->satsky)
   {
    if (indata->geoheight > 0)
      horz = -acos(6378137.0 / (6378137.0 + indata->geoheight)) / dgrad;
   };

  if (elev < horz)
   {
    p.setX(indata->mapwidth + 3);
    p.setY(indata->mapheight + 3);
    return p;
   };

  y = elev * dgrad;

  if (indata->skyprj)  // Mollweide projection
   {
    if (indata->satsky && (elev > (90.0 + horz)))
     {
      p.setX(indata->mapwidth + 3);
      p.setY(indata->mapheight + 3);
      return p;
     };

    y = y - horz*dgrad;
    x = (azim - indata->skazimuth) * dgrad;
    x = atan2(sin(x), cos(x));
    s = y;
    if (y > 1.22173) k = 8;
    else k = 3;
    y = sin(y);
    for (j=0; j<k; j++)
     {
      s = s - (sin(s) + s - M_PI * y) / (cos(s) + 1.0);
     };
    x = 0.900316 * x * cos(0.5 * s);
    y = 1.41421 * sin(0.5 * s);
   }
  else                // Stereographic projection
   {
    x = azim * dgrad;
    s = 2.0 * cos(y) / (1.0 + sin(y));
    y = s * cos(x);
    x = -s * sin(x);
   };

  // elevation
  s = -y * yscale;
  if (s < 0) s -= 0.5;
  else s += 0.5;
  k = int(s);
  k += origy;
  p.setY(k);

  // azimuth
  s = x * xscale;
  if (s < 0) s -= 0.5;
  else s += 0.5;
  k = int(s);
  k += origx;
  p.setX(k);

  if (p.y() > GBL_MAPHEIGHT) p.setY(indata->mapwidth + 3);
  if (p.x() > GBL_MAPWIDTH) p.setX(indata->mapwidth + 3);

  return p;
 }

void SatcalView::SkyPx (QPoint& p)
 {
  // get elevation, azimuth, Right Ascension and Declination
  //  from pixel p of the Sky Map and place them
  //  indata->el1, indata->az1, indata->el2, indata->az2
  //  respectively in degrees and in HH.MM and DD.MM.
  //  A point outside the map will be marked as indata->el1 = -100.
  const double dgrad = M_PI / 180.0;
  double s, s2, lat, lng, dt, horz;
  int k;
  Vec3 r1;
  Mat3 pmx;

  horz = 0.0;
  if (indata->satsky)
   {
    if (indata->geoheight > 0)
      horz = -acos(6378137.0 / (6378137.0 + indata->geoheight)) / dgrad;
   };

  // prelim. latitude
  k = p.y() - origy;
  s = k;
  lat = -s / yscale;

  // prelim. longitude
  k = p.x() - origx;
  s = k;
  lng = s / xscale;

  // get elevation and azimuth
  if (indata->skyprj)  // Mollweide
   {
    if (lat < 0) indata->el1 = -100.0;
    else
     {
      s = asin(lat*0.707107);
      indata->el1 = asin((sin(2*s) + 2*s)/M_PI);
      indata->el1 /= dgrad;
      indata->el1 += horz;
      s = cos(s);
      if (s >= 0)
       {
        lng = lng * 1.11072 / s;
        lng += indata->skazimuth*dgrad;
        lng = atan2(sin(lng),cos(lng));
        indata->az1 = lng / dgrad;
        if (indata->az1 < 0) indata->az1 += 360.0;
       }
      else indata->el1 = -100.0;
     };
   }
  else           // Stereographic
   {
    if ((lng == 0) && (lat == 0))
     {
      indata->az1 = 0;
      indata->el1 = 90;
     }
    else
     {
      lng = -lng;
      s = atan2(lng, lat);
      indata->az1 = s / dgrad;
      if (indata->az1 < 0) indata->az1 += 360.0;
      s2 = sin(s);
      indata->sdel1 = s;
      if (fabs(s2) > 1e-5) s = s2;
      else
       {
        s = cos(s);
        lng = lat;
       };
      s = 0.5 * lng / s;
      lat = (s*s - 1.0) / (s*s + 1.0);
      s = s*s / (s*s + 1.0);
      lat = s*s - lat;
      if (lat < 0) indata->el1 = -100.0;
      else
       {
        lng = -s + sqrt(lat);
        lng = asin(lng);
        indata->el1 = lng / dgrad;
        if (indata->el1 < horz) indata->el1 = -100.0;
       };
     };
   };

  if (indata->el1 != -100.0)  // get RA and Dec.
   {
    AppRADec (indata->mrefmjd, 0, indata->geolat*dgrad,
                           indata->geolong*dgrad, indata->az1*dgrad,
                           indata->el1*dgrad, lng, lat);
    // precession matrix epoch -> true of date
    if (indata->yrepoch != 0)
     {
      s = (indata->yrepoch - 2000.0) / 100.0;
      dt = julcent (indata->lastmjd);
      pmx = pmatequ (dt, s);
      r1[0] = 1.0;
      r1[1] = lng;
      r1[2] = lat;
      r1 = polcar(r1);
      r1 = mxvct (pmx, r1);
      r1 = carpol(r1);
      lng = r1[1];
      lat = r1[2];
     };

    // convert into HH.MM and DD.MM
    lng /= dgrad;
    lat /= dgrad;
    indata->az2 = DegFDms(lng/15.0);
    indata->el2 = DegFDms(lat);
   };
 }

void SatcalView::PltSky(QPainter &WorldMap)
 {
  // Draw the Sky 
  const double ae = 23454.77992; // 149597870.0/6378.14 =  1AE -> Earth Radii
  const int gx = 2709;
  const double dgrad = M_PI / 180.0;
  int j, k, xs, ys, dh;
  double lat, lng, rg, dt, s, horz;
  Sun200 sun;
  Moon200 moon;
  Plan200 pln;
  Vec3 r1, rm, rs;
  Mat3 pmx;
  QPoint p, p2;
  QColor tc;
  const int tz2 = 1;  // use these for Qt4
  const int tz3 = 2;
  const int tz5 = 4;
  const int tz7 = 6;

  double stard[gx] = {
		0.01530,	-6.0121,	4.41,
		0.08186,	+29.0456, 	2.06,
		0.09059,	+59.0829,	2.27,
		0.09201,	-45.4521,	3.88,
		0.13095,	+15.1031,	2.83,
		0.14339,	-18.5628,	4.44,
		0.19211,	-8.4956,	3.56,
		0.19596,	-64.5301,	4.23,
		0.25404,	-77.1546,	2.80,
		0.26126,	-42.1851,	2.39,
		0.26078,	-43.4117,	3.94,
		0.31287,	-62.5759,	4.37,
		0.32548,	+62.5525,	4.16,
		0.36480,	+33.4240,	4.36,
		0.36532,	+53.5319,	3.66,
		0.38286,	+29.1813,	4.37,
		0.39149,	+30.5110,	3.27,
		0.40253,	+56.3145,	2.23,
		0.43172,	-57.2817,	4.36,
		0.43309,	-17.5941,	2.04,
		0.47155,	+24.1532,	4.06,
		0.48363,	+7.3437,	4.43,
		0.49006,	+57.4829,	3.44,
		0.56370,	+60.4231,	2.47,
		0.56402,	+38.2928,	3.87,
		0.57076,	+23.2435,	4.42,
		0.58320,	-29.2156,	4.31,
		1.02519,	+7.5255,	4.28,
		1.06010,	-46.4336,	3.31,
		1.08193,	-55.1514,	3.92,
		1.08309,	-10.1125,	3.45,
		1.08310,	+86.1457,	4.25,
		1.09249,	+47.1402,	4.25,
		1.09389,	+35.3645,	2.06,
		1.11007,	+55.0831,	4.33,
		1.23569,	-8.1128,	3.60,
		1.25430,	+60.1339,	2.68,
		1.28180,	-43.1933,	3.41,
		1.31114,	-49.0450,	3.95,
		1.31242,	+15.2017,	3.62,
		1.36425,	+41.2353,	4.09,
		1.37395,	-57.1439,	0.46,
		1.37540,	+48.3714,	3.57,
		1.41212,	+5.2848,	4.44,
		1.43340,	+50.4052,	4.07,
		1.43599,	-15.5643,	3.50,
		1.45189,	+9.0901,	4.26,
		1.51232,	-10.2033,	3.73,
		1.52598,	+29.3418,	3.41,
		1.53352,	-46.1836,	4.41,
		1.54172,	+63.3946,	3.38,
		1.54334,	+20.4803,	2.64,
		1.55540,	-51.3659,	3.70,
		1.58434,	-61.3438,	2.86,
		1.59561,	-21.0506,	4.00,
		2.01581,	+2.4523,	3.79,
		2.03183,	+72.2451,	3.98,
		2.03484,	+42.1921,	2.26,
		2.07053,	+23.2719,	2.00,
		2.09272,	+34.5849,	3.00,
		2.12552,	+8.5023,	4.37,
		2.16274,	-51.3109,	3.56,
		2.17135,	+33.5025,	4.01,
		2.21433,	-68.3958,	4.09,
		2.26558,	-47.4238,	4.25,
		2.28047,	+8.2712,	4.28,
		2.30088,	+89.1527,	2.02,
		2.39243,	+0.1920,	4.07,
		2.39340,	-68.1624,	4.11,
		2.40365,	-39.5142,	4.11,
		2.43134,	+3.1346,	3.47,
		2.44031,	-13.5154,	4.25,
		2.44058,	+49.1320,	4.12,
		2.44517,	+10.0628,	4.27,
		2.45020,	-18.3444,	4.47,
		2.49017,	-32.2444,	4.46,
		2.49537,	+27.1516,	3.63,
		2.50293,	+38.1845,	4.23,
		2.50352,	+55.5322,	3.76,
		2.53499,	-49.5347,	4.00,
		2.54090,	+52.4523,	3.95,
		2.56213,	-8.54215,	3.89,
		2.58129,	-40.1838,	4.42,
		2.58123,	-40.1838,	3.42,
		3.02121,	+4.0502,	2.53,
		3.02195,	-23.3749,	4.09,
		3.04412,	+53.3003,	2.93,
		3.05048,	+38.5004,	3.39,
		3.08043,	+40.5700,	2.12,
		3.08575,	+49.3628,	4.05,
		3.09237,	+44.5106,	3.80,
		3.11326,	+19.4316,	4.35,
		3.12004,	-28.5934,	3.87,
		3.19270,	-21.4548,	3.69,
		3.19521,	-43.0432,	4.27,
		3.20149,	+29.0235,	4.47,
		3.24129,	+49.5122,	1.80,
		3.24440,	+9.0125,	3.60,
		3.27053,	+9.4339,	3.74,
		3.28568,	+59.5607,	4.21,
		3.30281,	+47.5925,	4.35,
		3.30474,	+12.5554,	4.11,
		3.32516,	-9.2748,	3.73,
		3.33433,	-21.3816,	4.26,
		3.36230,	+48.1116,	4.23,
		3.36478,	+0.2349,	4.28,
		3.42491,	+47.4658,	3.01,
		3.43106,	-9.4606,	3.54,
		3.44108,	-64.4842,	3.85,
		3.44135,	+32.1701,	3.83,
		3.44472,	+24.0631,	3.70,
		3.45055,	+42.3426,	3.77,
		3.45071,	+24.2745,	4.30,
		3.45442,	+24.2147,	3.88,
		3.46042,	-12.0623,	4.42,
		3.46142,	+23.5638,	4.18,
		3.46470,	-23.1515,	4.23,
		3.47156,	-74.1437,	3.24,
		3.49044,	+24.0256,	3.63,
		3.47237,	+24.0602,	2.87,
		3.49229,	+65.3118,	4.47,
		3.49239,	-36.1217,	4.17,
		3.54023,	+31.5245,	2.85,
		3.57452,	+40.0021,	2.89,
		3.57576,	-13.3046,	2.95,
		3.58520,	+35.4712,	4.04,
		4.00358,	+12.2910,	3.47,
		4.03046,	+5.5907,	3.91,
		4.04364,	+22.0440,	4.36,
		4.06283,	+50.2050,	4.29,
		4.08331,	+47.4231,	4.04,
		4.11475,	-6.5029,	4.04,
		4.13571,	-42.1753,	3.86,
		4.14243,	-62.2839,	3.35,
		4.14472,	+48.2420,	4.14,
		4.15122,	-7.3918,	4.43,
		4.15272,	+8.5319,	4.29,
		4.15592,	-51.2925,	4.25,
		4.16274,	-59.1820,	4.44,
		4.17503,	-33.4807,	3.56,
		4.19425,	+15.3727,	3.63,
		4.22509,	+17.3221,	3.76,
		4.23588,	-34.0113,	3.95,
		4.25168,	+22.1726,	4.22,
		4.25242,	+17.5529,	4.30,
		4.26131,	+22.4837,	4.29,
		4.26156,	+15.3654,	4.49,
		4.28293,	+15.5732,	3.85,
		4.28317,	+19.1038,	3.54,
		4.28346,	+15.5204,	3.42,
		4.33578,	-55.0253,	3.27,
		4.35295,	-30.3355,	3.82,
		4.35343,	+10.0928,	4.25,
		4.35501,	+16.3023,	0.85,
		4.36146,	-3.2120,	3.93,
		4.36351,	+41.1542,	4.25,
		4.38044,	+12.3029,	4.27,
		4.38067,	-14.1824,	3.87,
		4.40225,	-19.4028,	4.32,
		4.40308,	-41.5200,	4.45,
		4.42093,	+22.5715,	4.28,
		4.45256,	-3.1526,	4.02,
		4.49455,	+6.5731,	3.19,
		4.50318,	+8.5352,	4.36,
		4.51076,	+5.3609,	3.69,
		4.52492,	-5.2719,	4.39,
		4.53540,	+66.2025,	4.29,
		4.54104,	+2.2618,	3.72,
		4.56172,	+13.3044,	4.07,
		4.56537,	+33.0950,	2.69,
		4.57099,	+53.4459,	4.47,
		4.58282,	+1.4143,	4.47,
		5.01517,	+43.4916,	2.99,
		5.02224,	+41.0426,	3.75,
		5.03171,	+60.2625,	4.03,
		5.05239,	-22.2223,	3.19,
		5.06246,	+41.1357,	3.17,
		5.07465,	-5.0518,	2.79,
		5.09045,	-8.4521,	4.27,
		5.12137,	-11.5215,	4.45,
		5.12518,	-16.1226,	3.31,
		5.13097,	-12.5636,	4.36,
		5.13127,	+2.5134,	4.46,
		5.14279,	-8.1212,	0.12,
		5.16347,	+45.5948,	0.08,
		5.17320,	-6.5045,	3.60,
		5.19304,	-13.1042,	4.29,
		5.23525,	-7.4834,	4.14,
		5.24241,	-2.2354,	3.36,
		5.25030,	+6.2054,	1.64,
		5.26118,	+28.3623,	1.65,
		5.28109,	-20.4538,	2.84,
		5.30423,	+5.5649,	4.20,
		5.31096,	-35.2818,	3.87,
		5.31558,	-0.1801,	2.23,
		5.32075,	+18.3536,	4.38,
		5.32398,	-17.4924,	2.58,
		5.33367,	-62.2927,	3.40,
		5.34443,	+9.2919,	4.41,
		5.35033,	+9.5559,	3.66,
		5.35216,	-5.5439,	2.76,
		5.36082,	-1.1210,	1.70,
		5.36494,	+9.1724,	4.09,
		5.37333,	+21.0830,	3.00,
		5.38402,	-2.3603,	3.81,
		5.39357,	-34.0430,	2.64,
		5.40410,	-1.5637,	4.21,
		5.40410,	-1.5635,	2.05,
		5.44240,	-22.2656,	3.60,
		5.44462,	-65.4410,	4.35,
		5.46533,	-14.4921,	3.55,
		5.47150,	-51.0401,	3.85,
//		5.47411,	-9.4012,	2.06,
		5.47411,	-9.4012,	1.96,  // to make Orion look better
		5.50544,	-35.4608,	3.12,
		5.51154,	-20.5245,	3.81,
		5.51232,	+39.0853,	3.97,
		5.54176,	+20.1634,	4.41,
		5.55054,	+7.2425,	0.50,
		5.56202,	-14.1005,	3.71,
		5.57290,	-35.1700,	4.36,
		5.59061,	-42.4855,	3.96,
		5.59242,	+54.1705,	3.72,
		5.59251,	+44.5651,	1.90,
		5.59372,	+37.1245,	2.62,
		5.59494,	+45.5613,	4.26,
		6.02181,	+9.3851,	4.12,
		6.04017,	+23.1549,	4.16,
		6.07292,	+14.4607,	4.42,
		6.11513,	+14.1233,	4.48,
		6.14469,	-6.1627,	3.98,
		6.14472,	+22.3026,	3.28,
		6.15170,	+29.2955,	4.35,
		6.16299,	-35.0824,	4.37,
		6.19295,	+59.0042,	4.48,
		6.20153,	-30.0346,	3.02,
		6.22035,	-33.2608,	3.85,
		6.22380,	-17.5718,	1.98,
		6.22522,	+22.3052,	2.88,
		6.23413,	+4.3537,	4.44,
		6.23551,	-52.4141,	-0.72,
		6.28068,	-32.3445,	4.48,
		6.28524,	+20.1247,	4.15,
		6.31476,	-23.2502,	4.34,
		6.32494,	+7.2003,	4.50,
		6.34566,	-52.5827,	4.39,
		6.36371,	-19.1517,	3.95,
		6.37375,	+16.2402,	1.93,
		6.37429,	-43.1141,	3.17,
		6.37494,	-18.1410,	4.43,
		6.43504,	+25.0758,	2.98,
		6.43542,	+13.1346,	4.49,
		6.45049,	-16.4249,	-1.46,
		6.45123,	+12.5350,	3.36,
		6.47470,	+2.2450,	4.47,
		6.48105,	-61.5623,	3.27,
		6.49471,	-32.3024,	3.96,
		6.49494,	-53.3713,	4.40,
		6.49539,	-50.3646,	2.93,
		6.52414,	+33.5747,	3.60,
		6.54042,	-24.1055,	3.86,
		6.54072,	-12.0212,	4.07,
		6.56042,	-17.0307,	4.38,
		6.57088,	+58.2529,	4.35,
		7.01396,	-27.5557,	3.46,
		6.58340,	-28.5812,	1.50,
		7.02577,	-23.4952,	3.03,
		7.03414,	-15.3751,	4.11,
		7.04012,	+20.3421,	3.79,
		7.08198,	-26.2327,	1.86,
		7.08457,	-70.2948,	3.78,
		7.11027,	+30.1452,	4.41,
		7.11473,	-0.2925,	4.15,
		7.12311,	-46.4525,	4.49,
		7.14450,	-26.4612,	3.85,
		7.16499,	-67.5716,	3.98,
		7.17054,	-37.1705,	2.70,
		7.18004,	+16.3235,	3.58,
		7.18387,	-24.5705,	4.39,
		7.20020,	+21.5907,	3.53,
		7.24021,	-29.1800,	2.44,
		7.25380,	+27.4804,	3.79,
		7.28049,	+8.5544,	4.32,
		7.27042,	+8.1733,	2.90,
		7.29009,	+31.4715,	4.18,
		7.29110,	-43.1754,	3.25,
		7.33593,	-22.1734,	4.45,
		7.34304,	+31.5332,	2.85,
		7.34301,	+31.5330,	1.99,
		7.35498,	+26.5357,	4.06,
		7.38457,	-26.4754,	4.50,
		7.39133,	+5.1344,	0.38,
		7.41105,	-9.3251,	3.93,
		7.41504,	-72.3609,	3.95,
		7.43131,	+28.5314,	4.28,
		7.43448,	-28.5705,	3.96,
		7.44214,	+24.2406,	3.57,
		7.45121,	-37.5754,	3.59,
		7.45135,	+28.0148,	1.14,
		7.48014,	-25.5600,	4.50,
		7.49116,	-46.2210,	4.11,
		7.49139,	-24.5122,	3.34,
		7.52099,	-40.3419,	3.73,
		7.52355,	-38.5133,	4.49,
		7.53156,	-48.0557,	4.24,
		7.56444,	-52.5842,	3.47,
		7.56477,	-22.5234,	4.20,
		7.58118,	-49.1427,	4.41,
		8.02112,	+2.2019,	4.39,
		8.03319,	-39.5956,	2.25,
		8.07288,	-24.1800,	2.81,
		8.07556,	-68.3646,	4.35,
		8.08311,	-2.5846,	4.38,
		8.08576,	-19.1426,	4.40,
		8.09265,	-47.2028,	4.27,
		8.09292,	-47.1956,	1.82,
		8.11183,	-39.3651,	4.45,
		8.13597,	-40.2036,	4.44,
		8.16261,	+9.1125,	3.52,
		8.18290,	-36.3917,	4.45,
		8.18342,	-76.5454,	4.07,
		8.20413,	-77.2847,	4.35,
		8.22290,	-59.3017,	1.86,
		8.22440,	+43.1135,	4.25,
		8.25351,	-3.5405,	3.90,
		8.25432,	-66.0755,	3.77,
		8.30085,	+60.4324,	3.36,
		8.37346,	+5.4233,	4.16,
		8.37355,	-42.5902,	4.14,
		8.38407,	+3.2048,	4.44,
		8.40027,	-35.1810,	3.97,
		8.40150,	-52.5500,	3.62,
		8.40346,	-46.3836,	3.84,
		8.40351,	-59.4521,	4.33,
		8.43088,	+3.2415,	4.30,
		8.43319,	-33.1051,	3.68,
		8.44208,	-42.3838,	4.05,
		8.44360,	+18.0936,	3.94,
		8.44398,	-54.4210,	1.96,
		8.45587,	-46.0210,	3.91,
		8.46183,	-13.3232,	4.32,
		8.46364,	+28.4556,	4.02,
		8.46404,	-56.4551,	4.49,
		8.48212,	+5.5036,	4.36,
		8.46418,	+6.2528,	3.38,
		8.50281,	-27.4215,	4.01,
		8.55008,	-60.3820,	3.84,
		8.55189,	+5.5705,	3.11,
		8.58243,	+11.5149,	4.25,
		8.59063,	+48.0252,	3.14,
		9.00021,	-41.1452,	4.45,
		9.00326,	+41.4720,	3.97,
		9.02254,	-66.2324,	4.00,
		9.03314,	+47.0945,	3.60,
		9.04062,	-47.0530,	3.75,
		9.05091,	-72.3548,	4.48,
		9.07565,	-43.2535,	2.21,
		9.08460,	+51.3638,	4.48,
		9.10556,	-58.5739,	3.44,
		9.11147,	-62.1839,	3.97,
		9.13110,	-69.4240,	1.68,
		9.14172,	+2.1914,	3.88,
		9.16097,	-57.3206,	4.34,
		9.17030,	-59.1608,	2.25,
		9.18451,	+36.4833,	3.82,
		9.20578,	+34.2356,	3.13,
		9.22040,	-55.0015,	2.50,
		9.24340,	+26.1120,	4.46,
		9.27308,	-8.3908,	1.98,
		9.30384,	-40.2736,	3.60,
		9.31106,	-57.0140,	3.13,
		9.31247,	+63.0407,	3.67,
		9.31381,	+22.5829,	4.31,
		9.32124,	-62.4656,	4.00,
		9.32454,	+51.4103,	3.17,
		9.34241,	-59.1322,	4.08,
		9.34433,	+52.0330,	4.50,
		9.36465,	-49.2054,	4.35,
		9.36530,	+81.2000,	4.29,
		9.39468,	-1.0810,	3.91,
		9.41042,	+9.5357,	3.52,
		9.45123,	-62.3004,	3.40,
		9.45460,	+23.4652,	2.98,
		9.47039,	-65.0354,	3.15,
		9.47287,	+11.2609,	4.40,
		9.50530,	+59.0245,	3.80,
		9.51244,	-14.5022,	4.12,
		9.52407,	+26.0051,	3.88,
		9.56486,	-54.3338,	3.54,
		10.07151,+16.4612,	3.52,
		10.07205,+35.1508,	4.48,
		10.07495,+10.0018,	4.37,
		10.07517,-0.2151,	4.49,
		10.08175,+11.5828,	1.35,
		10.10309,-12.2048,	3.61,
		10.13421,-70.0150,	3.32,
		10.14404,-42.0652,	3.85,
		10.16364,+23.2529,	3.44,
		10.17004,+42.5519,	3.45,
		10.17020,-61.1929,	3.40,
		10.19537,+19.5053,	3.80,
		10.19534,+19.5057,	2.61,
		10.20515,-56.0208,	4.50,
		10.22144,+41.3026,	3.05,
		10.24219,-74.0126,	4.00,
		10.26011,-16.4943,	3.81,
		10.27050,-31.0337,	4.25,
		10.27478,+36.4254,	4.21,
		10.27494,-58.4354,	3.82,
		10.31582,-61.4039,	3.32,
		10.32439,+9.1852,	3.85,
		10.35271,-78.3600,	4.11,
		10.35318,-57.3259,	4.45,
		10.37143,-48.1305,	3.84,
		10.39148,-55.3544,	4.28,
		10.42541,-64.2312,	2.76,
		10.45461,-80.3156,	4.45,
		10.46423,-49.2444,	2.69,
		10.49331,-16.1109,	3.11,
		10.53137,+34.1323,	3.83,
		10.53260,-58.5043,	3.78,
		10.59421,-18.1727,	4.08,
		11.00051,-42.1304,	4.39,
		11.01451,+56.2326,	2.37,
		11.02150,+20.1116,	4.42,
		11.03382,+61.4532,	1.79,
		11.08315,-58.5801,	3.91,
		11.09348,+44.3024,	3.01,
		11.11351,-22.4904,	4.48,
		11.14017,+20.3155,	2.56,
		11.14097,+15.2616,	3.34,
		11.16351,-3.3836,	4.47,
		11.18062,+31.3216,	4.41,
		11.18239,+33.0609,	3.48,
		11.19160,-14.4614,	3.56,
		11.20563,-54.2858,	3.89,
		11.21036,+6.0215,	4.05,
		11.23508,+10.3215,	3.94,
		11.24484,-17.4033,	4.08,
		11.31190,+69.2022,	3.84,
		11.32557,-31.5058,	3.54,
		11.35427,-63.0041,	3.13,
		11.36523,-0.4856,	4.30,
		11.45321,-66.4314,	3.64,
		11.45469,+6.3216,	4.03,
		11.45583,+47.4716,	3.71,
		11.46264,-61.1012,	4.11,
		11.48590,+14.3450,	2.14,
		11.49367,-63.4649,	4.32,
		11.50370,+1.4623,	3.61,
		11.51042,-45.0955,	4.46,
		11.52500,-33.5359,	4.28,
		11.53451,+53.4211,	2.44,
		12.02570,-63.1816,	4.33,
		12.05080,+8.4429,	4.12,
		12.06482,-64.3619,	4.15,
		12.08005,-50.3911,	4.47,
		12.08168,-50.4251,	2.60,
		12.08201,-24.4314,	4.02,
		12.10028,-22.3641,	3.00,
		12.11344,-52.2137,	3.96,
		12.15039,-58.4426,	2.80,
		12.15211,+57.0227,	3.31,
		12.15437,-17.3201,	2.59,
		12.17293,-67.5709,	4.11,
		12.18153,-79.1814,	4.26,
		12.18213,-63.5941,	4.04,
		12.19498,-0.3931,	3.89,
		12.21167,-60.2335,	3.59,
		12.26315,-63.0528,	2.09,
		12.26308,-63.0527,	1.58,
		12.26518,+28.1636,	4.35,
		12.27575,-50.1321,	3.91,
		12.29472,-16.3026,	2.95,
		12.31049,-57.0617,	1.63,
		12.31596,-16.1116,	4.31,
		12.32225,-72.0729,	3.87,
		12.33252,+69.4747,	3.87,
		12.33403,+41.2156,	4.26,
		12.34185,-23.2319,	2.65,
		12.37055,-69.0738,	2.69,
		12.37372,-48.3159,	3.86,
		12.41260,-48.5705,	2.17,
		12.41350,-1.2628,	3.68,
		12.46114,-68.0600,	3.05,
		12.47379,-59.4050,	1.25,
		12.53017,-48.5606,	4.33,
		12.53212,-40.1015,	4.27,
		12.53578,+55.5805,	1.77,
		12.54303,-57.1011,	4.03,
		12.55317,+3.2420,	3.38,
		12.55575,+38.1935,	2.90,
		13.02061,+10.5802,	2.83,
		13.02099,-71.3227,	3.62,
		13.06494,-49.5354,	4.27,
		13.09523,-5.3152,	4.38,
		13.11482,+27.5309,	4.26,
		13.18504,-23.0949,	3.00,
		13.20307,-36.4216,	2.75,
		13.25099,+54.5954,	4.01,
		13.23510,+54.5559,	2.27,
		13.25068,-11.0913,	0.97,
		13.30574,-39.2359,	3.88,
		13.34370,-0.3518,	3.37,
		13.39475,-53.2732,	2.30,
		13.45361,-33.0210,	4.23,
		13.47115,+17.2751,	4.50,
		13.47289,+49.1915,	1.86,
		13.49215,-34.2636,	4.19,
		13.49243,+15.4819,	4.06,
		13.49249,-41.4049,	3.41,
		13.49315,-42.2759,	3.04,
		13.54368,+18.2419,	2.68,
		13.55267,-47.1652,	2.55,
		13.58108,-42.0537,	3.83,
		13.58352,-44.4747,	3.87,
		14.01342,+1.3306,	4.26,
		14.01378,-45.3547,	4.34,
		14.03430,-60.2157,	0.61,
		14.04209,+64.2259,	3.65,
		14.05573,-41.1021,	4.36,
		14.06172,-26.4031,	3.27,
		14.06356,-36.2146,	2.06,
		14.12489,-10.1601,	4.19,
		14.15356,+19.1125,	-0.04,
		14.15562,-5.5936,	4.08,
		14.16196,+46.0543,	4.18,
		14.19184,-46.0304,	3.55,
		14.20132,-56.2247,	4.33,
		14.20279,-37.5243,	4.05,
		14.22566,-39.3019,	4.42,
		14.25087,+51.5128,	4.05,
		14.26050,-45.2222,	4.35,
		14.26397,-83.3940,	4.32,
		14.27316,+75.4210,	4.25,
		14.31459,+30.2241,	3.58,
		14.32011,+38.1853,	3.03,
		14.32308,-50.2702,	4.42,
		14.34369,+29.4506,	4.46,
		14.35247,-42.0905,	2.31,
		14.37471,-49.2509,	4.05,
		14.39290,-60.4952,	1.33,
		14.39303,-60.4940,	-0.01,
		14.41046,+13.4405,	4.43,
		14.41498,-47.2255,	2.30,
		14.41520,-37.4714,	4.00,
		14.42231,-64.5807,	3.19,
		14.42589,-5.3906,	3.88,
		14.43339,-35.1002,	4.05,
		14.44553,+27.0450,	2.40,
		14.46104,+1.5357,	3.72,
		14.47401,-79.0219,	3.83,
		14.50120,-27.5715,	4.41,
		14.50425,+74.0942,	2.08,
		14.50477,-16.0208,	2.75,
		14.51325,-43.3409,	4.32,
		14.57063,-4.2026,	4.49,
		14.59038,-42.0554,	3.13,
		14.58260,-43.0741,	2.68,
		15.01534,+40.2347,	3.50,
		15.02495,+2.0549,	4.40,
		15.03589,-25.1634,	3.29,
		15.08445,-45.1627,	4.05,
		15.11498,-48.4356,	3.87,
		15.12106,-52.0537,	3.41,
		15.15265,+33.1913,	3.49,
		15.16556,-9.2239,	2.61,
		15.17237,-58.4745,	4.07,
		15.17443,-30.0836,	4.34,
		15.18257,-47.5211,	4.27,
		15.18460,-68.4027,	2.89,
		15.20438,+71.5022,	3.05,
		15.22164,-40.3832,	3.22,
		15.21426,-36.1522,	3.56,
		15.22347,-44.4103,	3.37,
		15.24260,+37.2257,	4.31,
		15.24538,+58.5817,	3.29,
		15.27460,+29.0639,	3.68,
		15.32522,+31.2151,	4.14,
		15.34375,+26.4311,	2.23,
		15.34439,+10.3239,	4.23,
		15.35024,-41.0943,	2.78,
		15.35265,-14.4705,	3.91,
		15.36349,-66.1844,	4.11,
		15.36560,-28.0749,	3.58,
		15.37572,-42.3345,	4.33,
		15.38339,-29.4623,	3.66,
		15.42408,+26.1801,	3.84,
		15.44066,+77.4757,	4.32,
		15.44117,+6.2549,	2.65,
		15.46071,+15.2535,	3.67,
		15.46222,+7.2128,	4.43,
		15.48403,+18.0846,	4.09,
		15.49325,-3.2533,	3.54,
		15.50445,+4.2856,	3.71,
		15.50518,-33.3722,	3.95,
		15.53444,-16.4330,	4.15,
		15.55006,-63.2534,	2.85,
		15.56230,+15.3959,	3.85,
		15.56476,-29.1235,	3.88,
		15.57315,+26.5256,	4.15,
		15.58457,-26.0636,	2.89,
		16.00013,-38.2334,	3.43,
		16.00147,-22.3703,	2.32,
		16.01516,+58.3409,	4.01,
		16.04172,-11.2209,	4.16,
		16.05210,-19.4805,	2.62,
		16.06296,-36.4754,	4.23,
		16.07190,-20.5153,	4.32,
		16.06432,-20.3955,	3.96,
		16.08433,+44.5620,	4.26,
		16.11545,-19.2724,	4.01,
		16.14160,-3.4126,	2.74,
		16.15180,-63.4055,	3.85,
		16.18145,-4.4120,	3.24,
		16.19417,+46.1901,	3.89,
		16.19437,-50.0907,	4.02,
		16.21058,-25.3522,	2.89,
		16.21512,+19.0924,	3.75,
		16.23582,+61.3103,	2.74,
		16.24009,-20.0202,	4.50,
		16.26562,-18.2711,	4.42,
		16.27045,-47.3306,	4.47,
		16.29189,-26.2544,	0.96,
		16.30093,+21.2934,	2.77,
		16.30503,+1.5914,	3.82,
		16.31032,-16.3634,	4.28,
		16.31170,-34.4204,	4.23,
		16.32028,-21.2748,	4.45,
		16.33130,-78.5338,	3.89,
		16.34033,+42.2624,	4.20,
		16.35474,-28.1247,	2.82,
		16.36166,-35.1510,	4.16,
		16.37046,-10.3351,	2.56,
		16.41138,+31.3620,	2.81,
		16.42507,+38.5530,	3.53,
		16.42518,-77.3052,	4.24,
		16.46071,+82.0224,	4.23,
		16.48303,-69.0131,	1.92,
		16.49393,-59.0220,	3.76,
		16.50040,-34.1726,	2.29,
		16.52140,-38.0054,	3.57,
		16.51461,-38.0242,	3.08,
		16.53562,+10.1004,	4.38,
		16.54287,-42.2132,	3.62,
		16.57358,+9.2238,	3.20,
		16.58297,-55.5916,	3.13,
		16.59279,-53.0930,	4.06,
		17.00139,+30.5543,	3.92,
		17.08469,+65.4259,	3.17,
		17.10175,-15.4323,	2.43,
		17.12028,-43.1414,	3.33,
		17.14348,+14.2331,	3.08,
		17.14582,+24.5027,	3.14,
		17.14597,+36.4839,	3.16,
		17.20446,-12.5043,	4.33,
		17.20548,-21.0641,	4.39,
		17.21551,-24.5953,	3.27,
		17.25161,-56.2235,	3.34,
		17.25105,-55.3143,	2.85,
		17.26167,-24.1027,	4.17,
		17.26264,+4.0830,	4.34,
		17.27155,-29.5157,	4.29,
		17.30239,+52.1809,	2.79,
		17.30397,-37.1741,	2.69,
		17.30407,+26.0642,	4.41,
		17.30578,-60.4058,	3.62,
		17.31435,-49.5230,	2.95,
		17.32416,+86.3515,	4.36,
		17.33304,-37.0610,	1.63,
		17.34519,+12.3340,	2.08,
		17.36266,-38.3804,	4.29,
		17.37127,-42.5949,	1.87,
		17.37300,-15.2352,	3.54,
		17.39254,+46.0026,	3.80,
		17.41198,-12.5229,	4.26,
		17.42230,-39.0146,	2.41,
		17.43239,+4.3404,	2.77,
		17.45351,-64.4324,	3.62,
		17.46240,+27.4317,	3.42,
		17.47280,-27.4949,	4.20,
		17.47288,-40.0736,	3.03,
		17.47491,+2.4228,	3.75,
		17.49454,-37.0235,	3.21,
		17.53302,+56.5222,	3.75,
		17.56121,+37.1502,	3.86,
		17.56343,+51.2921,	2.23,
		17.58267,+30.1122,	4.41,
		17.57424,+29.1453,	3.70,
		17.58566,-9.4625,	3.34,
		18.00342,+2.5554,	3.97,
		18.01406,+1.1819,	4.45,
		18.04555,-29.3449,	4.30,
		18.05228,+2.2959,	4.03,
		18.05427,-30.2527,	2.99,
		18.06309,-50.0530,	3.66,
		18.07167,+9.3349,	3.73,
		18.07290,+28.4544,	3.83,
		18.08262,-63.4007,	4.35,
		18.08417,+20.4851,	4.36,
		18.13404,-21.0334,	3.86,
		18.17316,-36.4544,	3.11,
		18.19486,+36.0350,	4.33,
		18.20467,+71.2014,	4.22,
		18.20539,-29.4944,	2.70,
		18.21050,+72.4356,	3.57,
		18.21139,-2.5357,	3.26,
		18.23053,-61.2941,	4.36,
		18.23381,+21.4608,	3.84,
		18.24044,-34.2308,	1.85,
		18.26518,-45.5810,	3.51,
		18.27527,-25.2522,	2.81,
		18.28430,-49.0419,	4.13,
		18.35075,-8.1443,	3.85,
		18.36533,+38.4656,	0.03,
		18.42516,-71.2547,	4.01,
		18.44433,+37.3612,	4.36,
		18.45338,-26.5933,	3.17,
		18.45359,+20.3241,	4.19,
		18.46573,+18.1047,	4.36,
		18.47057,-4.4458,	4.22,
		18.50015,+33.2139,	3.45,
		18.52047,-62.1122,	4.22,
		18.54271,+36.5349,	4.30,
		18.55104,-26.1755,	2.02,
		18.55174,+43.5639,	4.04,
		18.56479,-67.1408,	3.90,
		18.57384,-21.0631,	3.51,
		18.58533,+32.4115,	3.24,
		18.59333,+15.0358,	4.02,
		19.01360,-5.4428,	4.02,
		19.02310,-29.5257,	2.60,
		19.04356,-21.4438,	3.77,
		19.05205,+13.5140,	2.99,
		19.06102,-4.5306,	3.44,
		19.06508,-27.4022,	3.32,
		19.06222,-37.5425,	4.11,
		19.09405,-21.0134,	2.89,
		19.09555,-39.2035,	4.11,
		19.12333,+67.3932,	3.07,
		19.13425,+39.0836,	4.39,
		19.15348,+73.2110,	4.45,
		19.16190,+38.0752,	4.36,
		19.17041,+53.2156,	3.77,
		19.21352,-17.5100,	3.93,
		19.23067,-44.4810,	4.29,
		19.22318,-44.2743,	4.01,
		19.23470,-40.3708,	3.97,
		19.25254,+3.0642,	3.36,
		19.28386,+24.3942,	4.44,
		19.29401,+51.4336,	3.79,
		19.30397,+27.5723,	3.24,
		19.34010,+7.2232,	4.45,
		19.36241,+50.1303,	4.48,
		19.36386,-1.1724,	4.36,
		19.40018,+18.0037,	4.37,
		19.40589,+17.2821,	4.37,
		19.44557,+45.0737,	2.87,
		19.46113,+10.3634,	2.72,
		19.47193,+18.3150,	3.82,
		19.48107,+70.1551,	3.83,
		19.50304,+32.5437,	4.23,
		19.50426,+8.5151,	0.77,
		19.52238,+1.0006,	3.90,
		19.55095,-41.5221,	4.13,
		19.55144,+6.2411,	3.71,
		19.56150,+35.0446,	3.89,
		19.58414,+19.2917,	3.47,
		19.59383,-35.1650,	4.37,
		20.00252,-72.5453,	3.96,
		20.08348,-66.1110,	3.56,
		20.08565,+77.4225,	4.39,
		20.11137,-0.4934,	3.23,
		20.13218,+56.3347,	4.30,
		20.13351,+46.4412,	3.79,
		20.15255,+47.4235,	3.98,
		20.17339,-12.3046,	4.24,
		20.17583,-12.3258,	3.56,
		20.20556,-14.4710,	3.08,
		20.22105,+40.1507,	2.20,
		20.23480,+32.1107,	4.43,
		20.25318,-56.4424,	1.94,
		20.29201,+30.2149,	4.01,
		20.29334,+62.5921,	4.22,
		20.33085,+11.1753,	4.03,
		20.37278,-47.1749,	3.11,
		20.37287,+14.3524,	3.63,
		20.38156,-1.0638,	4.32,
		20.39341,+15.5424,	3.77,
		20.41228,+45.1630,	1.25,
		20.43233,+15.0409,	4.43,
		20.44495,-66.1231,	3.42,
		20.45156,+61.4959,	3.43,
		20.45360,+30.4251,	4.22,
		20.46004,-25.1635,	4.14,
		20.46090,+33.5753,	2.46,
		20.46353,+16.0708,	4.27,
		20.47357,-9.3005,	3.77,
		20.47395,-5.0200,	4.42,
		20.51439,-26.5529,	4.11,
		20.54416,-58.2736,	3.65,
		20.57071,+41.0941,	3.94,
		21.04526,+43.5519,	3.72,
		21.05518,-17.1420,	4.07,
		21.07024,-25.0043,	4.50,
		21.12524,+30.1315,	3.20,
		21.14245,+10.0003,	4.49,
		21.14439,+38.0221,	3.72,
		21.15449,+5.1430,	3.92,
		21.17214,+39.2318,	4.23,
		21.17514,+34.5326,	4.43,
		21.18326,+62.3445,	2.44,
		21.19456,-53.2721,	4.39,
		21.22010,+19.4753,	4.08,
		21.22098,-16.5028,	4.28,
		21.26193,-65.2223,	4.22,
		21.26349,-22.2504,	3.74,
		21.28385,+70.3315,	3.23,
		21.31288,-5.3440,	2.91,
		21.33555,+45.3507,	4.02,
		21.40005,-16.4009,	3.68,
		21.41188,-77.2349,	3.76,
		21.43277,+58.4623,	4.08,
		21.44067,+9.5205,	2.39,
		21.44265,+17.2035,	4.34,
		21.44346,+25.3817,	4.13,
		21.44515,-33.0158,	4.34,
		21.45243,+61.0650,	4.29,
		21.46443,+49.1809,	4.23,
		21.46575,-16.0803,	2.87,
		21.53503,-37.2219,	3.01,
		21.57490,-54.5959,	4.40,
		22.03448,+64.3714,	4.29,
		22.05424,-0.1938,	2.96,
		22.06015,-39.3302,	4.46,
		22.06214,-13.5237,	4.27,
		22.06565,+25.2016,	3.76,
		22.08084,-46.5806,	1.74,
		22.08178,-32.5945,	4.50,
		22.09552,+33.1015,	4.29,
		22.10075,+6.1126,	3.53,
		22.10482,+58.1138,	3.35,
		22.13489,+39.4227,	4.49,
		22.14587,+57.0210,	4.19,
		22.15543,+37.4429,	4.13,
		22.16453,-7.4727,	4.16,
		22.18240,-60.1602,	2.86,
		22.21347,-1.2342,	3.84,
		22.23301,+52.1317,	4.43,
		22.27138,-64.5827,	4.48,
		22.28454,-0.0140,	4.42,
		22.29069,+58.2427,	3.75,
		22.29108,-43.3012,	3.97,
		22.29282,+47.4157,	4.36,
		22.29401,-43.4525,	4.11,
		22.31138,+50.1629,	3.77,
		22.31253,-32.2114,	4.29,
		22.35168,-0.0731,	4.02,
		22.40269,+44.1607,	4.46,
		22.40344,-27.0305,	4.17,
		22.41232,+10.4925,	3.40,
		22.42347,-46.5333,	2.11,
		22.42559,+30.1248,	2.94,
		22.45546,-81.2322,	4.15,
		22.46275,+23.3328,	3.95,
		22.46371,+12.0954,	4.19,
		22.48279,-51.1929,	3.49,
		22.49308,-13.3602,	4.01,
		22.49376,+66.1133,	3.52,
		22.49559,+24.3537,	3.48,
		22.52266,-32.5301,	4.46,
		22.52322,-7.3515,	3.74,
		22.54343,-15.4944,	3.27,
		22.55520,-32.3252,	4.21,
		22.57341,-29.3749,	1.16,
		23.00475,-52.4544,	4.12,
		23.01511,+42.1904,	3.62,
		23.03421,+28.0429,	2.42,
		23.04412,+15.1150,	2.49,
		23.06361,-23.4505,	4.47,
		23.06478,-43.3143,	4.28,
		23.07510,+75.2246,	4.41,
		23.09220,-21.1050,	3.66,
		23.10165,-45.1518,	3.90,
		23.14147,-6.0326,	4.22,
		23.15488,-9.0545,	4.21,
		23.17053,+3.1627,	3.69,
		23.17206,-58.1328,	3.99,
		23.17495,-9.1126,	4.39,
		23.18446,-32.3225,	4.41,
		23.22535,-20.0631,	3.97,
		23.25183,+23.2345,	4.41,
		23.25581,-20.3901,	4.39,
		23.27535,+6.2215,	4.28,
		23.32535,-37.4936,	4.37,
		23.37294,+46.2700,	3.82,
		23.38038,+43.1535,	4.29,
		23.39171,+77.3727,	3.21,
		23.39524,+5.3705,	4.13,
		23.40201,+44.1932,	4.14,
		23.41582,+1.4618,	4.50,
		23.42387,-14.3311,	4.49,
		23.59141,+6.5118,	4.01,
		23.59504,-65.3508,	4.50
	  };

  dt = julcent (indata->lastmjd);
  lat = -0.015;  
  pmx = pmatequ (lat, dt);
  dt = indata->lastmjd;

  horz = 0.0;
  if (indata->satsky)
   {
    if (indata->geoheight > 0)
      horz = -acos(6378137.0 / (6378137.0 + indata->geoheight));
   };

  WorldMap.setBrush(Qt::SolidPattern);
  if (indata->mapbuf.emagrey == 2) WorldMap.setBrush(QColor(255, 255, 255));
  else WorldMap.setBrush(QColor(0,0,0));
   
  k = gx;
  for (j=0; j<k; j+=3)
   {
    lng = (360.0 - 15.0*DmsDegF(stard[j])) * dgrad;
    lat = DmsDegF(stard[j+1]) * dgrad;
    rs[0] = 1.0;
    rs[1] = -lng;
    rs[2] = lat;
    rs = polcar(rs);
    rs = mxvct(pmx, rs);
    AppPos (dt, 0, indata->geolat*dgrad, indata->geolong*dgrad,
                           indata->geoheight, 0, rs, lng, lat, rg);
    if (lat > horz)
     {
      lat /= dgrad;
      lng /= dgrad;
      p = PxSky(lat, lng);
      if (indata->mapbuf.emagrey == 2) tc = QColor( 255, 255, 255 );
      else tc = QColor( 0, 0, 0 );
      WorldMap.setPen(tc);

      // change both size and brightness to mark magnitude of star
      if (p.x() <= GBL_MAPWIDTH)
       {
        if (stard[j+2] <= 1.0)
         {
          WorldMap.setPen( tc );
          xs = p.x();
          ys = p.y();
          xs -= 1;
          ys -= 1;
          if (stard[j+2] <= 0.1) WorldMap.drawEllipse(xs, ys, tz3, tz3);
          else WorldMap.drawEllipse(xs, ys, tz2, tz2);
          WorldMap.setPen( QColor(255,255,255));
         }
        else
         {
          if (stard[j+2] > 2.0)
           {
            if (indata->mapbuf.emagrey == 2) tc = QColor( 200, 200, 200 );
            else tc = QColor( 50, 50, 50 );
           };
          if (stard[j+2] > 3.0)
           {
            if (indata->mapbuf.emagrey == 2) tc = QColor( 150, 150, 150 );
            else tc = QColor( 100, 100, 100 );
           };
          if (stard[j+2] > 4.0)
           {
            if (indata->mapbuf.emagrey == 2) tc = QColor( 100, 100, 100 );
            else tc = QColor( 150, 150, 150 );
           };
          WorldMap.setPen(tc); 
          WorldMap.drawPoint(p);
         };
       }; // p.x < mapwidth
     };
   };

  QFont f;
  f.setFamily("Helvetica");
  // mark horizon for zenith projection
  f.setPointSize(8);
  WorldMap.setFont(f);
  if (indata->skyprj == 0)
   {
    WorldMap.setPen( QColor( 200, 100, 50 ) );
    lat = 0;
    k = 0;
    p = PxSky(0, 0.0001);
    for (j=0; j<360; j++)
     {
      lng = double(j);
      p2 = PxSky(lat, lng);
      if ((p2.x() > indata->mapwidth)
               || (p2.y() > indata->mapheight)) k = 0;
      else
       {
        if (k) WorldMap.drawLine(p, p2);
        k = 1;
        p = p2;
       };
     };
   };  // skyprj

  if (indata->mapbuf.emagrey != 2) WorldMap.setPen( QColor(0, 0, 0 ));
  else WorldMap.setPen( QColor(255, 255, 255 ));

  // directions
  if (indata->skyprj) lat = 0.5;
  else lat = 10.0;
  lng = 0;
  p = PxSky(lat, lng);
  xs = p.x();
  ys = p.y();
  if (indata->skyprj == 0) ys += 10;
  WorldMap.drawText(xs, ys, "N");
  if (indata->skyprj == 0) lat = 13.0;
  lng = 180.0;
  p = PxSky(lat, lng);
  xs = p.x();
  ys = p.y();
  if (indata->skyprj == 0) ys -= 14;
  WorldMap.drawText(p, "S");
  if (indata->skyprj == 0) lat = 0;
  lng = 90.0;
  p = PxSky(lat, lng);
  xs = p.x();
  ys = p.y();
  if (indata->skyprj == 0) xs -= 8;
  WorldMap.drawText(xs, ys, "E");
  lng = 270.0;
  p = PxSky(lat, lng);
  WorldMap.drawText(p, "W");
  dh = 6;

  // Sun and Moon
  dt = indata->lastmjd + SetTtUt(indata, indata->year)/86400.0;
  dt = julcent (dt);
  rs = sun.position(dt);
  rm = eclequ(dt,rs);
  rm = rm * ae;
  AppPos (indata->lastmjd, 0, indata->geolat*dgrad, indata->geolong*dgrad,
                                     indata->geoheight, 1, rm, lng, lat, s);
  lat /= dgrad;
  lng /= dgrad;
  p = PxSky(lat, lng);
  if (p.x() <= GBL_MAPWIDTH)
   {
    xs = p.x();
    ys = p.y(); 
    xs -= 3;
    ys -= 3;
    WorldMap.setBrush(QColor(255, 255, 0));
    WorldMap.setPen( QColor( 255, 255, 0 ));
    WorldMap.drawEllipse(xs, ys, tz7, tz7);
    xs += 2;
    ys += 2;
    WorldMap.setBrush(QColor(255, 0, 0));
    WorldMap.setPen( QColor( 255, 0, 0 ) );
    WorldMap.drawEllipse(xs, ys, tz3, tz3);
    if (indata->skyprj == 0)
     {
      xs = 10;
      ys = 10;
      WorldMap.setBrush( QColor( 255, 255, 0 ) );
      WorldMap.setPen( QColor( 255, 255, 0 ) );
      WorldMap.drawEllipse(xs, ys, tz7, tz7);
      xs = 20;
      ys += dh;
      WorldMap.drawText(xs, ys, "Sun");
      xs = 12;
      ys = 12;
      WorldMap.setBrush( QColor( 255, 0, 0 ) );
      WorldMap.setPen( QColor( 255, 0, 0 ) );
      WorldMap.drawEllipse(xs, ys, tz3, tz3);
     };
   };  // sun

  rm = moon.position(dt);
  rm = eclequ(dt,rm);
  AppPos (indata->lastmjd, 0, indata->geolat*dgrad, indata->geolong*dgrad,
                                      indata->geoheight, 1, rm, lng, lat, s);
  lat /= dgrad;
  lng /= dgrad;
  p = PxSky(lat, lng);
  if (p.x() <= GBL_MAPWIDTH)
   {
    xs = p.x();
    ys = p.y(); 
    xs -= 3;
    ys -= 3;
    if (indata->mapbuf.emagrey == 2)
     {
      WorldMap.setBrush( QColor( 255, 255, 255 ) );
      WorldMap.setPen( QColor( 255, 255, 255 ) );
     }
    else
     {
      WorldMap.setBrush( QColor( 0, 0, 0 ) );
      WorldMap.setPen( QColor( 0, 0, 0 ) );
     };
    WorldMap.drawEllipse(xs, ys, tz7, tz7);
    if (indata->skyprj == 0)
     {
      xs = 10;
      ys = 25;
      WorldMap.drawEllipse(xs, ys, tz7, tz7);
      xs = 20;
      ys += dh;
      WorldMap.drawText(xs, ys, "Moon");
     };
   };  // moon

  // Planets
  rm = pln.Mercury(dt);
  rm = rs + rm;
  rm = eclequ(dt,rm);
  rm = rm * ae;
  AppPos (indata->lastmjd, 0, indata->geolat*dgrad, indata->geolong*dgrad,
                                          indata->geoheight, 1, rm, lng, lat, s);
  lat /= dgrad;
  lng /= dgrad;
  p = PxSky(lat, lng);
  if (p.x() <= GBL_MAPWIDTH)
   {
    xs = p.x();
    ys = p.y(); 
    xs -= 2;
    ys -= 2;
    WorldMap.setBrush( QColor( 200, 100, 50 ) );
    WorldMap.setPen( QColor( 200, 100, 50 ) );
    WorldMap.drawEllipse(xs, ys, tz5, tz5);
    if (indata->skyprj == 0)
     {
      xs = 11;
      ys = 40;
      WorldMap.drawEllipse(xs, ys, tz5, tz5);
      xs = 20;
      ys += dh;
      WorldMap.drawText(xs, ys, "Mercury");
     };
   };

  rm = pln.Venus(dt);
  rm = rs + rm;
  rm = eclequ(dt,rm);
  rm = rm * ae;
  AppPos (indata->lastmjd, 0, indata->geolat*dgrad, indata->geolong*dgrad,
                                      indata->geoheight, 1, rm, lng, lat, s);
  lat /= dgrad;
  lng /= dgrad;
  p = PxSky(lat, lng);
  if (p.x() <= GBL_MAPWIDTH)
   {
    xs = p.x();
    ys = p.y(); 
    xs -= 2;
    ys -= 2;
    WorldMap.setBrush( QColor( 0, 0, 255 ) );
    WorldMap.setPen( QColor( 0, 0, 255 ) );
    WorldMap.drawEllipse(xs, ys, tz5, tz5);
    if (indata->skyprj == 0)
     {
      xs = 11;
      ys = 55;
      WorldMap.drawEllipse(xs, ys, tz5, tz5);
      xs = 20;
      ys += dh;
      WorldMap.drawText(xs, ys, "Venus");
     };
   };

  rm = pln.Mars(dt);
  rm = rs + rm;
  rm = eclequ(dt,rm);
  rm = rm * ae;
  AppPos (indata->lastmjd, 0, indata->geolat*dgrad, indata->geolong*dgrad,
                                       indata->geoheight, 1, rm, lng, lat, s);
  lat /= dgrad;
  lng /= dgrad;
  p = PxSky(lat, lng);
  if (p.x() <= GBL_MAPWIDTH)
   {
    xs = p.x();
    ys = p.y(); 
    xs -= 2;
    ys -= 2;
    WorldMap.setBrush( QColor( 255, 0, 0 ) );
    WorldMap.setPen( QColor( 255, 0, 0 ) );
    WorldMap.drawEllipse(xs, ys, tz5, tz5);
    if (indata->skyprj == 0)
     {
      xs = 11;
      ys = 70;
      WorldMap.drawEllipse(xs, ys, tz5, tz5);
      xs = 20;
      ys += dh;
      WorldMap.drawText(xs, ys, "Mars");
     };
   };

  rm = pln.Jupiter(dt);
  rm = rs + rm;
  rm = eclequ(dt,rm);
  rm = rm * ae;
  AppPos (indata->lastmjd, 0, indata->geolat*dgrad, indata->geolong*dgrad,
                                        indata->geoheight, 1, rm, lng, lat, s);
  lat /= dgrad;
  lng /= dgrad;
  p = PxSky(lat, lng);
  if (p.x() <= GBL_MAPWIDTH)
   {
    xs = p.x();
    ys = p.y(); 
    xs -= 2;
    ys -= 2;
    WorldMap.setBrush( QColor( 0, 255, 0 ) );
    WorldMap.setPen( QColor( 0, 255, 0 ) );
    WorldMap.drawEllipse(xs, ys, tz5, tz5);
    if (indata->skyprj == 0)
     {
      xs = 11;
      ys = 85;
      WorldMap.drawEllipse(xs, ys, tz5, tz5);
      xs = 20;
      ys += dh;
      WorldMap.drawText(xs, ys, "Jupiter");
     };
   };

  rm = pln.Saturn(dt);
  rm = rs + rm;
  rm = eclequ(dt,rm);
  rm = rm * ae;
  AppPos (indata->lastmjd, 0, indata->geolat*dgrad, indata->geolong*dgrad,
                                         indata->geoheight, 1, rm, lng, lat, s);
  lat /= dgrad;
  lng /= dgrad;
  p = PxSky(lat, lng);
  if (p.x() <= GBL_MAPWIDTH)
   {
    xs = p.x();
    ys = p.y(); 
    xs -= 2;
    ys -= 2;
    WorldMap.setBrush( QColor( 255, 255, 0 ) );
    WorldMap.setPen( QColor( 255, 255, 0 ) );
    WorldMap.drawEllipse(xs, ys, tz5, tz5);
    if (indata->skyprj == 0)
     {
      xs = 11;
      ys = 100;
      WorldMap.drawEllipse(xs, ys, tz5, tz5);
      xs = 20;
      ys += dh;
      WorldMap.drawText(xs, ys, "Saturn");
     };
   };

  WorldMap.setBrush(Qt::NoBrush);
  WorldMap.setPen(QColor( 130, 160, 180 ));
  xs = GBL_MAPWIDTH - 1;
  ys = GBL_MAPHEIGHT;
  WorldMap.drawLine(xs,0,xs,ys);  // right border of map
 }

 
// ********  special interface routines for dialogs  **************

void QsToChar(const QString &qname, char* cname, int len)
{
 // copy QString to Char* with maximum length len
  QByteArray ba;
  int k, j;

  ba = qname.toAscii();
  k = ba.size()+1;

  char *ga =new char[k];

  strcpy (ga, ba.data());
  k = strlen(ga);
  cname[0] = '\0';
  if (k > 0)
   {
    if (k > len) k = len;
    for (j=0; j<k; j++)
     {
      cname[j] = ga[j];
      if (cname[j] == '\r') cname[j] = '\0';
     };
    cname[k] = '\0';
   };
  delete[] ga;
}

void vsLastSat(QString &s, int &nsw, int &nrd, int &nsat, int &lwo, int &rlo)
 {
  s = indata->orbelm.sname[GBL_MXSAT];
  nsw = indata->orbelm.nrflg;
  lwo = indata->orbelm.lflg;
  rlo = indata->orbelm.nflg;
  nrd = indata->orbelm.norad[GBL_MXSAT];
  nsat = indata->sidxcn;
 }  

QString vsSatList()  // list of currently selected satellites
 {
  QString s1, s2;
  int j;

  s1 = "TLE filename: ";
  s2 = indata->tlename;
  s1 += s2;
  if (indata->sidxcn > 0)
   {
    s2 = "\n\nSelected Satellites:\n";
    s1 += s2;
    for (j=0; j<indata->sidxcn; j++)
     {
      s2 = "\n  ";
      s1 += s2;
      s2 = indata->orbelm.fsname[j];
      s1 += s2;
     }; 
   }
  else
   {
    s2 = "\n\n*** No Satellites Selected ***";
    s1 += s2;
   };        

  return s1;
 }

void vsResetSatList()
 {
  indata->sidxcn = 0;
  indata->satidx = 0;
  indata->lastsx = 9999;  // reset sx marker for SGP4 initialization
 }

void vsSatSel()
 {
  if (indata->sidxcn >= GBL_MXSAT) indata->sidxcn = GBL_MXSAT-1;

  ShiftElm (indata, indata->sidxcn, GBL_MXSAT);
  indata->sidxcn++;
  if (indata->sidxcn > GBL_MXSAT) indata->sidxcn = GBL_MXSAT;

  indata->orbelm.fsname[GBL_MXSAT][0] = '#';
 }

void vsFindSat(QString &s, int &nrd, int nrw, int lwo, int rlo)
 {
  fseek (satfle, 0L, SEEK_SET);
  vsReComp(s, nrd, nrw, 0, lwo, rlo);
 }

void vsReComp(QString &s, int &nrd, int nrw, int morflg, int lwo, int rlo)
{
  char otxt[541];

  indata->orbelm.nrflg =  nrw;
  indata->orbelm.lflg =  lwo;
  indata->orbelm.nflg =  rlo;
  QsToChar(s, indata->orbelm.sname[GBL_MXSAT], 20);
  indata->orbelm.norad[GBL_MXSAT] = nrd;

  nrd = tleread(indata, satfle, otxt, morflg);

  s = otxt;
}

void vsLowRecent(int lwo, int rlo)
{
  indata->orbelm.lflg =  lwo;
  indata->orbelm.nflg =  rlo;
}

QString vsSatName(int j)  // get name of j-th satellite
{
 QString s;

 s = indata->orbelm.fsname[j-1];

 return s;
}

void vsFileName(char *fname)  // complete file name
{
 char dname[105];
 int i, j, k, len;

 if (fname[0] == '*')  // need to append TLE directory
  {
   for (j=0; j<105; j++) dname[j] = '\0'; // to avoid strange surprises
   strcpy(dname, indata->tlename);
   len = strlen(dname);
   if (len > 0)
    {
     k= -1;
     for(j=0; j<len; j++)
      {
       if (dname[j] ==  '/') k = j;
      };
     if (k < 0) fname[0] = '/';
     else
      {
       k++;
       if (k > 103) k = 0;  // just to be safe
       i = 1;
       for (j=k; j<104; j++)
        {
         dname[j] = fname[i];
         i++;
        };
       strcpy(fname, dname);
      };
    }
   else fname[0] = '/';
  };
}

int vsGetPassDTxt(double &t, QString &s, bool pole)  // detailed pass text
{
  int mm;
  char otxt[335];

  mm = 0;
  if (pole) GetPoleTxt(indata, otxt);
  else mm = GetPassDTxt(indata, t, otxt, 0);

  s = otxt;

  return mm;
}

void vsSavePassDt(int flsav)  // write detailed pass text to save file
{
  char otxt[335];
  int mm, clnt;
  double t;
  QString s1, s2;
  FILE *svfptr;

  if (indata->svfirst)
   {
    vsFileName (indata->svfname);
    svfptr = fopen (indata->svfname,"w");
   } 
  else svfptr = fopen (indata->svfname,"a");

  if (svfptr != NULL)
   {
    indata->svfirst = 0;
  
    t = 0;
    clnt = 0;
    mm = GetPassDTxt(indata, t, otxt, flsav);
    if (mm) fprintf(svfptr, otxt);
    while (mm)
     {
      clnt++;                    // limit number of lines
      if (clnt < 16250) mm = GetPassDTxt(indata, t, otxt, flsav);
      else mm = 0;
      if (mm) fprintf(svfptr, otxt);
     }; // end while

    if ((clnt > 1) && (flsav < 2))
     {
      GetPoleTxt(indata, otxt);
      fprintf(svfptr, otxt);
     };
      
    fclose(svfptr); 

    s1 = "Saved on File ";
    s2 = indata->svfname;
    s1 += s2;
    QMessageBox::information(0, "SatCal", s1);
   };

  return;
}

void vsGetPict(double &ra1, double &dec1, double &ra2, double &dec2)
{
 ra1 = indata->rstarra[3];
 dec1 = indata->rstardec[3];
 ra2 = indata->rstarra[4];
 dec2 = indata->rstardec[4];

 return;
}
  
void vsGetAdj(bool &usesdp4, bool &adjflg, double &adjtim, double &adjn0, double &adjra, int &wgs)
{
 usesdp4 = indata->usesdp4;
 adjflg = indata->adjflg;
 adjtim = indata->adjtim;
 adjn0 = indata->adjn0;
 adjra = indata->adjra;
 wgs = indata->wgs84;
}
  
void vsPutAdj(bool usesdp4, bool adjflg, double adjtim, double adjn0, double adjra, int wgs)
{
 indata->usesdp4 = usesdp4;
 indata->adjflg = adjflg;
 indata->adjtim = adjtim;
 indata->adjn0 = adjn0;
 indata->adjra = adjra;
 indata->wgs84 = wgs;
 indata->lastsx = 9999;  // reset sx marker for SGP4 initialization
}

void RKFun(double t, double y[], double z[])
// function for numerical integration
// placed here to provide the indata parameter set
 {
  RKfun (t, y, &z[0], indata);
 }

void vsCkHyp(double& ecc, double& t0)
// check for hyperbolic numeric orbit
{
 int sx;
 double gm, t, m0, a, ran, aper, inc;
 Vec3 rr1, vv1;

 sx = indata->satidx;

 rr1[0] = indata->orbelm.x0[sx]/1000.0;
 rr1[1] = indata->orbelm.y0[sx]/1000.0;
 rr1[2] = indata->orbelm.z0[sx]/1000.0;
 vv1[0] = indata->orbelm.vx0[sx]/1000.0;
 vv1[1] = indata->orbelm.vy0[sx]/1000.0;
 vv1[2] = indata->orbelm.vz0[sx]/1000.0;
 t = indata->orbelm.mjdep[sx];

 gm = 2.97553635177984e15;
 vv1 *= 86400.0;
 oscelm (gm, t, rr1, vv1, t0, m0, a, ecc, ran, aper, inc);
}

void cvSGP4(double& xincl, double& xnodeo,
	double& eo, double& omegao, double& xmo, double& xno )
// convert state vector into SGP4 mean elements 
{
 int sx;
 Vec3 rr1, vv1;

 sx = indata->satidx;

 rr1[0] = indata->orbelm.x0[sx];
 rr1[1] = indata->orbelm.y0[sx];
 rr1[2] = indata->orbelm.z0[sx];
 vv1[0] = indata->orbelm.vx0[sx];
 vv1[1] = indata->orbelm.vy0[sx];
 vv1[2] = indata->orbelm.vz0[sx];
 rv2el(rr1, vv1, xincl, xnodeo, eo, omegao, xmo, xno);
 indata->lastsx = 9999;  // reset sx marker for SGP4 initialization

 return;  
}

void vsRefState(int tmjeold, int tmjenew, double t,
            double& r0, double& r1, double& r2, double& v0, double& v1, double& v2)
/* convert state vector from old reference system into a new one.
  tmjeold, tmjenew: 0 True of Date, 1 Mean of Date, 2 J2000, 3 Earth fixed.
  t = MJD (UTC) of epoch
  r, v in km and km/sec.
 */
{
 const double omr = 7.2921158553e-5;

 double tut1, d, ep2;
 Vec3 rr1, vv1, vr;
 Mat3 pmx, nx, plx, efx, ert, ivx;

 rr1[0] = r0*1000.0;      
 rr1[1] = r1*1000.0;
 rr1[2] = r2*1000.0;
 vv1[0] = v0*1000.0;
 vv1[1] = v1*1000.0;
 vv1[2] = v2*1000.0;
   
 tut1 = t;
 if (indata->IERSflg) tut1 += indata->Ut1Utc/86400.0;
 nx = nutmat(julcent(t), ep2, indata->IERSflg); // Nutation
 d = lsidtim (tut1, 0.0, ep2)* M_PI/12.0;
 ert = zrot(d);   // Earth rotation

 efx = mxcon(0.0);
 efx.PutMij(omr, 1, 2);
 efx.PutMij(-omr, 2, 1);
 efx = efx * zrot(d); // derivative of Earth rotation
 if ((indata->IERSxp != 0.0) || (indata->IERSyp != 0.0))
  {
   plx = PoleMx(indata->IERSxp, indata->IERSyp);
   efx = plx * efx;
 //  ert = plx *ert; // had to bypass this due to an error
  }
 else plx = mxidn() ;

 pmx = pmatequ(0.0, julcent(t));  // precession

 // convert into True of Date
 if (tmjeold != 0)
  {
   if (tmjeold == 2)
    {
     rr1 = mxvct(pmx, rr1);
     vv1 = mxvct(pmx, vv1); 
    };
   if ((tmjeold == 1) || (tmjeold == 2))
    {
     rr1 = mxvct(nx, rr1);
     vv1 = mxvct(nx, vv1);
    };
   if (tmjeold == 3)
    {
     ivx = mxtrn(efx);
     vr = mxvct(ivx,rr1);
     ivx = mxtrn(plx);
     rr1 = mxvct(ivx, rr1);
     vv1 = mxvct(ivx, vv1);
     ivx = mxtrn(ert); 
     rr1 = mxvct(ivx, rr1);
     vv1 = mxvct(ivx, vv1);
     vv1 = vv1 + vr;
    };
  };

 // convert from True of Date
 if (tmjenew != 0)
  {
   if ((tmjenew == 1) || (tmjenew == 2))
    {
     nx = mxtrn(nx); 
     rr1 = mxvct(nx, rr1);
     vv1 = mxvct(nx, vv1);
    };  
   if (tmjenew == 2)
    {
     pmx = mxtrn(pmx); 
     rr1 = mxvct(pmx, rr1);
     vv1 = mxvct(pmx, vv1);      
    };
   if (tmjenew == 3)
    {
     vr = mxvct(efx,rr1);
     rr1 = mxvct(ert, rr1);
     vv1 = mxvct(ert, vv1);
     rr1 = mxvct(plx, rr1);
     vv1 = mxvct(plx, vv1);
     vv1 = vv1 + vr;
    };
  };  
  
 r0 = rr1[0]/1000.0;
 r1 = rr1[1]/1000.0;
 r2 = rr1[2]/1000.0;
 v0 = vv1[0]/1000.0;
 v1 = vv1[1]/1000.0;
 v2 = vv1[2]/1000.0;
}
            
void vsGetState(bool nmflg, double& f107, double& fb107, double& kp, int& tmje,
            double& pcr, double& pcd, double &pam, double &r0, double& r1, double& r2,
            double& v0, double& v1, double& v2, double& mjep)
// get the state vector parameters for the State Dialog
{
 int sx;

 sx = indata->satidx;
 tmje = indata->tmje;

 if (nmflg)
  {
   r0 = indata->orbelm.x0[sx]/1000.0;
   r1 = indata->orbelm.y0[sx]/1000.0;
   r2 = indata->orbelm.z0[sx]/1000.0;
   v0 = indata->orbelm.vx0[sx]/1000.0;
   v1 = indata->orbelm.vy0[sx]/1000.0;
   v2 = indata->orbelm.vz0[sx]/1000.0;
   mjep = indata->orbelm.mjdep[sx];
  }; 

 f107 = indata->orbelm.F107[sx];
 fb107 = indata->orbelm.FB107[sx];
 kp = indata->orbelm.Kp[sx];
 pcd = indata->orbelm.Cd[sx];
 pcr = indata->orbelm.Cr[sx];
 pam = indata->orbelm.Am[sx];
}
            
void vsPutState(bool nmflg, int year, double day, double f107, double fb107, double kp,
            int tmje, double pcr, double pcd, double pam, double r0, double r1, double r2,
            double v0, double v1, double v2, bool cstate)
// store the state vector parameters from the State Dialog
{
  int sx;
  
  sx = indata->satidx;
  indata->tmje = tmje;
  indata->orbelm.mjdep[sx] = mjd(0,1,year,0) + day;

  indata->orbelm.x0[sx] = r0*1000.0;
  indata->orbelm.y0[sx] = r1*1000.0;
  indata->orbelm.z0[sx] = r2*1000.0;
  indata->orbelm.vx0[sx] = v0*1000.0;
  indata->orbelm.vy0[sx] = v1*1000.0;
  indata->orbelm.vz0[sx] = v2*1000.0;
  if (cstate)
   {
    indata->orbelm.x[sx] = indata->orbelm.x0[sx];
    indata->orbelm.y[sx] = indata->orbelm.y0[sx];
    indata->orbelm.z[sx] = indata->orbelm.z0[sx];
    indata->orbelm.vx[sx] = indata->orbelm.vx0[sx];
    indata->orbelm.vy[sx] = indata->orbelm.vy0[sx];
    indata->orbelm.vz[sx] = indata->orbelm.vz0[sx];
    indata->orbelm.mjdcur[sx] = indata->orbelm.mjdep[sx];
   }; 

  if (nmflg) 
   {
    indata->orbelm.F107[sx] = f107;
    indata->orbelm.FB107[sx] = fb107;
    indata->orbelm.Kp[sx] = kp;
    indata->orbelm.Cd[sx] = pcd;
    indata->orbelm.Cr[sx] = pcr;
    indata->orbelm.Am[sx] = pam;
   };
}

